@@ -114,6 +114,27 @@ public:
   typedef QMap<QString, QString> MetaDataMap;
   enum MimeDataFlags { DefaultMimeDataFlags = 0, NoTextExport = 1 };
 
+  /**
+   * Options to be used in adjustPath
+   */
+  enum AdjustPathOption
+  {
+    /**
+     * strips a trailing '/', except when the path is already just "/".
+     */
+    RemoveTrailingSlash,
+
+    /**
+     * Do not change the path.
+     */
+    LeaveTrailingSlash,
+
+    /**
+     * adds a trailing '/' if there is none yet
+     */
+    AddTrailingSlash
+  };
+
   /**
    * \class List kurl.h <KUrl>
    *
@@ -146,18 +167,41 @@ public:
        * @param list the list containing the URLs
        */
       List(const QList<KUrl> &list);
+      /**
+       * Creates a list that contains the URLs from the given QList<KUrl>.
+       * @param list the list containing the URLs
+       * @since 4.7
+       */
+      List(const QList<QUrl> &list);
       /**
        * Converts the URLs of this list to a list of strings.
        * @return the list of strings
        */
       QStringList toStringList() const;
 
+      /**
+       * Converts the URLs of this list to a list of strings.
+       *
+       * @param trailing use to add or remove a trailing slash to/from the path.
+       *
+       * @return the list of strings
+       *
+       * @since 4.6
+       */
+      QStringList toStringList(KUrl::AdjustPathOption trailing) const;
+
       /**
        * Converts this KUrl::List to a QVariant, this allows to use KUrl::List
        * in QVariant() constructor
        */
       operator QVariant() const;
 
+      /**
+       * Converts this KUrl::List into a list of QUrl instances.
+       * @since 4.7
+       */
+      operator QList<QUrl>() const;
+
       /**
        * Adds URLs data into the given QMimeData.
        *
@@ -388,27 +432,6 @@ public:
    **/
   bool hasHost() const;
 
-  /**
-   * Options to be used in adjustPath
-   */
-  enum AdjustPathOption
-  {
-    /**
-     * strips a trailing '/', except when the path is already just "/".
-     */
-    RemoveTrailingSlash,
-
-    /**
-     * Do not change the path.
-     */
-    LeaveTrailingSlash,
-
-    /**
-     * adds a trailing '/' if there is none yet
-     */
-    AddTrailingSlash
-  };
-
   /**
    * @param trailing use to add or remove a trailing slash to/from the path. see adjustPath
 
@@ -596,8 +619,10 @@ public:
 
   /**
    * Checks whether the file is local.
-   * @return true if the file is a plain local file and has no filter protocols
-   *         attached to it.
+   * @return true if the file is a plain local file (i.e. uses the file protocol
+   *   and no hostname, or the local hostname).
+   *  When isLocalFile returns true, you can use toLocalFile to read the file contents.
+   *  Otherwise you need to use KIO (e.g. KIO::get).
    */
   bool isLocalFile() const;
 
@@ -867,7 +892,9 @@ public:
    * ignore trailing '/' characters.
    * @deprecated Use equals() instead.
    */
+#ifndef KDE_NO_DEPRECATED
   KDE_DEPRECATED bool cmp( const KUrl &u, bool ignore_trailing = false ) const;
+#endif
 
 
   /**
@@ -969,8 +996,9 @@ public:
 
   /**
    * Creates a KUrl object from a QString representing an absolute path.
-   * KUrl url( somePath ) does the same, but this method is more explicit
-   * and avoids the path-or-url detection in the KUrl constructor.
+   * KUrl url( somePath ) is almost the same, but this method is more explicit,
+   * avoids the path-or-url detection in the KUrl constructor, and parses
+   * "abc:def" as a filename, not as URL.
    *
    * @param text the path
    * @return the new KUrl
@@ -982,7 +1010,9 @@ public:
    * Since KDE4 you can pass both urls and paths to the KUrl constructors.
    * Use KUrl(text) instead.
    */
+#ifndef KDE_NO_DEPRECATED
   static KDE_DEPRECATED KUrl fromPathOrUrl( const QString& text );
+#endif
 
   /**
    * Creates a KUrl from a string, using the standard conventions for mime data
@@ -1021,9 +1051,11 @@ public:
    * returns a QByteArray and not a QString. Which makes sense since
    * everything is 7 bit (ascii) after being percent-encoded.
    */
+#ifndef KDE_NO_DEPRECATED
   static KDE_DEPRECATED QString encode_string(const QString &str) {
     return QString::fromLatin1( QUrl::toPercentEncoding( str ).constData() ); //krazy:exclude=qclasses
   }
+#endif
 
   /**
    * Convert unicoded string to local encoding and use %-style
@@ -1036,9 +1068,11 @@ public:
    * everything is 7 bit (ascii) after being percent-encoded.
    *
    */
+#ifndef KDE_NO_DEPRECATED
   static KDE_DEPRECATED QString encode_string_no_slash(const QString &str) {
       return QString::fromLatin1( QUrl::toPercentEncoding( str, "/" ).constData() ); //krazy:exclude=qclasses
   }
+#endif
 
   /**
    * Decode %-style encoding and convert from local encoding to unicode.
@@ -1050,9 +1084,11 @@ public:
    * everything is 7 bit (ascii) when being percent-encoded.
    *
    */
+#ifndef KDE_NO_DEPRECATED
   static KDE_DEPRECATED QString decode_string(const QString &str) {
       return QUrl::fromPercentEncoding( str.toLatin1() ); //krazy:exclude=qclasses
   }
+#endif
 
 
   /**
@@ -1129,7 +1165,9 @@ Q_DECLARE_METATYPE(KUrl::List)
  * That means no malformed URL equals anything else.
  * @deprecated use KUrl(_url1).equals(KUrl(_url2)) instead.
  */
+#ifndef KDE_NO_DEPRECATED
 KDECORE_EXPORT_DEPRECATED bool urlcmp( const QString& _url1, const QString& _url2 ); // KDE5: remove, KUrl::equals is better API
+#endif
 
 /**
  * \relates KUrl
@@ -1143,7 +1181,9 @@ KDECORE_EXPORT_DEPRECATED bool urlcmp( const QString& _url1, const QString& _url
  * @param options a set of KUrl::EqualsOption flags
  * @deprecated use KUrl(_url1).equals(KUrl(_url2), options) instead.
  */
+#ifndef KDE_NO_DEPRECATED
 KDECORE_EXPORT_DEPRECATED bool urlcmp( const QString& _url1, const QString& _url2, const KUrl::EqualsOptions& options ); // KDE5: remove, KUrl::equals is better API
+#endif
 
 KDECORE_EXPORT uint qHash(const KUrl& kurl);
 
