@@ -37,6 +37,7 @@ class QDateTime;
 class KDateTime;
 class KCalendarSystem;
 class KCurrencyCode;
+class KDayPeriod;
 
 class KLocalePrivate;
 
@@ -80,13 +81,16 @@ public:
      * any of the specified languages, the default language (en_US) will be
      * used.
      *
-     * If you specify a configuration file, it has to be valid until
-     * the KLocale object is destroyed.
+     * If you specify a configuration file, it has to be valid until the KLocale
+     * object is destroyed.  Note that a setLocale() will be performed on the
+     * config using the current locale language, which may cause a sync()
+     * and reparseConfiguration() which will save any changes you have made and
+     * load any changes other shared copies have made.
      *
      * @param catalog the name of the main language file
      * @param config  a configuration file with a Locale group detailing
      *                locale-related preferences (such as language and
-     *                formatting options)
+     *                formatting options).
      */
     explicit KLocale(const QString& catalog, KSharedConfig::Ptr config = KSharedConfig::Ptr());
 
@@ -96,12 +100,16 @@ public:
      * Allows you to override the language and, optionally, the
      * country of this locale.
      *
+     * If you specify a configuration file, a setLocale() will be performed on
+     * the config using the current locale language, which may cause a sync()
+     * and reparseConfiguration() which will save any changes you have made.
+     *
      * @param catalog  the name of the main language file
-     * @param language the language for the locale
-     * @param country  the country for the locale
+     * @param language the ISO Language Code for the locale, e.g. "en" for English
+     * @param country  the ISO Country Code for the locale, e.g. "us" for USA
      * @param config   a configuration file with a Locale group detailing
-     *                 locale-related preferences (such as date and time
-     *                 formatting options)
+     *                 locale-related preferences (such as language and
+     *                 formatting options).
      */
     KLocale(const QString& catalog, const QString &language, const QString &country = QString(),
             KConfig *config = 0);
@@ -460,7 +468,7 @@ public:
      *
      * @return Default number of fractional digits used by locale.
      */
-    int fracDigits() const;
+    KDE_DEPRECATED int fracDigits() const;
 
     /**
      * @since 4.4
@@ -691,16 +699,22 @@ public:
      *        a sanely-sized number.
      * @return converted size as a translated string including the units.
      *         E.g. "1.23 KiB", "2 GB" (JEDEC), "4.2 kB" (Metric).
+     * @see BinaryUnitDialect
      */
     QString formatByteSize(double size, int precision,
                            BinaryUnitDialect dialect = KLocale::DefaultBinaryDialect,
                            BinarySizeUnits specificUnit = KLocale::DefaultBinaryUnits) const;
 
     /**
-     * Returns the user's default binary unit dialect.
+     * Returns the user's configured binary unit dialect.
+     * e.g. if MetricBinaryDialect is returned then the values
+     * configured for how much a set of bytes are worth would
+     * be 10^(3*n) and KB (1000 bytes == 1 KB), in this case.
+     *
+     * Will never return DefaultBinaryDialect.
      *
      * @since 4.4
-     * @return User's default binary unit dialect
+     * @return User's configured binary unit dialect
      * @see BinaryUnitDialect
      */
     BinaryUnitDialect binaryUnitDialect() const;
@@ -752,8 +766,61 @@ public:
      *
      * @return If nouns are declined
      */
-    bool nounDeclension() const;
+    KDE_DEPRECATED bool nounDeclension() const;
 
+    //KDE5 move to KDateTime namespace
+    /**
+     * @since 4.6
+     *
+     * Available Calendar Systems
+     *
+     * @see setCalendarSystem()
+     * @see calendarSystem()
+     */
+    enum CalendarSystem {
+        QDateCalendar = 1, /**< KDE Default, hybrid of Gregorian and Julian as used by QDate */
+        //BahaiCalendar = 2, /**< Baha'i Calendar */
+        //BuddhistLunarCalendar = 3, /**< Buddhist Lunar Calendar*/
+        //ChineseCalendar = 4, /**< Chinese Calendar */
+        CopticCalendar = 5, /**< Coptic Calendar as used Coptic Church and some parts of Egypt */
+        EthiopianCalendar = 6, /**< Ethiopian Calendar, aka Ethiopic Calendar */
+        //EthiopianAmeteAlemCalendar = 7, /**< Ethiopian Amete Alem version, aka Ethiopic Amete Alem */
+        GregorianCalendar = 8, /**< Gregorian Calendar, pure proleptic implementation */
+        HebrewCalendar = 9, /**< Hebrew Calendar, aka Jewish Calendar */
+        //HinduCalendar = 10, /**< Hindu Lunar Calendar */
+        //IslamicLunarCalendar = 11, /**< Islamic Lunar Calendar */
+        IslamicCivilCalendar = 12, /**< Islamic Civil Calendar, aka Hijri, not the Lunar Calendar */
+        //IslamicUmAlQuraCalendar = 13, /**< Islamic Lunar Calendar, Um Al Qura varient used in Saudi Arabia */
+        IndianNationalCalendar = 14, /**< Indian National Calendar, not the Lunar Calendar */
+        //Iso8601Calendar = 15, /**< ISO 8601 Standard Calendar */
+        JalaliCalendar = 16, /**< Jalali Calendar, aka Persian or Iranian, also used in Afghanistan */
+        //JalaliBirashkCalendar = 17, /**< Jalali Calendar, Birashk Algorythm variant */
+        //Jalali33YearCalendar = 18, /**< Jalali Calendar, 33 Year cycle variant */
+        JapaneseCalendar= 19, /**< Japanese Calendar, Gregorian calculation using Japanese Era (Nengô) */
+        //JucheCalendar = 20, /**< Juche Calendar, used in North Korea */
+        JulianCalendar = 21, /**< Julian Calendar, as used in Orthodox Churches */
+        MinguoCalendar= 22, /**< Minguo Calendar, aka ROC, Republic of China or Taiwanese */
+        ThaiCalendar = 23 /**< Thai Calendar, aka Buddhist or Thai Buddhist */
+    };
+
+    //KDE5 move to KDateTime namespace
+    /**
+     * @since 4.6
+     *
+     * System used for Week Numbers
+     *
+     * @see setWeekNumberSystem()
+     * @see weekNumberSystem()
+     */
+    enum WeekNumberSystem {
+        DefaultWeekNumber = -1, /**< The system locale default */
+        IsoWeekNumber     =  0, /**< ISO Week Number */
+        FirstFullWeek     =  1, /**< Week 1 starts on the first Week Start Day in year ends after 7 days */
+        FirstPartialWeek  =  2, /**< Week 1 starts Jan 1st ends day before first Week Start Day in year */
+        SimpleWeek        =  3  /**< Week 1 starts Jan 1st ends after 7 days */
+    };
+
+    //KDE5 move to KDateTime namespace
     /**
      * @since 4.4
      *
@@ -765,6 +832,90 @@ public:
          UnicodeFormat     /**< UNICODE Standard (Qt/Java/OSX/Windows) */
     };
 
+    //KDE5 move to KDateTime namespace
+    /**
+     * @since 4.6
+     *
+     * Mode to use when parsing a Date Time input string
+     */
+    enum DateTimeParseMode {
+         LiberalParsing   /**< Parse Date/Time liberally.  So long as the
+                               input string contains at least a reconizable
+                               month and day the input will be accepted. */
+         //ModerateParsing, /**< Parse Date/Time with modeate tolerance.
+         //                      The date components in the format must all
+         //                      occur in the input and in the same order,
+         //                      but the spacing and the componants themselves
+         //                      may vary from the strict format. */
+         //StrictParsing    /**< Parse Date/Time strictly to the format. */
+    };
+
+    //KDE5 move to KDateTime namespace
+    /**
+     * @since 4.6
+     *
+     * The various Components that make up a Date / Time
+     * In the future the Components may be combined as flags for dynamic
+     * generation of Date Formats.
+     *
+     * @see KCalendarSystem
+     * @see KLocalizedDate
+     * @see DateTimeComponentFormat
+     */
+    enum DateTimeComponent {
+        Year          = 0x1,        /**< The Year portion of a date, may be number or name */
+        YearName      = 0x2,        /**< The Year Name portion of a date */
+        Month         = 0x4,        /**< The Month portion of a date, may be number or name */
+        MonthName     = 0x8,        /**< The Month Name portion of a date */
+        Day           = 0x10,       /**< The Day portion of a date, may be number or name */
+        DayName       = 0x20,       /**< The Day Name portion of a date */
+        JulianDay     = 0x40,       /**< The Julian Day of a date */
+        EraName       = 0x80,       /**< The Era Name portion of a date */
+        EraYear       = 0x100,      /**< The Era and Year portion of a date */
+        YearInEra     = 0x200,      /**< The Year In Era portion of a date */
+        DayOfYear     = 0x400,      /**< The Day Of Year portion of a date, may be number or name */
+        DayOfYearName = 0x800,      /**< The Day Of Year Name portion of a date */
+        DayOfWeek     = 0x1000,     /**< The Day Of Week / Weekday portion of a date, may be number or name */
+        DayOfWeekName = 0x2000,     /**< The Day Of Week Name / Weekday Name portion of a date */
+        Week          = 0x4000,     /**< The Week Number portion of a date */
+        WeekYear      = 0x8000,     /**< The Week Year portion of a date */
+        MonthsInYear  = 0x10000,    /**< The Months In Year portion of a date */
+        WeeksInYear   = 0x20000,    /**< The Weeks In Year portion of a date */
+        DaysInYear    = 0x40000,    /**< The Days In Year portion of a date */
+        DaysInMonth   = 0x80000,    /**< The Days In Month portion of a date */
+        DaysInWeek    = 0x100000,   /**< The Days In Week portion of a date */
+        Hour          = 0x200000,   /**< The Hours portion of a date */
+        Minute        = 0x400000,   /**< The Minutes portion of a date */
+        Second        = 0x800000,   /**< The Seconds portion of a date */
+        Millisecond   = 0x1000000,  /**< The Milliseconds portion of a date */
+        DayPeriod     = 0x2000000,  /**< The Day Period portion of a date, e.g. AM/PM */
+        DayPeriodHour = 0x4000000,  /**< The Day Period Hour portion of a date */
+        Timezone      = 0x8000000,  /**< The Time Zone portion of a date, may be offset or name */
+        TimezoneName  = 0x10000000, /**< The Time Zone Name portion of a date */
+        UnixTime      = 0x20000000  /**< The UNIX Time portion of a date */
+    };
+    Q_DECLARE_FLAGS(DateTimeComponents, DateTimeComponent)
+
+    //KDE5 move to KDateTime namespace
+    /**
+     * @since 4.6
+     *
+     * Format used for individual Date/Time Components when converted to/from a string
+     * Largely equivalent to the UNICODE CLDR format width definitions 1..5
+     *
+     * @see DateTimeComponentFormat
+     */
+    enum DateTimeComponentFormat {
+        DefaultComponentFormat = -1, /**< The system locale default for the componant */
+        ShortNumber = 0,             /**< Number at its natural width, e.g. 2 for the 2nd*/
+        LongNumber,                  /**< Number padded to a required width, e.g. 02 for the 2nd*/
+        //OrdinalNumber                /**< Ordinal number format, e.g. "2nd" for the 2nd */
+        NarrowName = 3,              /**< Narrow text format, may not be unique, e.g. M for Monday */
+        ShortName,                   /**< Short text format, e.g. Mon for Monday */
+        LongName                     /**< Long text format, e.g. Monday for Monday */
+    };
+
+    //KDE5 move to KDateTime namespace
     /**
      * Format for date string.
      */
@@ -782,6 +933,7 @@ public:
         IsoOrdinalDate    /**< ISO-8601 Ordinal Date format YYYY-DDD, e.g. 2009-001 */
     };
 
+    //KDE5 move to KDateTime namespace
     /**
      * Returns a string formatted to the current locale's conventions
      * regarding dates.
@@ -793,6 +945,7 @@ public:
      */
     QString formatDate(const QDate &date, DateFormat format = LongDate) const;
 
+    //KDE5 move to KDateTime namespace
     /**
      * Returns a string formatted to the current locale's conventions
      * regarding both date and time.
@@ -807,6 +960,7 @@ public:
     QString formatDateTime(const QDateTime &dateTime, DateFormat format = ShortDate,
                            bool includeSecs = false) const;
 
+    //KDE5 move to KDateTime namespace
     /**
      * Options for formatting date-time values.
      */
@@ -816,6 +970,7 @@ public:
     };
     Q_DECLARE_FLAGS(DateTimeFormatOptions, DateTimeFormatOption)
 
+    //KDE5 move to KDateTime namespace
     /**
      * Returns a string formatted to the current locale's conventions
      * regarding both date and time.
@@ -853,6 +1008,7 @@ public:
      */
     QString formatTime(const QTime &pTime, bool includeSecs = false, bool isDuration = false) const;
 
+    //KDE5 move to KDateTime namespace
     /**
      * @since 4.4
      *
@@ -866,14 +1022,20 @@ public:
                                     ///< keep the 12/24h format as specified by locale time
                                     ///< format, eg. "07.33.05" instead of "07.33.05 pm" for
                                     ///< time format "%I.%M.%S %p".
-        TimeDuration       = 0x6    ///< Read/format time string as duration. This will strip
+        TimeDuration       = 0x6,   ///< Read/format time string as duration. This will strip
                                     ///< the am/pm suffix and read/format times with an hour
                                     ///< value of 0-23 hours, eg. "19.33.05" instead of
                                     ///< "07.33.05 pm" for time format "%I.%M.%S %p".
                                     ///< This automatically implies @c TimeWithoutAmPm.
+        TimeFoldHours      = 0xE    ///< Read/format time string as duration. This will not
+                                    ///< not output the hours part of the duration but will
+                                    ///< add the hours (times sixty) to the number of minutes,
+                                    ///< eg. "70.23" instead of "01.10.23" for time format
+                                    ///< "%I.%M.%S %p".
     };
     Q_DECLARE_FLAGS(TimeFormatOptions, TimeFormatOption)
 
+    //KDE5 move to KDateTime namespace
     /**
      * @since 4.4
      *
@@ -905,6 +1067,17 @@ public:
      */
     bool use12Clock() const;
 
+    /**
+     * @since 4.6
+     *
+     * Returns the Day Period matching the time given
+     *
+     * @param time the time to return the day period for
+     * @param format the format to return teh day period in
+     * @return the Day Period for the given time
+     */
+    QString dayPeriodText(const QTime &time, DateTimeComponentFormat format = DefaultComponentFormat) const;
+
     /**
      * Use this to determine which day is the first day of the week.
      *
@@ -943,22 +1116,85 @@ public:
      */
     const KCalendarSystem * calendar() const;
 
+    //KDE5 remove
     /**
+     * @deprecated use calendarSystem() instead
+     *
      * Returns the name of the calendar system that is currently being
      * used by the system.
      *
+     * @see calendarSystem()
      * @return the name of the calendar system
      */
-    QString calendarType() const;
+    KDE_DEPRECATED QString calendarType() const;
 
     /**
-     * Changes the current calendar system to the calendar specified.
-     * Currently "gregorian" and "hijri" are supported. If the calendar
-     * system specified is not found, gregorian will be used.
+     * @since 4.6
      *
+     * Returns the type of Calendar System used in this Locale
+     *
+     * @see KLocale::CalendarSystem
+     * @see KCalendarSystem
+     * @return the type of Calendar System
+     */
+    KLocale::CalendarSystem calendarSystem() const;
+
+    //KDE5 remove
+    /**
+     * @deprecated use setCalendarSystem() instead
+     *
+     * Changes the current calendar system to the calendar specified.
+     * If the calendar system specified is not found, gregorian will be used.
+     *
+     * @see setCalendarSystem()
      * @param calendarType the name of the calendar type
      */
-    void setCalendar(const QString & calendarType);
+    KDE_DEPRECATED void setCalendar(const QString & calendarType);
+
+    /**
+     * @since 4.6
+     *
+     * Sets the type of Calendar System to use in this Locale
+     *
+     * @see KLocale::CalendarSystem
+     * @see KCalendarSystem
+     * @param calendarSystem the Calendar System to use
+     */
+    void setCalendarSystem(KLocale::CalendarSystem calendarSystem);
+
+    /**
+     * @since 4.6
+     *
+     * Sets the type of Week Number System to use in this Locale
+     *
+     * @see Klocale::WeekNumberSystem
+     * @see weekNumberSystem()
+     * @param weekNumberSystem the Week Number System to use
+     */
+    void setWeekNumberSystem(KLocale::WeekNumberSystem weekNumberSystem);
+
+    //KDE5 remove in favour of const version
+    /**
+     * @since 4.6
+     *
+     * Returns the type of Week Number System used in this Locale
+     *
+     * @see Klocale::WeekNumberSystem
+     * @see setWeekNumberSystem()
+     * @returns the Week Number System used
+     */
+    KLocale::WeekNumberSystem weekNumberSystem();
+
+    /**
+     * @since 4.7
+     *
+     * Returns the type of Week Number System used in this Locale
+     *
+     * @see Klocale::WeekNumberSystem
+     * @see setWeekNumberSystem()
+     * @returns the Week Number System used
+     */
+    KLocale::WeekNumberSystem weekNumberSystem() const;
 
     /**
      * Converts a localized monetary string to a double.
@@ -982,6 +1218,7 @@ public:
      */
     double readNumber(const QString &numStr, bool * ok = 0) const;
 
+    //KDE5 move to KDateTime namespace
     /**
      * Converts a localized date string to a QDate.  This method will try all
      * ReadDateFlag formats in preferred order to read a valid date.
@@ -997,6 +1234,7 @@ public:
      */
     QDate readDate(const QString &str, bool* ok = 0) const;
 
+    //KDE5 move to KDateTime namespace
     /**
      * Converts a localized date string to a QDate, using the specified format.
      * You will usually not want to use this method.
@@ -1004,6 +1242,7 @@ public:
      */
     QDate readDate(const QString &intstr, const QString &fmt, bool* ok = 0) const;
 
+    //KDE5 move to KDateTime namespace
     /**
      * Flags for readDate()
      */
@@ -1020,6 +1259,7 @@ public:
                                            ISO Week date format (YYYY-DDD) */
     };
 
+    //KDE5 move to KDateTime namespace
     /**
      * Converts a localized date string to a QDate.
      * This method is stricter than readDate(str,&ok): it will only accept
@@ -1130,7 +1370,14 @@ public:
 
     /**
      * Returns the country code of the country where the user lives.
-     * defaultCountry() is returned by default, if no other available.
+     *
+     * The returned code complies with the ISO 3166-1 alpha-2 standard,
+     * except by KDE convention it is returned in lowercase whereas the
+     * official standard is uppercase.
+     * See http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2 for details.
+     *
+     * defaultCountry() is returned by default, if no other available,
+     * this will always be uppercase 'C'.
      *
      * Use countryCodeToName(country) to get human readable, localized
      * country names.
@@ -1141,6 +1388,26 @@ public:
      */
     QString country() const;
 
+    /**
+     * @since 4.6
+     *
+     * Returns the Country Division Code of the Country where the user lives.
+     * When no value is set, then the Country Code will be returned.
+     *
+     * The returned code complies with the ISO 3166-2 standard.
+     * See http://en.wikipedia.org/wiki/ISO_3166-2 for details.
+     *
+     * Note that unlike country() this method will return the correct case,
+     * i.e. normally uppercase..
+     *
+     * In KDE 4.6 it is the apps responsibility to obtain a translation for the
+     * code, translation and other services will be priovided in KDE 4.7.
+     *
+     * @return the Country Division Code for the user
+     * @see setCountryDivisionCode
+     */
+    QString countryDivisionCode() const;
+
     /**
      * Returns the language codes selected by user, ordered by decreasing
      * priority.
@@ -1428,7 +1695,7 @@ public:
      *
      * @param digits The default number of digits to use.
      */
-    void setFracDigits(int digits);
+    KDE_DEPRECATED void setFracDigits(int digits);
 
     /**
      * @since 4.4
@@ -1532,7 +1799,7 @@ public:
     /**
      * Adds another catalog to search for translation lookup.
      * This function is useful for extern libraries and/or code,
-     * that provide there own messages.
+     * that provide their own messages.
      *
      * If the catalog does not exist for the chosen language,
      * it will be ignored and en_US will be used.
@@ -1570,9 +1837,24 @@ public:
      * @return list of all language codes
      *
      * @see languageCodeToName
+     * @see installedLanguages
      */
     QStringList allLanguagesList() const;
 
+    /**
+     * @since 4.6
+     *
+     * Provides list of all installed KDE Language Translations.
+     *
+     * Use languageCodeToName(language) to get human readable, localized
+     * language names.
+     *
+     * @return list of all installed language codes
+     *
+     * @see languageCodeToName
+     */
+    QStringList installedLanguages() const;
+
     /**
      * Convert a known language code to a human readable, localized form.
      * If an unknown language code is supplied, empty string is returned;
@@ -1587,6 +1869,7 @@ public:
      * @see language
      * @see languageList
      * @see allLanguagesList
+     * @see installedLanguages
      */
     QString languageCodeToName(const QString &language) const;
 
@@ -1667,7 +1950,12 @@ public:
     static QString defaultLanguage();
 
     /**
-     * Returns the name of the default country.
+     * Returns the code of the default country, i.e. "C"
+     *
+     * This function will not provide a sensible value to use in your app,
+     * please use country() instead.
+     *
+     * @see country
      *
      * @return Name of the default country
      */
@@ -1708,24 +1996,51 @@ public:
      * unchanged if failed. It will force a reload of the country specific
      * configuration.
      *
+     * An empty country value will set the country to the system default.
+     *
+     * If you specify a configuration file, a setLocale() will be performed on
+     * the config using the current locale language, which may cause a sync()
+     * and reparseConfiguration() which will save any changes you have made.
+     *
      * @param country the ISO 3166 country code
      * @param config  a configuration file with a Locale group detailing
-     *                locale-related preferences (such as date and time
-     *                formatting)
+     *                locale-related preferences (such as language and
+     *                formatting options).
      *
      * @return @c true on success, @c false on failure
      */
     bool setCountry(const QString & country, KConfig *config);
 
+    /**
+     * @since 4.6
+     *
+     * Sets the Country Division Code of the Country where the user lives.
+     *
+     * The code must comply with the ISO 3166-2 standard.
+     * See http://en.wikipedia.org/wiki/ISO_3166-2 for details.
+     *
+     * In KDE 4.6 it is the apps responsibility to validate the input,
+     * full validation and other services will be provided in KDE 4.7.
+     *
+     * @param countryDivision the Country Division Code for the user
+     * @return @c true on success, @c false on failure
+     * @see countryDivisionCode
+     */
+    bool setCountryDivisionCode(const QString & countryDivision);
+
     /**
      * Changes the current language. The current language will be left
      * unchanged if failed. It will force a reload of the country specific
      * configuration as well.
      *
+     * If you specify a configuration file, a setLocale() will be performed on
+     * the config using the current locale language, which may cause a sync()
+     * and reparseConfiguration() which will save any changes you have made.
+     *
      * @param language the language code
-     * @param config  a configuration file with a Locale group detailing
-     *                locale-related preferences (such as date and time
-     *                formatting)
+     * @param config   a configuration file with a Locale group detailing
+     *                 locale-related preferences (such as language and
+     *                 formatting options).
      *
      * @return true on success
      */
@@ -1805,11 +2120,23 @@ public:
     QString convertDigits(const QString &str, DigitSet digitSet,
                           bool ignoreContext = false) const;
 
+    /**
+     * @since 4.8
+     *
+     * Reparse locale configuration files for the current selected
+     * language.
+     */
+    void reparseConfiguration();
+
 private:
+    friend class KLocalePrivate;
+    friend class KLocaleTest;
+    friend class KDateTimeFormatter;
     KLocalePrivate * const d;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(KLocale::DateTimeFormatOptions)
+Q_DECLARE_OPERATORS_FOR_FLAGS(KLocale::DateTimeComponents)
 Q_DECLARE_OPERATORS_FOR_FLAGS(KLocale::TimeFormatOptions)
 Q_DECLARE_OPERATORS_FOR_FLAGS(KLocale::TimeProcessingOptions)
 
