@@ -21,11 +21,9 @@
 #include "kprotocolinfo_p.h"
 #include "kprotocolinfofactory.h"
 
+#include <kmimetypetrader.h>
 #include <kstandarddirs.h>
-#include <kglobal.h>
-#include <kdebug.h>
 #include <kconfig.h>
-#include <kstringhandler.h>
 #include <kconfiggroup.h>
 
 //
@@ -56,11 +54,16 @@ KProtocolInfo::KProtocolInfo(const QString &path)
   d->canRenameFromFile = config.readEntry( "renameFromFile", false );
   d->canRenameToFile = config.readEntry( "renameToFile", false );
   d->canDeleteRecursive = config.readEntry( "deleteRecursive", false );
-  d->fileNameUsedForCopying = config.readEntry( "fileNameUsedForCopying", "FromURL" ) == "Name";
+  const QString fnu = config.readEntry( "fileNameUsedForCopying", "FromURL" );
+  d->fileNameUsedForCopying = FromUrl;
+  if (fnu == QLatin1String("Name"))
+    d->fileNameUsedForCopying = Name;
+  else if (fnu == QLatin1String("DisplayName"))
+    d->fileNameUsedForCopying = DisplayName;
 
   m_listing = config.readEntry( "listing", QStringList() );
   // Many .protocol files say "Listing=false" when they really mean "Listing=" (i.e. unsupported)
-  if ( m_listing.count() == 1 && m_listing.first() == "false" )
+  if ( m_listing.count() == 1 && m_listing.first() == QLatin1String("false") )
     m_listing.clear();
   m_supportsListing = ( m_listing.count() > 0 );
   m_defaultMimetype = config.readEntry( "defaultMimetype" );
@@ -72,17 +75,17 @@ KProtocolInfo::KProtocolInfo(const QString &path)
   d->maxSlavesPerHost = config.readEntry( "maxInstancesPerHost", 0);
 
   QString tmp = config.readEntry( "input" );
-  if ( tmp == "filesystem" )
+  if ( tmp == QLatin1String("filesystem") )
     m_inputType = KProtocolInfo::T_FILESYSTEM;
-  else if ( tmp == "stream" )
+  else if ( tmp == QLatin1String("stream") )
     m_inputType = KProtocolInfo::T_STREAM;
   else
     m_inputType = KProtocolInfo::T_NONE;
 
   tmp = config.readEntry( "output" );
-  if ( tmp == "filesystem" )
+  if ( tmp == QLatin1String("filesystem") )
     m_outputType = KProtocolInfo::T_FILESYSTEM;
-  else if ( tmp == "stream" )
+  else if ( tmp == QLatin1String("stream") )
     m_outputType = KProtocolInfo::T_STREAM;
   else
     m_outputType = KProtocolInfo::T_NONE;
@@ -91,7 +94,7 @@ KProtocolInfo::KProtocolInfo(const QString &path)
   if (d->docPath.isEmpty())
     d->docPath = config.readPathEntry( "DocPath", QString() );
   d->protClass = config.readEntry( "Class" ).toLower();
-  if (d->protClass[0] != ':')
+  if (d->protClass[0] != QLatin1Char(':'))
      d->protClass.prepend(QLatin1Char(':'));
 
   const QStringList extraNames = config.readEntry( "ExtraNames", QStringList() );
@@ -104,7 +107,7 @@ KProtocolInfo::KProtocolInfo(const QString &path)
       d->extraFields.append( ExtraField( *it, static_cast<ExtraField::Type>(type) ) );
   }
 
-  d->showPreviews = config.readEntry( "ShowPreviews", d->protClass == ":local" );
+  d->showPreviews = config.readEntry( "ShowPreviews", d->protClass == QLatin1String(":local") );
 
   d->capabilities = config.readEntry( "Capabilities", QStringList() );
   d->proxyProtocol = config.readEntry( "ProxiedBy" );
@@ -171,7 +174,7 @@ KProtocolInfo::load( QDataStream& _str)
    d->canRenameFromFile = (i_canRenameFromFile != 0);
    d->canRenameToFile = (i_canRenameToFile != 0);
    d->canDeleteRecursive = (i_canDeleteRecursive != 0);
-   d->fileNameUsedForCopying = (i_fileNameUsedForCopying != 0);
+   d->fileNameUsedForCopying = FileNameUsedForCopying(i_fileNameUsedForCopying);
    m_determineMimetypeFromExtension = (i_determineMimetypeFromExtension != 0);
    d->showPreviews = (i_showPreviews != 0);
 }
@@ -211,7 +214,7 @@ KProtocolInfoPrivate::save( QDataStream& _str)
    i_canRenameFromFile = canRenameFromFile ? 1 : 0;
    i_canRenameToFile = canRenameToFile ? 1 : 0;
    i_canDeleteRecursive = canDeleteRecursive ? 1 : 0;
-   i_fileNameUsedForCopying = fileNameUsedForCopying ? 1 : 0;
+   i_fileNameUsedForCopying = int(fileNameUsedForCopying);
    i_determineMimetypeFromExtension = q->m_determineMimetypeFromExtension ? 1 : 0;
    i_showPreviews = showPreviews ? 1 : 0;
   i_uriMode = 0;
@@ -271,7 +274,7 @@ QString KProtocolInfo::config( const QString& _protocol )
   if ( !prot )
     return QString();
 
-  return QString("kio_%1rc").arg(prot->m_config);
+  return QString::fromLatin1("kio_%1rc").arg(prot->m_config);
 }
 
 int KProtocolInfo::maxSlaves( const QString& _protocol )
@@ -301,13 +304,19 @@ bool KProtocolInfo::determineMimetypeFromExtension( const QString &_protocol )
   return prot->m_determineMimetypeFromExtension;
 }
 
-QString KProtocolInfo::exec( const QString& _protocol )
+QString KProtocolInfo::exec(const QString& protocol)
 {
-  KProtocolInfo::Ptr prot = KProtocolInfoFactory::self()->findProtocol(_protocol);
-  if ( !prot )
-    return QString();
+  KProtocolInfo::Ptr prot = KProtocolInfoFactory::self()->findProtocol(protocol);
+  if ( prot ) {
+      return prot->m_exec;
+  }
 
-  return prot->m_exec;
+  // Maybe it's "helper protocol", i.e. launches an app?
+  const KService::Ptr service = KMimeTypeTrader::self()->preferredService(QString::fromLatin1("x-scheme-handler/") + protocol);
+  if (service)
+      return service->exec();
+
+  return QString();
 }
 
 KProtocolInfo::ExtraFieldList KProtocolInfo::extraFields( const KUrl &url )
@@ -402,7 +411,7 @@ bool KProtocolInfo::canDeleteRecursive() const
 KProtocolInfo::FileNameUsedForCopying KProtocolInfo::fileNameUsedForCopying() const
 {
     Q_D(const KProtocolInfo);
-  return d->fileNameUsedForCopying ? Name : FromUrl;
+    return d->fileNameUsedForCopying;
 }
 
 bool KProtocolInfo::isFilterProtocol( const KUrl &url )
@@ -419,10 +428,11 @@ bool KProtocolInfo::isHelperProtocol( const QString &protocol )
 {
   // We call the findProtocol directly (not via KProtocolManager) to bypass any proxy settings.
   KProtocolInfo::Ptr prot = KProtocolInfoFactory::self()->findProtocol(protocol);
-  if ( !prot )
-    return false;
+  if ( prot )
+      return prot->m_isHelperProtocol;
 
-  return prot->m_isHelperProtocol;
+  const KService::Ptr service = KMimeTypeTrader::self()->preferredService(QString::fromLatin1("x-scheme-handler/") + protocol);
+  return !service.isNull();
 }
 
 bool KProtocolInfo::isKnownProtocol( const KUrl &url )
@@ -434,7 +444,7 @@ bool KProtocolInfo::isKnownProtocol( const QString &protocol )
 {
   // We call the findProtocol (const QString&) to bypass any proxy settings.
   KProtocolInfo::Ptr prot = KProtocolInfoFactory::self()->findProtocol(protocol);
-  return prot;
+  return prot || isHelperProtocol(protocol);
 }
 
 QDataStream& operator>>( QDataStream& s, KProtocolInfo::ExtraField& field )  {
