@@ -17,8 +17,10 @@
     Boston, MA 02110-1301, USA.
 */
 
-#include <qtest_kde.h>
 #include "kconfigtest.h"
+
+#include <qtest_kde.h>
+#include <ktempdir.h>
 #include <kdesktopfile.h>
 #include <kstandarddirs.h>
 #include "kconfigtest.moc"
@@ -68,15 +70,17 @@ QTEST_KDEMAIN_CORE( KConfigTest )
 #define BYTEARRAYLISTENTRY1 QList<QByteArray>() << "" << "1,2" << "end"
 #define VARIANTLISTENTRY (QVariantList() << true << false << QString("joe") << 10023)
 #define VARIANTLISTENTRY2 (QVariantList() << POINTENTRY << SIZEENTRY)
-#define HOMEPATH QDir::homePath()+"/foo"
-#define HOMEPATHESCAPE QDir::homePath()+"/foo/$HOME"
-#define SUBGROUPLIST (QStringList() << "SubGroup/3" << "SubGroup1" << "SubGroup2")
-#define PARENTGROUPKEYS (QStringList() << "parentgrpstring")
-#define SUBGROUP3KEYS (QStringList() << "sub3string")
+#define HOMEPATH QString(QDir::homePath()+"/foo")
+#define HOMEPATHESCAPE QString(QDir::homePath()+"/foo/$HOME")
 #define DOLLARGROUP "$i"
 
 void KConfigTest::initTestCase()
 {
+  // Use a different directory for the config files created by this test,
+  // so that cleanupTestCase doesn't delete kdebugrc
+  // This makes the save location for the config resource, "$HOME/.kde-unit-test/kconfigtest/"
+  KGlobal::dirs()->addResourceType("config", 0, "kconfigtest");
+
   // to make sure all files from a previous failed run are deleted
   cleanupTestCase();
 
@@ -105,13 +109,13 @@ void KConfigTest::initTestCase()
   cg.writeEntry( "stringEntry5", STRINGENTRY5 );
   cg.writeEntry( "keywith=equalsign", STRINGENTRY1 );
   cg.deleteEntry( "stringEntry5" );
-  cg.deleteEntry( "stringEntry6" ); // deleting a nonexistant entry
+  cg.deleteEntry( "stringEntry6" ); // deleting a nonexistent entry
   cg.writeEntry( "byteArrayEntry1", QByteArray( STRINGENTRY1 ),
                  KConfig::Global|KConfig::Persistent );
   cg.writeEntry( "doubleEntry1", DOUBLEENTRY );
   cg.writeEntry( "floatEntry1", FLOATENTRY );
 
-  sc.deleteGroup("deleteMe"); // deleting a nonexistant group
+  sc.deleteGroup("deleteMe"); // deleting a nonexistent group
 
   cg = KConfigGroup(&sc, "Complex Types");
   cg.writeEntry( "rectEntry", RECTENTRY );
@@ -130,6 +134,9 @@ void KConfigTest::initTestCase()
   cg = KConfigGroup(&cg, "Nested Group 2.1");
   cg.writeEntry( "stringEntry3", STRINGENTRY3 );
 
+  cg = KConfigGroup(&ct, "Nested Group 3");
+  cg.writeEntry( "stringEntry3", STRINGENTRY3 );
+
   cg = KConfigGroup(&sc, "List Types" );
   cg.writeEntry( "listOfIntsEntry1", INTLISTENTRY1 );
   cg.writeEntry( "listOfByteArraysEntry1", BYTEARRAYLISTENTRY1 );
@@ -194,16 +201,33 @@ void KConfigTest::initTestCase()
 
 void KConfigTest::cleanupTestCase()
 {
-  QDir local = QDir::homePath() + "/.kde-unit-test/share/config";
+  const QString localConfig = KGlobal::dirs()->saveLocation("config");
+  KTempDir::removeDir(localConfig);
+  QVERIFY(!QFile::exists(localConfig));
+}
 
-  foreach(const QString &file, local.entryList(QDir::Files))
-    if(!local.remove(file))
-      qWarning("%s: removing failed", qPrintable( file ));
 
-  QCOMPARE((int)local.entryList(QDir::Files).count(), 0);
+static QList<QByteArray> readLinesFrom(const QString& path)
+{
+    QFile file(path);
+    const bool opened = file.open(QIODevice::ReadOnly|QIODevice::Text);
+    Q_ASSERT(opened);
+    Q_UNUSED(opened);
+    QList<QByteArray> lines;
+    QByteArray line;
+    do {
+        line = file.readLine();
+        if (!line.isEmpty())
+            lines.append(line);
+    } while(!line.isEmpty());
+    return lines;
+}
 
-  local.cdUp();
-  local.rmpath("config");
+static QList<QByteArray> readLines(const char* fileName = "kconfigtest")
+{
+    const QString path = KStandardDirs::locateLocal("config", fileName);
+    Q_ASSERT(!path.isEmpty());
+    return readLinesFrom(path);
 }
 
 // ### TODO: call this, and test the state of things afterwards
@@ -415,6 +439,7 @@ void KConfigTest::testPath()
   KConfigGroup sc3(&sc2, "Path Type");
   QCOMPARE( sc3.readPathEntry( "homepath", QString() ), HOMEPATH );
   QCOMPARE( sc3.readPathEntry( "homepathescape", QString() ), HOMEPATHESCAPE );
+  QCOMPARE( sc3.entryMap()["homepath"], HOMEPATH );
 
   {
       QFile file(KStandardDirs::locateLocal("config", "pathtest"));
@@ -424,23 +449,29 @@ void KConfigTest::testPath()
       out << "[Test Group]" << endl
               << "homePath=$HOME/foo" << endl
               << "homePath2=file://$HOME/foo" << endl
-              << "URL[$e]=file://$HOME/foo" << endl
-              << "hostname[$e]=$(hostname)" << endl;
+              << "withBraces[$e]=file://${HOME}/foo" << endl
+              << "URL[$e]=file://${HOME}/foo" << endl
+              << "hostname[$e]=$(hostname)" << endl
+              << "noeol=foo"; // no EOL
   }
   KConfig cf2("pathtest");
   KConfigGroup group = cf2.group("Test Group");
   QVERIFY(group.hasKey("homePath"));
   QCOMPARE(group.readPathEntry("homePath", QString()), HOMEPATH);
   QVERIFY(group.hasKey("homePath2"));
-  QCOMPARE(group.readPathEntry("homePath2", QString()), QString("file://") + HOMEPATH );
+  QCOMPARE(group.readPathEntry("homePath2", QString()), QString("file://" + HOMEPATH) );
+  QVERIFY(group.hasKey("withBraces"));
+  QCOMPARE(group.readPathEntry("withBraces", QString()), QString("file://" + HOMEPATH) );
   QVERIFY(group.hasKey("URL"));
-  QCOMPARE(group.readEntry("URL", QString()), QString("file://") + HOMEPATH );
+  QCOMPARE(group.readEntry("URL", QString()), QString("file://" + HOMEPATH) );
 #if !defined(Q_OS_WIN32) && !defined(Q_OS_MAC)
   // I don't know if this will work on windows
   // This test hangs on OS X
   QVERIFY(group.hasKey("hostname"));
   QCOMPARE(group.readEntry("hostname", QString()), QHostInfo::localHostName());
 #endif
+  QVERIFY(group.hasKey("noeol"));
+  QCOMPARE(group.readEntry("noeol", QString()), QString("foo"));
 }
 
 void KConfigTest::testPersistenceOfExpandFlagForPath()
@@ -596,6 +627,7 @@ void KConfigTest::testChangeGroup()
     KConfigGroup sc3(&sc, "Hello");
     QCOMPARE(sc3.name(), QString("Hello"));
     KConfigGroup newGroup(sc3);
+#ifndef KDE_NO_DEPRECATED
     newGroup.changeGroup("FooBar"); // deprecated!
     QCOMPARE(newGroup.name(), QString("FooBar"));
     QCOMPARE(sc3.name(), QString("Hello")); // unchanged
@@ -604,30 +636,67 @@ void KConfigTest::testChangeGroup()
     newGroup.writeEntry("InFooBar", "FB");
     QCOMPARE(KConfigGroup(&sc, "FooBar").entryMap().value("InFooBar"), QString("FB"));
     QCOMPARE(KConfigGroup(&sc, "Hello").entryMap().value("InFooBar"), QString());
+#endif
 
     KConfigGroup rootGroup(sc.group(""));
     QCOMPARE(rootGroup.name(), QString("<default>"));
     KConfigGroup sc32(rootGroup.group("Hello"));
     QCOMPARE(sc32.name(), QString("Hello"));
     KConfigGroup newGroup2(sc32);
+#ifndef KDE_NO_DEPRECATED
     newGroup2.changeGroup("FooBar"); // deprecated!
     QCOMPARE(newGroup2.name(), QString("FooBar"));
     QCOMPARE(sc32.name(), QString("Hello")); // unchanged
+#endif
+}
+
+// Simple test for deleteEntry
+void KConfigTest::testDeleteEntry()
+{
+    const char* configFile = "kconfigdeletetest";
+    {
+        KConfig conf(configFile);
+        conf.group("Hello").writeEntry("DelKey", "ToBeDeleted");
+    }
+    const QList<QByteArray> lines = readLines(configFile);
+    Q_ASSERT(lines.contains("[Hello]\n"));
+    Q_ASSERT(lines.contains("DelKey=ToBeDeleted\n"));
+
+    KConfig sc(configFile);
+    KConfigGroup group(&sc, "Hello");
+
+    group.deleteEntry("DelKey");
+    QCOMPARE( group.readEntry("DelKey", QString("Fietsbel")), QString("Fietsbel") );
+
+    group.sync();
+    Q_ASSERT(!readLines(configFile).contains("DelKey=ToBeDeleted\n"));
+    QCOMPARE( group.readEntry("DelKey", QString("still deleted")), QString("still deleted") );
 }
 
 void KConfigTest::testDelete()
 {
   KConfig sc( "kconfigtest" );
-  KConfigGroup sc3(&sc, "Hello");
-
-  sc3.deleteEntry("Test");
-  QCOMPARE( sc3.readEntry("Test", QString("Fietsbel")), QString("Fietsbel") );
 
   KConfigGroup ct(&sc, "Complex Types");
+
+  // First delete a nested group
+  KConfigGroup delgr(&ct, "Nested Group 3");
+  QVERIFY(delgr.exists());
+  QVERIFY(ct.hasGroup("Nested Group 3"));
+  delgr.deleteGroup();
+  QVERIFY(!delgr.exists());
+  QVERIFY(!ct.hasGroup("Nested Group 3"));
+  QVERIFY(ct.groupList().contains("Nested Group 3"));
+
   KConfigGroup ng(&ct, "Nested Group 2");
+  QVERIFY(sc.hasGroup("Complex Types"));
+  QVERIFY(!sc.hasGroup("Does not exist"));
   sc.deleteGroup("Complex Types");
   QCOMPARE(sc.group("Complex Types").keyList().count(), 0);
-  QVERIFY(sc.group("Complex Types").exists()); // yep, we deleted it, but it still "exists"...
+  QVERIFY(!sc.hasGroup("Complex Types")); // #192266
+  QVERIFY(!sc.group("Complex Types").exists());
+  QVERIFY(!ct.hasGroup("Nested Group 1"));
+
   QCOMPARE(ct.group("Nested Group 1").keyList().count(), 0);
   QCOMPARE(ct.group("Nested Group 2").keyList().count(), 0);
   QCOMPARE(ng.group("Nested Group 2.1").keyList().count(), 0);
@@ -639,6 +708,17 @@ void KConfigTest::testDelete()
   QVERIFY( !sc.entryMap("Hello").isEmpty() ); //not deleted group
   QVERIFY( sc.entryMap("FooBar").isEmpty() ); //inexistant group
 
+  cg.sync();
+  // Check what happens on disk
+  const QList<QByteArray> lines = readLines();
+  //qDebug() << lines;
+  QVERIFY(!lines.contains("[Complex Types]\n"));
+  QVERIFY(!lines.contains("[Complex Types][Nested Group 1]\n"));
+  QVERIFY(!lines.contains("[Complex Types][Nested Group 2]\n"));
+  QVERIFY(!lines.contains("[Complex Types][Nested Group 2.1]\n"));
+  QVERIFY(!lines.contains("[AAA]\n"));
+  QVERIFY(lines.contains("[Hello]\n")); // a group that was not deleted
+
   // test for entries that are marked as deleted when there is no default
   KConfig cf("kconfigtest", KConfig::SimpleConfig); // make sure there are no defaults
   cg = cf.group("Portable Devices");
@@ -648,7 +728,7 @@ void KConfigTest::testDelete()
 
   int count=0;
   foreach(const QByteArray& item, readLines())
-      if (item.startsWith("devices|"))
+      if (item.startsWith("devices|")) // krazy:exclude=strings
           count++;
   QCOMPARE(count, 2);
   cg.deleteEntry("devices|manual|/mnt/ipod");
@@ -685,11 +765,9 @@ void KConfigTest::testDefaultGroup()
         QCOMPARE(emptyGroup.readEntry("TestKey", QString()), QString("defaultGroup"));
     }
 
-#ifdef Q_OS_UNIX
     QList<QByteArray> lines = readLines();
     QVERIFY(!lines.contains("[]\n"));
     QCOMPARE(lines.first(), QByteArray("TestKey=defaultGroup\n"));
-#endif
 
     // Now that the group exists make sure it isn't returned from groupList()
     foreach(const QString& group, sc.groupList()) {
@@ -700,10 +778,8 @@ void KConfigTest::testDefaultGroup()
     sc.sync();
 
     // Test if deleteGroup worked
-#ifdef Q_OS_UNIX
     lines = readLines();
     QVERIFY(lines.first() != QByteArray("TestKey=defaultGroup\n"));
-#endif
 }
 
 void KConfigTest::testEmptyGroup()
@@ -734,11 +810,9 @@ void KConfigTest::testEmptyGroup()
         QCOMPARE(emptyGroup2.readEntry("TestKey", QString()), QString("emptyGroup"));
     }
 
-#ifdef Q_OS_UNIX
     QList<QByteArray> lines = readLines();
     QVERIFY(!lines.contains("[]\n")); // there's no support for the [] group, in fact.
     QCOMPARE(lines.first(), QByteArray("TestKey=emptyGroup\n"));
-#endif
 
     // Now that the group exists make sure it isn't returned from groupList()
     foreach(const QString& group, sc.groupList()) {
@@ -748,10 +822,8 @@ void KConfigTest::testEmptyGroup()
     sc.sync();
 
     // Test if deleteGroup worked
-#ifdef Q_OS_UNIX
     lines = readLines();
     QVERIFY(lines.first() != QByteArray("TestKey=defaultGroup\n"));
-#endif
 }
 
 void KConfigTest::testMerge()
@@ -886,13 +958,16 @@ void KConfigTest::testSubGroup()
     KConfigGroup srcg( &rcg, "ParentGroup" );
     QCOMPARE(srcg.readEntry( "parentgrpstring", ""), QString("somevalue") );
 
-    QCOMPARE(cg.groupList(), SUBGROUPLIST );
+    QCOMPARE(cg.groupList(), (QStringList() << "SubGroup/3" << "SubGroup1" << "SubGroup2"));
 
-    QCOMPARE(subcg3.keyList(), SUBGROUP3KEYS);
-    QCOMPARE(cg.keyList(), PARENTGROUPKEYS);
+    const QStringList expectedSubgroup3Keys = (QStringList() << "sub3string");
+    QCOMPARE(subcg3.keyList(), expectedSubgroup3Keys);
+    const QStringList expectedParentGroupKeys(QStringList() << "parentgrpstring");
 
-    QCOMPARE(QStringList(cg.entryMap().keys()), PARENTGROUPKEYS);
-    QCOMPARE(QStringList(subcg3.entryMap().keys()), SUBGROUP3KEYS);
+    QCOMPARE(cg.keyList(), expectedParentGroupKeys);
+
+    QCOMPARE(QStringList(cg.entryMap().keys()), expectedParentGroupKeys);
+    QCOMPARE(QStringList(subcg3.entryMap().keys()), expectedSubgroup3Keys);
 
     // Create A group containing only other groups. We want to make sure it
     // shows up in groupList of sc
@@ -910,19 +985,20 @@ void KConfigTest::testSubGroup()
     QVERIFY(negsub1.exists());
 
     // But it doesn't exist if it has no content
-    QEXPECT_FAIL("", "Currently groups without content do not exist!", Continue);
-    QVERIFY(negsub2.exists());
+    // Ossi and David say: this is how it's supposed to work.
+    // However you could add a dummy entry for now, or we could add a "Persist" feature to kconfig groups
+    // which would make it written out, much like "immutable" already makes them persistent.
+    QVERIFY(!negsub2.exists());
 
-    // A subgroup does not qualify as content if it is also empty!
-    QEXPECT_FAIL("", "Currently groups without content do not exist!", Continue);
-    QVERIFY(negsub3.exists());
+    // A subgroup does not qualify as content if it is also empty
+    QVERIFY(!negsub3.exists());
 
     // A subgroup with content is ok
     QVERIFY(negsub4.exists());
 
     // Only subgroups with content show up in groupList()
-    QEXPECT_FAIL("", "Empty subgroups do not show up in groupList()", Continue);
-    QCOMPARE(neg.groupList(), QStringList() << "NEG Child1" << "NEG Child2" << "NEG Child3" << "NEG Child4");
+    //QEXPECT_FAIL("", "Empty subgroups do not show up in groupList()", Continue);
+    //QCOMPARE(neg.groupList(), QStringList() << "NEG Child1" << "NEG Child2" << "NEG Child3" << "NEG Child4");
     // This is what happens
     QCOMPARE(neg.groupList(), QStringList() << "NEG Child1" << "NEG Child4");
 
@@ -933,6 +1009,16 @@ void KConfigTest::testSubGroup()
       QVERIFY(!group.contains("subgroup"));
       QVERIFY(!group.contains("SubGroup"));
     }
+
+    sc.sync();
+
+    // Check that the empty groups are not written out.
+    const QList<QByteArray> lines = readLines();
+    QVERIFY(lines.contains("[NoEntryGroup][NEG Child1]\n"));
+    QVERIFY(!lines.contains("[NoEntryGroup][NEG Child2]\n"));
+    QVERIFY(!lines.contains("[NoEntryGroup][NEG Child3]\n"));
+    QVERIFY(!lines.contains("[NoEntryGroup][NEG Child4]\n")); // implicit group, not written out
+    QVERIFY(lines.contains("[NoEntryGroup][NEG Child4][NEG Child4-1]\n"));
 }
 
 void KConfigTest::testAddConfigSources()
@@ -1167,23 +1253,6 @@ void KConfigTest::testDirtyOnEqualOverdo()
     QCOMPARE(cgLocal.readEntry("someKey", defvalr), val2);
 }
 
-QList<QByteArray> KConfigTest::readLines(const char* fileName)
-{
-    const QString path = KStandardDirs::locateLocal("config", fileName);
-    Q_ASSERT(!path.isEmpty());
-    QFile file(path);
-    const bool opened = file.open(QIODevice::ReadOnly);
-    Q_ASSERT(opened);
-    Q_UNUSED(opened);
-    QList<QByteArray> lines;
-    QByteArray line;
-    do {
-        line = file.readLine();
-        if (!line.isEmpty())
-            lines.append(line);
-    } while(!line.isEmpty());
-    return lines;
-}
 
 void KConfigTest::testCreateDir()
 {
@@ -1320,12 +1389,9 @@ void KConfigTest::testDeleteWhenLocalized()
     // The current state is: (Just return before this comment.)
     // [...]
     // foobool[ca]=true
-    // foobool[$d]
     // foobool[de]=wahr
-    // foostring[ca][$d]
     // foostring=ugly
     // foostring[de]=schoen
-    // [...]
 
     // Now switch the locale to "de" and repeat the checks. Results should be
     // the same. But they currently are not. The localized value are
@@ -1362,12 +1428,14 @@ void KConfigTest::testDeleteWhenLocalized()
     cg.deleteGroup();
     config.sync();
 
-    // Current state: The same as above [$d] for all values.
+    // Current state: [ca] and [de] entries left... oops.
+    //qDebug() << readLinesFrom(file);
 
-    // The group still exists but the values are all gone.
+    // Bug: The group still exists [because of the localized entries]...
     QVERIFY(cg.exists());
     QVERIFY(!cg.hasKey("foo"));
     QVERIFY(!cg.hasKey("foostring"));
+    QEXPECT_FAIL("", "Currently localized values are not deleted correctly", Continue);
     QVERIFY(!cg.hasKey("foobool"));
 
     // Now switch the locale to "de" and repeat the checks. All values
@@ -1422,13 +1490,8 @@ void KConfigTest::testKdeGlobals()
 
     // Check we wrote into kdeglobals
     const QList<QByteArray> lines = readLines("kdeglobals");
-#ifdef Q_WS_WIN
-#define LINEENDING "\r\n"
-#else
-#define LINEENDING "\n"
-#endif
-    QVERIFY(lines.contains("[General]"LINEENDING));
-    QVERIFY(lines.contains("testKG=1"LINEENDING));
+    QVERIFY(lines.contains("[General]\n"));
+    QVERIFY(lines.contains("testKG=1\n"));
 
     // Writing using NoGlobals
     {
@@ -1458,3 +1521,49 @@ void KConfigTest::testAnonymousConfig()
     general.writeEntry("Foo", "Bar");
     QCOMPARE(general.readEntry("Foo"), QString("Bar"));
 }
+
+void KConfigTest::testNoKdeHome()
+{
+    const QString kdeHome = QDir::homePath() + "/.kde-unit-test-does-not-exist";
+    setenv("KDEHOME", QFile::encodeName( kdeHome ), 1);
+    KTempDir::removeDir(kdeHome);
+    QVERIFY(!QFile::exists(kdeHome));
+
+    // Do what kde4-config does, and ensure kdehome doesn't get created (#233892)
+    KComponentData componentData("KConfigTest");
+    QVERIFY(!QFile::exists(kdeHome));
+    componentData.dirs();
+    QVERIFY(!QFile::exists(kdeHome));
+    componentData.config();
+    QVERIFY(!QFile::exists(kdeHome));
+
+    // Now try to actually save something, see if it works.
+    KConfigGroup group(componentData.config(), "Group");
+    group.writeEntry("Key", "Value");
+    group.sync();
+    QVERIFY(QFile::exists(kdeHome));
+    QVERIFY(QFile::exists(kdeHome + "/share/config/KConfigTestrc"));
+
+    // Cleanup
+    KTempDir::removeDir(kdeHome);
+}
+
+#include <QThreadPool>
+#include <qtconcurrentrun.h>
+
+// To find multithreading bugs: valgrind --tool=helgrind --track-lockorders=no ./kconfigtest testThreads
+void KConfigTest::testThreads()
+{
+    QThreadPool::globalInstance()->setMaxThreadCount(6);
+    QList<QFuture<void> > futures;
+    // Run in parallel some tests that work on different config files,
+    // otherwise unexpected things might indeed happen.
+    futures << QtConcurrent::run(this, &KConfigTest::testAddConfigSources);
+    futures << QtConcurrent::run(this, &KConfigTest::testSimple);
+    futures << QtConcurrent::run(this, &KConfigTest::testDefaults);
+    // QEXPECT_FAIL triggers race conditions, it should be fixed to use QThreadStorage...
+    //futures << QtConcurrent::run(this, &KConfigTest::testDeleteWhenLocalized);
+    //futures << QtConcurrent::run(this, &KConfigTest::testEntryMap);
+    Q_FOREACH(QFuture<void> f, futures) // krazy:exclude=foreach
+        f.waitForFinished();
+}
