@@ -352,6 +352,9 @@ void KMimeTypeTest::testFindByUrl()
 
     QCOMPARE( mime->name(), QString::fromLatin1( "application/octet-stream" ) ); // HTTP can't know before downloading
 
+    mime = KMimeType::findByUrl(KUrl("http://foo/s0/"));
+    QCOMPARE( mime->name(), QString::fromLatin1( "application/octet-stream" ) ); // HTTP can't know before downloading
+
     if ( !KProtocolInfo::isKnownProtocol(KUrl("man:/")) )
         QSKIP( "man protocol not installed", SkipSingle );
 
@@ -613,7 +616,7 @@ static bool offerListHasService( const KService::List& offers,
 {
     bool found = false;
     KService::List::const_iterator it = offers.begin();
-    for ( ; it != offers.end() ; it++ )
+    for ( ; it != offers.end() ; ++it )
     {
         if ( (*it)->entryPath() == entryPath ) {
             if( found ) { // should be there only once
