@@ -169,7 +169,7 @@ void KAcceleratorManagerPrivate::manage(QWidget *widget)
 {
     if (!widget)
     {
-        kDebug(131) << "null pointer given to manage";
+        kDebug(240) << "null pointer given to manage";
         return;
     }
 
@@ -243,9 +243,11 @@ void KAcceleratorManagerPrivate::calculateAccelerators(Item *item, QString &used
                 continue;
             }
         }
+
         // we possibly reserved an accel, but we won't set it as it looks silly
-        if ( qobject_cast<QGroupBox*>( it->m_widget ) )
-             continue;
+        QGroupBox *groupBox = qobject_cast<QGroupBox*>(it->m_widget);
+        if (groupBox && !groupBox->isCheckable())
+            continue;
 
         int tprop = it->m_widget->metaObject()->indexOfProperty("text");
         if (tprop != -1)  {
@@ -338,6 +340,11 @@ void KAcceleratorManagerPrivate::manageWidget(QWidget *w, Item *item)
       qobject_cast<QAbstractSpinBox*>(w) || w->inherits( "KMultiTabBar" ) )
       return;
 
+  if ( w->inherits("KUrlRequester") ) {
+    traverseChildren(w, item);
+    return;
+  }
+
   // now treat 'ordinary' widgets
   QLabel *label =  qobject_cast<QLabel*>(w);
   if ( label  ) {
@@ -358,7 +365,7 @@ void KAcceleratorManagerPrivate::manageWidget(QWidget *w, Item *item)
     int tprop = w->metaObject()->indexOfProperty("text");
     if (tprop != -1)  {
         QMetaProperty p = w->metaObject()->property( tprop );
-        if ( p.isValid() )
+        if ( p.isValid() && p.isWritable() )
             variant = p.read (w);
         else
             tprop = -1;
@@ -368,7 +375,7 @@ void KAcceleratorManagerPrivate::manageWidget(QWidget *w, Item *item)
         tprop = w->metaObject()->indexOfProperty("title");
         if (tprop != -1)  {
             QMetaProperty p = w->metaObject()->property( tprop );
-            if ( p.isValid() )
+            if ( p.isValid() && p.isWritable() )
                 variant = p.read (w);
         }
     }
@@ -386,9 +393,17 @@ void KAcceleratorManagerPrivate::manageWidget(QWidget *w, Item *item)
         if (qobject_cast<QPushButton*>(w) || qobject_cast<QCheckBox*>(w) || qobject_cast<QRadioButton*>(w) || qobject_cast<QLabel*>(w))
             weight = KAccelManagerAlgorithm::ACTION_ELEMENT_WEIGHT;
 
-        // don't put weight on group boxes, as usually the contents are more important
-        if (qobject_cast<QGroupBox*>(w))
-            weight = KAccelManagerAlgorithm::GROUP_BOX_WEIGHT;
+        // don't put weight on non-checkable group boxes,
+        // as usually the contents are more important
+        QGroupBox *groupBox = qobject_cast<QGroupBox*>(w);
+        if (groupBox)
+        {
+            if (groupBox->isCheckable())
+                weight = KAccelManagerAlgorithm::CHECKABLE_GROUP_BOX_WEIGHT;
+            else
+                weight = KAccelManagerAlgorithm::GROUP_BOX_WEIGHT;
+        }
+
         i->m_content = KAccelString(content, weight);
         item->addChild(i);
     }
@@ -406,7 +421,7 @@ void KAcceleratorManagerPrivate::manageTabBar(QTabBar *bar, Item *item)
   {
     QMainWindow* mainWindow = qobject_cast<QMainWindow*>(parentWidget);
     // TODO: find better hints that this is a QTabBar for QDockWidgets
-    if( mainWindow ) // && (mainWindow->layout()->indexOf(bar) != -1)) QMainWindowLayout lacks proper support 
+    if( mainWindow ) // && (mainWindow->layout()->indexOf(bar) != -1)) QMainWindowLayout lacks proper support
       return;
   }
 
@@ -611,7 +626,7 @@ void KAccelString::calculateWeights(int initialWeight)
     // try to preserve the wanted accelarators
     if ((int)pos == accel()) {
         weight += KAccelManagerAlgorithm::WANTED_ACCEL_EXTRA_WEIGHT;
-        // kDebug(131) << "wanted " << m_pureText << " " << KAcceleratorManagerPrivate::standardName(m_origText);
+        // kDebug(240) << "wanted " << m_pureText << " " << KAcceleratorManagerPrivate::standardName(m_origText);
         if (KAcceleratorManagerPrivate::standardName(m_origText))  {
             weight += KAccelManagerAlgorithm::STANDARD_ACCEL;
         }
@@ -710,7 +725,7 @@ void KAccelString::dump()
 
  TODO:
 
- * The result is always correct, but not neccesarily optimal. Perhaps
+ * The result is always correct, but not necessarily optimal. Perhaps
    it would be a good idea to add another algorithm with higher complexity
    that gets used when this one fails, i.e. leaves widgets without
    accelerators.
@@ -723,7 +738,7 @@ void KAccelManagerAlgorithm::findAccelerators(KAccelStringList &result, QString
 {
   KAccelStringList accel_strings = result;
 
-  // initally remove all accelerators
+  // initially remove all accelerators
   for (KAccelStringList::Iterator it = result.begin(); it != result.end(); ++it) {
     (*it).setAccel(-1);
   }
@@ -890,7 +905,7 @@ void QWidgetStackAccelManager::currentChanged(int child)
 {
     if (child < 0 || child >= static_cast<QStackedWidget*>(parent())->count())
     {
-        kDebug(131) << "invalid index provided";
+        // NOTE: QStackedWidget emits currentChanged(-1) when it is emptied
         return;
     }
 
