@@ -40,7 +40,9 @@
 #include <kdebug.h>
 #include <klockfile.h>
 #include <ksavefile.h>
+#ifndef _WIN32_WCE
 #include <ksvgrenderer.h>
+#endif
 #include <kdefakes.h>
 
 #include <config.h>
@@ -1467,6 +1469,7 @@ QPixmap KPixmapCache::loadFromFile(const QString& filename)
     return pix;
 }
 
+#ifndef _WIN32_WCE
 QPixmap KPixmapCache::loadFromSvg(const QString& filename, const QSize& size)
 {
     QFileInfo fi(filename);
@@ -1481,7 +1484,7 @@ QPixmap KPixmapCache::loadFromSvg(const QString& filename, const QSize& size)
     QString key = QString("file:%1_%2_%3").arg(filename).arg(size.width()).arg(size.height());
     if (!find(key, pix)) {
         // It wasn't in the cache, so load it...
-        KSvgRenderer svg;
+        QSvgRenderer svg;
         if (!svg.load(filename)) {
             return pix;  // null pixmap
         } else {
@@ -1499,4 +1502,5 @@ QPixmap KPixmapCache::loadFromSvg(const QString& filename, const QSize& size)
 
     return pix;
 }
+#endif
 
