@@ -203,8 +203,8 @@ void KPushButton::init( const KGuiItem &item )
 
     setWhatsThis(item.whatsThis());
 
-    connect( KGlobalSettings::self(), SIGNAL( settingsChanged(int) ),
-             SLOT( slotSettingsChanged(int) ) );
+    connect( KGlobalSettings::self(), SIGNAL(settingsChanged(int)),
+             SLOT(slotSettingsChanged(int)) );
 }
 
 bool KPushButton::isDragEnabled() const
@@ -347,6 +347,10 @@ void KPushButton::setAuthAction(KAuth::Action *action)
 
     if (action != 0) {
         d->authAction = action;
+
+        // Set the parent widget
+        d->authAction->setParentWidget(this);
+
         connect(d->authAction->watcher(), SIGNAL(statusChanged(int)),
                 this, SLOT(authStatusChanged(int)));
         d->authStatusChanged(d->authAction->status());
@@ -369,7 +373,7 @@ void KPushButton::paintEvent( QPaintEvent * )
     QStylePainter p(this);
     QStyleOptionButton option;
     initStyleOption(&option);
-    
+
     if (d->delayedMenu)
         option.features |= QStyleOptionButton::HasMenu;
 
