@@ -92,9 +92,24 @@ class KToolBar::Private
     Private(KToolBar *qq)
       : q(qq),
         isMainToolBar(false),
+#ifndef KDE_NO_DEPRECATED
         enableContext(true),
+#endif
         unlockedMovable(true),
-        xmlguiClient(0),
+        contextOrient(0),
+        contextMode(0),
+        contextSize(0),
+        contextButtonTitle(0),
+        contextShowText(0),
+        contextButtonAction(0),
+        contextTop(0),
+        contextLeft(0),
+        contextRight(0),
+        contextBottom(0),
+        contextIcons(0),
+        contextTextRight(0),
+        contextText(0),
+        contextTextUnder(0),
         contextLockAction(0),
         dropIndicatorAction(0),
         context(0),
@@ -107,6 +122,7 @@ class KToolBar::Private
     void slotContextAboutToHide();
     void slotContextLeft();
     void slotContextRight();
+    void slotContextShowText();
     void slotContextTop();
     void slotContextBottom();
     void slotContextIcons();
@@ -118,29 +134,36 @@ class KToolBar::Private
 
     void init(bool readConfig = true, bool isMainToolBar = false);
     QString getPositionAsString() const;
-    KMenu *contextMenu();
+    KMenu *contextMenu(const QPoint &globalPos);
     void setLocked(bool locked);
     void adjustSeparatorVisibility();
     void loadKDESettings();
     void applyCurrentSettings();
 
+    QAction *findAction(const QString &actionName, KXMLGUIClient **client = 0) const;
+
     static Qt::ToolButtonStyle toolButtonStyleFromString(const QString& style);
     static QString toolButtonStyleToString(Qt::ToolButtonStyle);
     static Qt::ToolBarArea positionFromString(const QString& position);
 
     KToolBar *q;
     bool isMainToolBar : 1;
+#ifndef KDE_NO_DEPRECATED
     bool enableContext : 1;
+#endif
     bool unlockedMovable : 1;
     static bool s_editable;
     static bool s_locked;
 
-    KXMLGUIClient *xmlguiClient;
+    QSet<KXMLGUIClient *> xmlguiClients;
 
     QMenu* contextOrient;
     QMenu* contextMode;
     QMenu* contextSize;
 
+    QAction* contextButtonTitle;
+    QAction* contextShowText;
+    QAction* contextButtonAction;
     QAction* contextTop;
     QAction* contextLeft;
     QAction* contextRight;
@@ -219,7 +242,7 @@ void KToolBar::Private::init(bool readConfig, bool _isMainToolBar)
     // Get notified when settings change
     connect(q, SIGNAL(allowedAreasChanged(Qt::ToolBarAreas)),
              q->mainWindow(), SLOT(setSettingsDirty()));
-    connect(q, SIGNAL(iconSizeChanged(const QSize&)),
+    connect(q, SIGNAL(iconSizeChanged(QSize)),
              q->mainWindow(), SLOT(setSettingsDirty()));
     connect(q, SIGNAL(toolButtonStyleChanged(Qt::ToolButtonStyle)),
              q->mainWindow(), SLOT(setSettingsDirty()));
@@ -261,11 +284,15 @@ QString KToolBar::Private::getPositionAsString() const
   }
 }
 
-KMenu *KToolBar::Private::contextMenu()
+KMenu *KToolBar::Private::contextMenu(const QPoint &globalPos)
 {
   if (!context) {
     context = new KMenu(q);
-    context->addTitle(i18n("Toolbar Settings"));
+
+    contextButtonTitle = context->addTitle(i18nc("@title:menu", "Show Text"));
+    contextShowText = context->addAction(QString(), q, SLOT(slotContextShowText()));
+
+    context->addTitle(i18nc("@title:menu", "Toolbar Settings"));
 
     contextOrient = new KMenu(i18n("Orientation"), context);
 
@@ -372,6 +399,13 @@ KMenu *KToolBar::Private::contextMenu()
     connect(context, SIGNAL(aboutToShow()), q, SLOT(slotContextAboutToShow()));
   }
 
+  contextButtonAction = q->actionAt(q->mapFromGlobal(globalPos));
+  if (contextButtonAction) {
+      contextShowText->setText(contextButtonAction->text());
+      contextShowText->setIcon(contextButtonAction->icon());
+      contextShowText->setCheckable(true);
+  }
+
   contextOrient->menuAction()->setVisible(!q->toolBarsLocked());
   // Unplugging a submenu from abouttohide leads to the popupmenu floating around
   // So better simply call that code from after exec() returns (DF)
@@ -508,6 +542,20 @@ void KToolBar::Private::applyCurrentSettings()
         kmw->setSettingsDirty();
 }
 
+QAction *KToolBar::Private::findAction(const QString &actionName, KXMLGUIClient **clientOut) const
+{
+    foreach (KXMLGUIClient* client, xmlguiClients) {
+        QAction* action = client->actionCollection()->action(actionName);
+        if (action) {
+            if (clientOut) {
+                *clientOut = client;
+            }
+            return action;
+        }
+    }
+    return 0;
+}
+
 void KToolBar::Private::slotContextAboutToShow()
 {
   /**
@@ -523,9 +571,7 @@ void KToolBar::Private::slotContextAboutToShow()
   // try to find "configure toolbars" action
   QAction *configureAction = 0;
   const char* actionName = KStandardAction::name(KStandardAction::ConfigureToolbars);
-  if (xmlguiClient) {
-    configureAction = xmlguiClient->actionCollection()->action(actionName);
-  }
+  configureAction = findAction(actionName);
 
   if (!configureAction && kmw) {
     configureAction = kmw->actionCollection()->action(actionName);
@@ -588,6 +634,15 @@ void KToolBar::Private::slotContextAboutToShow()
       contextTop->setChecked(true);
       break;
   }
+
+  const bool showButtonSettings = contextButtonAction
+                                  && !contextShowText->text().isEmpty()
+                                  && contextTextRight->isChecked();
+  contextButtonTitle->setVisible(showButtonSettings);
+  contextShowText->setVisible(showButtonSettings);
+  if (showButtonSettings) {
+    contextShowText->setChecked(contextButtonAction->priority() >= QAction::NormalPriority);
+  }
 }
 
 void KToolBar::Private::slotContextAboutToHide()
@@ -604,9 +659,7 @@ void KToolBar::Private::slotContextAboutToHide()
   // Unplug the configure toolbars action too, since it's afterwards anyway
   QAction *configureAction = 0;
   const char* actionName = KStandardAction::name(KStandardAction::ConfigureToolbars);
-  if (xmlguiClient) {
-    configureAction = xmlguiClient->actionCollection()->action(actionName);
-  }
+  configureAction = findAction(actionName);
 
   if (!configureAction && kmw) {
     configureAction = kmw->actionCollection()->action(actionName);
@@ -629,6 +682,37 @@ void KToolBar::Private::slotContextRight()
   q->mainWindow()->addToolBar(Qt::RightToolBarArea, q);
 }
 
+void KToolBar::Private::slotContextShowText()
+{
+    Q_ASSERT(contextButtonAction);
+    const QAction::Priority priority = contextShowText->isChecked()
+                                       ? QAction::NormalPriority : QAction::LowPriority;
+    contextButtonAction->setPriority(priority);
+
+    // Find to which xml file and componentData the action belongs to
+    KComponentData componentData;
+    QString filename;
+    KXMLGUIClient *client;
+    if (findAction(contextButtonAction->objectName(), &client)) {
+        componentData = client->componentData();
+        filename = client->xmlFile();
+    }
+    if (filename.isEmpty()) {
+        componentData = KGlobal::mainComponent();
+        filename = componentData.componentName() + "ui.rc";
+    }
+
+    // Save the priority state of the action
+    const QString configFile = KXMLGUIFactory::readConfigFile(filename, componentData);
+
+    QDomDocument document;
+    document.setContent(configFile);
+    QDomElement elem = KXMLGUIFactory::actionPropertiesElement(document);
+    QDomElement actionElem = KXMLGUIFactory::findActionByName(elem, contextButtonAction->objectName(), true);
+    actionElem.setAttribute("priority", priority);
+    KXMLGUIFactory::saveConfigFile(document, filename, componentData);
+}
+
 void KToolBar::Private::slotContextTop()
 {
   q->mainWindow()->addToolBar(Qt::TopToolBarArea, q);
@@ -727,15 +811,19 @@ KToolBar::~KToolBar()
   delete d;
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KToolBar::setContextMenuEnabled(bool enable)
 {
   d->enableContext = enable;
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 bool KToolBar::contextMenuEnabled() const
 {
   return d->enableContext;
 }
+#endif
 
 void KToolBar::saveSettings(KConfigGroup &cg)
 {
@@ -763,16 +851,31 @@ void KToolBar::saveSettings(KConfigGroup &cg)
     }
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KToolBar::setXMLGUIClient(KXMLGUIClient *client)
 {
-    d->xmlguiClient = client;
+    d->xmlguiClients.clear();
+    d->xmlguiClients << client;
+}
+#endif
+
+void KToolBar::addXMLGUIClient( KXMLGUIClient *client )
+{
+    d->xmlguiClients << client;
+}
+
+void KToolBar::removeXMLGUIClient( KXMLGUIClient *client )
+{
+    d->xmlguiClients.remove(client);
 }
 
 void KToolBar::contextMenuEvent(QContextMenuEvent* event)
 {
+#ifndef KDE_NO_DEPRECATED
     if (mainWindow() && d->enableContext) {
         QPointer<KToolBar> guard(this);
-        d->contextMenu()->exec(event->globalPos());
+        const QPoint globalPos = event->globalPos();
+        d->contextMenu(globalPos)->exec(globalPos);
 
         // "Configure Toolbars" recreates toolbars, so we might not exist anymore.
         if (guard) {
@@ -780,6 +883,7 @@ void KToolBar::contextMenuEvent(QContextMenuEvent* event)
         }
         return;
     }
+#endif
 
     QToolBar::contextMenuEvent(event);
 }
@@ -1282,7 +1386,9 @@ void KToolBar::actionEvent(QActionEvent * event)
         foreach (QWidget* child, widget->findChildren<QWidget*>())
             child->installEventFilter(this);
         // Center widgets that do not have any use for more space. See bug 165274
-        if (!(widget->sizePolicy().horizontalPolicy() & QSizePolicy::GrowFlag)) {
+        if (!(widget->sizePolicy().horizontalPolicy() & QSizePolicy::GrowFlag)
+            // ... but do not center when using text besides icon in vertical toolbar. See bug 243196
+            && !(orientation() == Qt::Vertical && toolButtonStyle() == Qt::ToolButtonTextBesideIcon)) {
             const int index = layout()->indexOf(widget);
             if (index != -1) {
                 layout()->itemAt(index)->setAlignment(Qt::AlignJustify);
