@@ -117,32 +117,6 @@ static HBITMAP QPixmapMask2HBitmap(const QPixmap &pix)
     return CreateBitmap( w, h, 1, 1, bits );
 }
 
-static HICON QPixmap2HIcon(const QPixmap &pix)
-{
-    if ( pix.isNull() )
-        return 0;
-
-    ICONINFO ii;
-    ii.fIcon    = true;
-    ii.hbmMask  = QPixmapMask2HBitmap( pix );
-    ii.hbmColor = pix.toWinHBITMAP( QPixmap::PremultipliedAlpha );
-    ii.xHotspot = 0;
-    ii.yHotspot = 0;
-    HICON result = CreateIconIndirect( &ii );
-
-    DeleteObject( ii.hbmMask );
-    DeleteObject( ii.hbmColor );
-
-    return result;
-}
-
-static QPixmap HIcon2QPixmap( HICON hIcon )
-{
-    QPixmap pix  = QPixmap::fromWinHICON( hIcon );
-
-    return pix;
-}
-
 KWindowSystemPrivate::KWindowSystemPrivate(int what) : QWidget(0),activated(false)
 {
     //i think there is no difference in windows we always load everything
@@ -219,9 +193,12 @@ bool KWindowSystemPrivate::winEvent ( MSG * message, long * result )
             KWindowSystem::s_d_func()->windowRemoved(reinterpret_cast<WId>(message->lParam));
             break;
           case HSHELL_WINDOWACTIVATED:
+#ifndef _WIN32_WCE
           case HSHELL_RUDEAPPACTIVATED:
+#endif
             KWindowSystem::s_d_func()->windowActivated(reinterpret_cast<WId>(message->lParam));
             break;
+#ifndef _WIN32_WCE
           case HSHELL_GETMINRECT:
             KWindowSystem::s_d_func()->windowStateChanged(reinterpret_cast<WId>(message->lParam));
             break;
@@ -231,6 +208,7 @@ bool KWindowSystemPrivate::winEvent ( MSG * message, long * result )
           case HSHELL_FLASH:
             KWindowSystem::s_d_func()->windowFlash(reinterpret_cast<WId>(message->lParam));
             break;
+#endif
         }
     }
     return QWidget::winEvent(message,result);
@@ -238,13 +216,13 @@ bool KWindowSystemPrivate::winEvent ( MSG * message, long * result )
 
 bool CALLBACK KWindowSystemPrivate::EnumWindProc(WId hWnd, LPARAM lparam)
 {
-    QByteArray windowText = QByteArray ( GetWindowTextLength(hWnd)+1, 0 ) ;
-    GetWindowTextA(hWnd, windowText.data(), windowText.size());
+    QByteArray windowText = QByteArray ( (GetWindowTextLength(hWnd)+1) * sizeof(wchar_t), 0 ) ;
+    GetWindowTextW(hWnd, (LPWSTR)windowText.data(), windowText.size());
 	DWORD ex_style = GetWindowExStyle(hWnd);
     KWindowSystemPrivate *p = KWindowSystem::s_d_func();
 
     QString add;
-    if( !QString(windowText).trimmed().isEmpty() && IsWindowVisible( hWnd ) && !(ex_style&WS_EX_TOOLWINDOW)
+    if( !QString::fromWCharArray((wchar_t*)windowText.data()).trimmed().isEmpty() && IsWindowVisible( hWnd ) && !(ex_style&WS_EX_TOOLWINDOW)
        && !GetParent(hWnd) && !GetWindow(hWnd,GW_OWNER) && !p->winInfos.contains(hWnd) ) {
 
 //        kDebug()<<"Adding window to windowList " << add + QString(windowText).trimmed();
@@ -260,30 +238,34 @@ bool CALLBACK KWindowSystemPrivate::EnumWindProc(WId hWnd, LPARAM lparam)
 
 void KWindowSystemPrivate::readWindowInfo ( WId hWnd , InternalWindowInfo *winfo)
 {
-    QByteArray windowText = QByteArray ( GetWindowTextLength(hWnd)+1, 0 ) ;
-    GetWindowTextA(hWnd, windowText.data(), windowText.size());
+    QByteArray windowText = QByteArray ( (GetWindowTextLength(hWnd)+1) * sizeof(wchar_t), 0 ) ;
+    GetWindowTextW(hWnd, (LPWSTR)windowText.data(), windowText.size());
      //maybe use SendMessageTimout here?
     QPixmap smallIcon;
     HICON hSmallIcon = (HICON)SendMessage(hWnd, WM_GETICON, ICON_SMALL, 0);
     //if(!hSmallIcon) hSmallIcon = (HICON)SendMessage(hWnd, WM_GETICON, ICON_SMALL2, 0);
     if(!hSmallIcon) hSmallIcon = (HICON)SendMessage(hWnd, WM_GETICON, ICON_BIG, 0);
+#ifndef _WIN32_WCE
     if(!hSmallIcon) hSmallIcon = (HICON)GetClassLong(hWnd, GCL_HICONSM);
     if(!hSmallIcon) hSmallIcon = (HICON)GetClassLong(hWnd, GCL_HICON);
+#endif
     if(!hSmallIcon) hSmallIcon = (HICON)SendMessage(hWnd, WM_QUERYDRAGICON, 0, 0);
-    if(hSmallIcon)  smallIcon  = HIcon2QPixmap(hSmallIcon);
+    if(hSmallIcon)  smallIcon  = QPixmap::fromWinHICON(hSmallIcon);
 
     QPixmap bigIcon;
     HICON hBigIcon = (HICON)SendMessage(hWnd, WM_GETICON, ICON_BIG, 0);
     //if(!hBigIcon) hBigIcon = (HICON)SendMessage(hWnd, WM_GETICON, ICON_SMALL2, 0);
     if(!hBigIcon) hBigIcon = (HICON)SendMessage(hWnd, WM_GETICON, ICON_SMALL, 0);
+#ifndef _WIN32_WCE
     if(!hBigIcon) hBigIcon = (HICON)GetClassLong(hWnd, GCL_HICON);
     if(!hBigIcon) hBigIcon = (HICON)GetClassLong(hWnd, GCL_HICONSM);
+#endif
     if(!hBigIcon) hBigIcon = (HICON)SendMessage(hWnd, WM_QUERYDRAGICON, 0, 0);
-    if(hBigIcon)  bigIcon  = HIcon2QPixmap(hBigIcon);
+    if(hBigIcon)  bigIcon  = QPixmap::fromWinHICON(hBigIcon);
 
     winfo->bigIcon    = bigIcon;
     winfo->smallIcon  = smallIcon;
-    winfo->windowName = QString(windowText).trimmed();
+    winfo->windowName = QString::fromWCharArray((wchar_t*)windowText.data()).trimmed();
 }
 
 
@@ -307,6 +289,10 @@ void KWindowSystemPrivate::windowRemoved   (WId wid)
 void KWindowSystemPrivate::windowActivated (WId wid)
 {
 //     kDebug() << "window activated!";
+    if (!wid) {
+        return;
+    }
+
     KWindowSystem::s_d_func()->reloadStackList();
     emit KWindowSystem::self()->activeWindowChanged(wid);
     emit KWindowSystem::self()->stackingOrderChanged();
@@ -423,17 +409,31 @@ void KWindowSystem::forceActiveWindow( WId win, long time )
     // But the mouse cursor still acts as if the widgets were there (e.g. button clicking works),
     // which indicates the issue is at the window/backingstore level.
     // This is probably a side effect of bypassing Qt's internal window state handling.
+#ifndef _WIN32_WCE
     if ( IsIconic( win ) /*|| !IsWindowVisible( win ) */) {
         // Do not activate the window as we restore it,
         // otherwise the window appears see-through (contents not updated).
         ShowWindow( win, SW_SHOWNOACTIVATE );
     }
+#endif
     // Puts the window in front and activates it.
-    SetForegroundWindow( win );
+    //to bring a window to the front while the user is active in a different apllication we
+    //have to atach our self to the current active window
+    HWND hwndActiveWin = GetForegroundWindow();
+    int  idActive      = GetWindowThreadProcessId(hwndActiveWin, NULL);
+    if ( AttachThreadInput(GetCurrentThreadId(), idActive, TRUE) )
+    {
+        SetForegroundWindow( win );
+        SetFocus( win ); 
+        AttachThreadInput(GetCurrentThreadId(), idActive, FALSE);
+    }
+    
 }
 
 void KWindowSystem::demandAttention( WId win, bool set )
 {
+// One can not flash a windows in wince
+#ifndef _WIN32_WCE
     FLASHWINFO fi;
     fi.cbSize = sizeof( FLASHWINFO );
     fi.hwnd = win;
@@ -442,6 +442,7 @@ void KWindowSystem::demandAttention( WId win, bool set )
     fi.dwTimeout = 0;
 
     FlashWindowEx( &fi );
+#endif
 }
 
 
@@ -462,7 +463,8 @@ QPixmap KWindowSystem::icon( WId win, int width, int height, bool scale )
         if( width < 24 || height < 24 )
             size = ICON_SMALL;
         HICON hIcon = (HICON)SendMessage( win, WM_GETICON, size, 0);
-        pm = HIcon2QPixmap( hIcon );
+        if(hIcon != NULL)
+            pm = QPixmap::fromWinHICON( hIcon );
     }
     if( scale )
         pm = pm.scaled( width, height );
@@ -485,8 +487,8 @@ void KWindowSystem::setIcons( WId win, const QPixmap& icon, const QPixmap& miniI
         s_d->winInfos[win].bigIcon   = icon;
     }
 
-    HICON hIconBig = QPixmap2HIcon(icon);
-    HICON hIconSmall = QPixmap2HIcon(miniIcon);
+    HICON hIconBig = icon.toWinHICON();
+    HICON hIconSmall = miniIcon.toWinHICON();
 
     hIconBig = (HICON)SendMessage( win, WM_SETICON, ICON_BIG,   (LPARAM)hIconBig );
     hIconSmall = (HICON)SendMessage( win, WM_SETICON, ICON_SMALL, (LPARAM)hIconSmall );
@@ -496,11 +498,13 @@ void KWindowSystem::setIcons( WId win, const QPixmap& icon, const QPixmap& miniI
 void KWindowSystem::setState( WId win, unsigned long state )
 {
     bool got = false;
+#ifndef _WIN32_WCE
     if (state & NET::SkipTaskbar) {
         got = true;
         LONG_PTR lp = GetWindowLongPtr(win, GWL_EXSTYLE);
         SetWindowLongPtr(win, GWL_EXSTYLE, lp | WS_EX_TOOLWINDOW);
     }
+#endif
     if (state & NET::KeepAbove) {
         got = true;
         SetWindowPos(win, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
@@ -521,11 +525,13 @@ void KWindowSystem::clearState( WId win, unsigned long state )
 {
     bool got = false;
 
+#ifndef _WIN32_WCE
     if (state & NET::SkipTaskbar) {
         got = true;
         LONG_PTR lp = GetWindowLongPtr(win, GWL_EXSTYLE);
         SetWindowLongPtr(win, GWL_EXSTYLE, lp & ~WS_EX_TOOLWINDOW);
     }
+#endif
     if (state & NET::KeepAbove) {
         got = true;
         //lets hope this remove the topmost
@@ -553,7 +559,16 @@ void KWindowSystem::unminimizeWindow( WId win, bool animation )
 
 void KWindowSystem::raiseWindow( WId win )
 {
-    SetWindowPos( win, HWND_TOP, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE ); // mhhh?
+
+    //to bring a window to the front while the user is active in a different apllication we
+    //have to atach our self to the current active window
+    HWND hwndActiveWin = GetForegroundWindow();
+    int  idActive      = GetWindowThreadProcessId(hwndActiveWin, NULL);
+    if ( AttachThreadInput(GetCurrentThreadId(), idActive, TRUE) )
+    {
+        SetForegroundWindow( win );
+        AttachThreadInput(GetCurrentThreadId(), idActive, FALSE);
+    }
 }
 
 void KWindowSystem::lowerWindow( WId win )
@@ -613,9 +628,9 @@ void KWindowSystem::connectNotify( const char* signal )
         what = INFO_WINDOWS;
     else if( QLatin1String( signal ) == SIGNAL(strutChanged()))
         what = INFO_WINDOWS;
-    else if( QLatin1String( signal ) == QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,const unsigned long*))).constData())
+    else if( QLatin1String( signal ) == QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,const ulong*))).constData())
         what = INFO_WINDOWS;
-    else if( QLatin1String( signal ) ==  QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,unsigned int))).constData())
+    else if( QLatin1String( signal ) ==  QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,uint))).constData())
         what = INFO_WINDOWS;
     else if( QLatin1String( signal ) ==  QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId))).constData())
         what = INFO_WINDOWS;
@@ -682,7 +697,15 @@ bool KWindowSystem::hasWId(WId w)
 
 void KWindowSystem::allowExternalProcessWindowActivation( int pid )
 {
+#ifndef _WIN32_WCE
     AllowSetForegroundWindow( pid == -1 ? ASFW_ANY : pid );
+#endif
+}
+
+void KWindowSystem::setBlockingCompositing( WId window, bool active )
+{
+    //TODO
+    kDebug() << "setBlockingCompositing( WId window, bool active ) isn't yet implemented!";
 }
 
 #include "kwindowsystem.moc"
