@@ -89,9 +89,9 @@ public:
         ShortcutOption shortcutOption = KXMLGUIFactoryPrivate::SetActiveShortcut );
 
     QDomDocument shortcutSchemeDoc(KXMLGUIClient *client);
-    void applyShortcutScheme(KXMLGUIClient *client, QDomDocument scheme);
-    void refreshActionProperties(KXMLGUIClient *client, QDomDocument doc);
-    void saveDefaultActionProperties(KXMLGUIClient *client);
+    void applyShortcutScheme(KXMLGUIClient *client, const QList<QAction*>& actions, const QDomDocument& scheme);
+    void refreshActionProperties(KXMLGUIClient *client, const QList<QAction*>& actions, const QDomDocument& doc);
+    void saveDefaultActionProperties(const QList<QAction*>& actions);
 
     ContainerNode *m_rootNode;
 
@@ -116,13 +116,13 @@ public:
 
 QString KXMLGUIFactory::readConfigFile( const QString &filename, const KComponentData &_componentData )
 {
-    KComponentData componentData = _componentData.isValid() ? _componentData : KGlobal::mainComponent();
     QString xml_file;
 
     if (!QDir::isRelativePath(filename))
         xml_file = filename;
     else
     {
+        KComponentData componentData = _componentData.isValid() ? _componentData : KGlobal::mainComponent();
         xml_file = KStandardDirs::locate("data", componentData.componentName() + '/' + filename);
         if ( !QFile::exists( xml_file ) )
           xml_file = KStandardDirs::locate( "data", filename );
@@ -201,6 +201,9 @@ KXMLGUIFactory::KXMLGUIFactory( KXMLGUIBuilder *builder, QObject *parent )
 
 KXMLGUIFactory::~KXMLGUIFactory()
 {
+    foreach (KXMLGUIClient *client, d->m_clients) {
+        client->setFactory ( 0L );
+    }
     delete d;
 }
 
@@ -261,8 +264,9 @@ void KXMLGUIFactory::addClient( KXMLGUIClient *client )
     }
 
     // load shortcut schemes, user-defined shortcuts and other action properties
-    d->saveDefaultActionProperties(client);
-    d->refreshActionProperties(client, doc);
+    d->saveDefaultActionProperties(client->actionCollection()->actions());
+    if (!doc.isNull())
+        d->refreshActionProperties(client, client->actionCollection()->actions(), doc);
 
     BuildHelper( *d, d->m_rootNode ).build( docElement );
 
@@ -315,60 +319,74 @@ void KXMLGUIFactory::refreshActionProperties()
             client->reloadXML();
             doc = client->domDocument();
         }
-        d->refreshActionProperties(client, doc);
+        d->refreshActionProperties(client, client->actionCollection()->actions(), doc);
     }
     d->guiClient = 0;
 }
 
-void KXMLGUIFactoryPrivate::refreshActionProperties(KXMLGUIClient *client, QDomDocument doc)
+static QString currentShortcutScheme()
+{
+    const KConfigGroup cg = KGlobal::config()->group("Shortcut Schemes");
+    return cg.readEntry("Current Scheme", "Default");
+}
+
+// Find the right ActionProperties element, otherwise return null element
+static QDomElement findActionPropertiesElement(const QDomDocument& doc)
+{
+    const QLatin1String tagActionProp("ActionProperties");
+    const QString schemeName = currentShortcutScheme();
+    QDomElement e = doc.documentElement().firstChildElement();
+    for( ; !e.isNull(); e = e.nextSiblingElement() ) {
+        if (QString::compare(e.tagName(), tagActionProp, Qt::CaseInsensitive) == 0
+            && (e.attribute("scheme", "Default") == schemeName) ) {
+            return e;
+        }
+    }
+    return QDomElement();
+}
+
+void KXMLGUIFactoryPrivate::refreshActionProperties(KXMLGUIClient *client, const QList<QAction*>& actions, const QDomDocument& doc)
 {
     // try to find and apply shortcuts schemes
     QDomDocument scheme = shortcutSchemeDoc(client);
-    applyShortcutScheme(client, scheme);
+    applyShortcutScheme(client, actions, scheme);
 
     // try to find and apply user-defined shortcuts
-    QDomElement actionPropElement = KXMLGUIFactory::actionPropertiesElement(doc);
-    if (actionPropElement.childNodes().isEmpty())
-        doc.documentElement().removeChild(actionPropElement);
-
+    const QDomElement actionPropElement = findActionPropertiesElement(doc);
     if ( !actionPropElement.isNull() )
         applyActionProperties( actionPropElement );
 }
 
-void KXMLGUIFactoryPrivate::saveDefaultActionProperties(KXMLGUIClient *client)
+void KXMLGUIFactoryPrivate::saveDefaultActionProperties(const QList<QAction *>& actions)
 {
     // This method is called every time the user activated a new
     // kxmlguiclient. We only want to execute the following code only once in
     // the lifetime of an action.
-    foreach (QAction *action, client->actionCollection()->actions())
-    {
-        // Skip actions we have seen already.
-        if (action->property("_k_DefaultShortcut").isValid()) continue;
+    foreach (QAction *action, actions) {
+        // Skip NULL actions or those we have seen already.
+        if (!action || action->property("_k_DefaultShortcut").isValid()) continue;
 
-        if (KAction* kaction = qobject_cast<KAction*>(action))
-        {
+        if (KAction* kaction = qobject_cast<KAction*>(action)) {
             // Check if the default shortcut is set
             KShortcut defaultShortcut = kaction->shortcut(KAction::DefaultShortcut);
             KShortcut activeShortcut  = kaction->shortcut(KAction::ActiveShortcut);
+            //kDebug() << kaction->objectName() << "default=" << defaultShortcut.toString() << "active=" << activeShortcut.toString();
 
             // Check if we have an empty default shortcut and an non empty
             // custom shortcut. This should only happen if a developer called
             // QAction::setShortcut on an KAction. Print out a warning and
             // correct the mistake
-            if ((!activeShortcut.isEmpty()) && defaultShortcut.isEmpty())
-            {
-                kError() << "Shortcut for KAction " << kaction->objectName() << kaction->text() << "set with QShortcut::setShortcut()! See KAction documentation.";
+            if ((!activeShortcut.isEmpty()) && defaultShortcut.isEmpty()) {
+                kError(240) << "Shortcut for KAction " << kaction->objectName() << kaction->text() << "set with QShortcut::setShortcut()! See KAction documentation.";
                 kaction->setProperty("_k_DefaultShortcut", activeShortcut);
-            }
-            else
-            {
+            } else {
                 kaction->setProperty("_k_DefaultShortcut", defaultShortcut);
             }
         }
         else
         {
             // A QAction used with KXMLGUI? Set our property and ignore it.
-            kError() << "Attempt to use QAction" << action->objectName() << "with KXMLGUIFactory!";
+            kError(240) << "Attempt to use QAction" << action->objectName() << "with KXMLGUIFactory!";
             action->setProperty("_k_DefaultShortcut", KShortcut());
         }
 
@@ -384,6 +402,11 @@ void KXMLGUIFactory::changeShortcutScheme(const QString &scheme)
     refreshActionProperties();
 }
 
+void KXMLGUIFactory::forgetClient( KXMLGUIClient *client )
+{
+    d->m_clients.removeAll( client );
+}
+
 void KXMLGUIFactory::removeClient( KXMLGUIClient *client )
 {
     //kDebug(260) << client;
@@ -548,6 +571,10 @@ void KXMLGUIFactory::plugActionList( KXMLGUIClient *client, const QString &name,
 
     d->m_rootNode->plugActionList( *d );
 
+    // Load shortcuts for these new actions
+    d->saveDefaultActionProperties(actionList);
+    d->refreshActionProperties(client, actionList, client->domDocument());
+
     d->BuildState::reset();
     d->popState();
 }
@@ -568,10 +595,8 @@ void KXMLGUIFactory::unplugActionList( KXMLGUIClient *client, const QString &nam
 void KXMLGUIFactoryPrivate::applyActionProperties( const QDomElement &actionPropElement,
         ShortcutOption shortcutOption )
 {
-    for (QDomNode n = actionPropElement.firstChild();
-         !n.isNull(); n = n.nextSibling() )
-    {
-        QDomElement e = n.toElement();
+    for (QDomElement e = actionPropElement.firstChildElement();
+         !e.isNull(); e = e.nextSiblingElement()) {
         if ( !equals(e.tagName(), "action") )
             continue;
 
@@ -670,7 +695,7 @@ QDomDocument KXMLGUIFactoryPrivate::shortcutSchemeDoc(KXMLGUIClient *client)
     return doc;
 }
 
-void KXMLGUIFactoryPrivate::applyShortcutScheme(KXMLGUIClient *client, QDomDocument scheme)
+void KXMLGUIFactoryPrivate::applyShortcutScheme(KXMLGUIClient *client, const QList<QAction*> &actions, const QDomDocument& scheme)
 {
     static const QString &actionPropElementName = KGlobal::staticQString( "ActionProperties" );
 
@@ -679,33 +704,33 @@ void KXMLGUIFactoryPrivate::applyShortcutScheme(KXMLGUIClient *client, QDomDocum
 
     //First clear all existing shortcuts
     if (schemeName != "Default") {
-        foreach (QAction *action, client->actionCollection()->actions())
-        {
-            if (KAction *kaction = qobject_cast<KAction*>(action))
-            {
+        foreach (QAction *action, actions) {
+            if (KAction *kaction = qobject_cast<KAction*>(action)) {
                 kaction->setShortcut(KShortcut(), KAction::ActiveShortcut);
                 // We clear the default shortcut as well because the shortcut scheme will set its own defaults
                 kaction->setShortcut(KShortcut(), KAction::DefaultShortcut);
+                continue;
             }
-            else
+            if (action) {
                 action->setProperty("shortcut", KShortcut());
+            }
         }
     } else {
         // apply saved default shortcuts
-        foreach (QAction *action, client->actionCollection()->actions())
-        {
-            if (KAction *kaction = qobject_cast<KAction*>(action))
-            {
+        foreach (QAction *action, actions) {
+            if (KAction *kaction = qobject_cast<KAction*>(action)) {
                 QVariant savedDefaultShortcut = kaction->property("_k_DefaultShortcut");
-                if (savedDefaultShortcut.isValid())
-                {
+                if (savedDefaultShortcut.isValid()) {
                     KShortcut shortcut = savedDefaultShortcut.value<KShortcut>();
+                    //kDebug() << "scheme said" << shortcut.toString() << "for action" << kaction->objectName();
                     kaction->setShortcut(shortcut, KAction::ActiveShortcut);
                     kaction->setShortcut(shortcut, KAction::DefaultShortcut);
                     continue;
                 }
             }
-            action->setProperty("shortcut", KShortcut());
+            if (action) {
+                action->setProperty("shortcut", KShortcut());
+            }
         }
     }
 
@@ -716,55 +741,42 @@ void KXMLGUIFactoryPrivate::applyShortcutScheme(KXMLGUIClient *client, QDomDocum
     QDomElement actionPropElement = docElement.namedItem( actionPropElementName ).toElement();
 
     //Check if we really have the shortcut configuration here
-    if (!actionPropElement.isNull())
-    {
+    if (!actionPropElement.isNull()) {
         kDebug(260) << "Applying shortcut scheme for XMLGUI client" << client->componentData().componentName();
 
         //Apply all shortcuts we have
         applyActionProperties(actionPropElement, KXMLGUIFactoryPrivate::SetDefaultShortcut);
-    }
-    else
+    } else {
         kDebug(260) << "Invalid shortcut scheme file";
+    }
 }
 
 int KXMLGUIFactory::configureShortcuts(bool letterCutsOk , bool bSaveSettings )
 {
-	KShortcutsDialog dlg(KShortcutsEditor::AllActions,
-         letterCutsOk ? KShortcutsEditor::LetterShortcutsAllowed : KShortcutsEditor::LetterShortcutsDisallowed,
-         qobject_cast<QWidget*>(parent()));
-	foreach (KXMLGUIClient *client, d->m_clients)
-	{
-		if(client && !client->xmlFile().isEmpty())
-			dlg.addCollection( client->actionCollection() );
-	}
-	return dlg.configure(bSaveSettings);
+    KShortcutsDialog dlg(KShortcutsEditor::AllActions,
+                         letterCutsOk ? KShortcutsEditor::LetterShortcutsAllowed : KShortcutsEditor::LetterShortcutsDisallowed,
+                         qobject_cast<QWidget*>(parent()));
+    foreach (KXMLGUIClient *client, d->m_clients) {
+        if(client) {
+            dlg.addCollection(client->actionCollection());
+        }
+    }
+    return dlg.configure(bSaveSettings);
 }
 
+// Find or create
 QDomElement KXMLGUIFactory::actionPropertiesElement( QDomDocument& doc )
 {
-	KConfigGroup cg = KGlobal::config()->group( "Shortcut Schemes" );
-	QString schemeName = cg.readEntry("Current Scheme", "Default");
+    // first, lets see if we have existing properties
+    QDomElement elem = findActionPropertiesElement(doc);
 
-	const QString tagActionProp = QLatin1String("ActionProperties");
-	// first, lets see if we have existing properties
-	QDomElement elem;
-	QDomNode it = doc.documentElement().firstChild();
-	for( ; !it.isNull(); it = it.nextSibling() ) {
-		QDomElement e = it.toElement();
-		if( ((e.tagName() == tagActionProp) || (e.tagName() == tagActionProp.toLower()))
-				&& (e.attribute("scheme", "Default") == schemeName) ) {
-			elem = e;
-			break;
-		}
-	}
-
-	// if there was none, create one
-	if( elem.isNull() ) {
-		elem = doc.createElement( tagActionProp );
-		elem.setAttribute( "scheme", schemeName );
-		doc.documentElement().appendChild( elem );
-	}
-	return elem;
+    // if there was none, create one
+    if(elem.isNull()) {
+        elem = doc.createElement(QLatin1String("ActionProperties"));
+        elem.setAttribute("scheme", currentShortcutScheme());
+        doc.documentElement().appendChild( elem );
+    }
+    return elem;
 }
 
 QDomElement KXMLGUIFactory::findActionByName( QDomElement& elem, const QString& sName, bool create )
