@@ -35,6 +35,9 @@
 #include <QtGui/QKeyEvent>
 #include <QtGui/QStyleOption>
 
+namespace KDEPrivate
+{
+
 QPointer<KUrlNavigatorMenu> KUrlNavigatorButton::m_subDirsMenu;
 
 KUrlNavigatorButton::KUrlNavigatorButton(const KUrl& url, QWidget* parent) :
@@ -42,6 +45,7 @@ KUrlNavigatorButton::KUrlNavigatorButton(const KUrl& url, QWidget* parent) :
     m_hoverArrow(false),
     m_pendingTextChange(false),
     m_replaceButton(false),
+    m_showMnemonic(false),
     m_wheelSteps(0),
     m_url(url),
     m_subDir(),
@@ -68,14 +72,27 @@ void KUrlNavigatorButton::setUrl(const KUrl& url)
 {
     m_url = url;
 
-    if (m_url.isLocalFile()) {
-        setText(m_url.fileName());
-    } else {
+    bool startTextResolving = !m_url.isLocalFile();
+    if (startTextResolving) {
+        // Doing a text-resolving with KIO::stat() for all non-local
+        // URLs leads to problems for protocols where a limit is given for
+        // the number of parallel connections. A black-list
+        // is given where KIO::stat() should not be used:
+        static QSet<QString> protocols;
+        if (protocols.isEmpty()) {
+            protocols << "fish" << "ftp" << "nfs" << "sftp" << "smb" << "webdav";
+        }
+        startTextResolving = !protocols.contains(m_url.protocol());
+    }
+
+    if (startTextResolving) {
         m_pendingTextChange = true;
         KIO::StatJob* job = KIO::stat(m_url, KIO::HideProgressInfo);
         connect(job, SIGNAL(result(KJob*)),
                 this, SLOT(statFinished(KJob*)));
         emit startedTextResolving();
+    } else {
+        setText(m_url.fileName().replace('&', "&&"));
     }
 }
 
@@ -105,10 +122,8 @@ void KUrlNavigatorButton::setActiveSubDirectory(const QString& subDir)
 {
     m_subDir = subDir;
 
-    QFont adjustedFont(font());
-    adjustedFont.setBold(m_subDir.isEmpty());
-    setFont(adjustedFont);
-
+    // We use a different (bold) font on active, so the size hint changes
+    updateGeometry();
     update();
 }
 
@@ -119,18 +134,37 @@ QString KUrlNavigatorButton::activeSubDirectory() const
 
 QSize KUrlNavigatorButton::sizeHint() const
 {
+    QFont adjustedFont(font());
+    adjustedFont.setBold(m_subDir.isEmpty());
     // the minimum size is textWidth + arrowWidth() + 2 * BorderWidth; for the
     // preferred size we add the BorderWidth 2 times again for having an uncluttered look
-    const int width = fontMetrics().width(text()) + arrowWidth() + 4 * BorderWidth;
+    const int width = QFontMetrics(adjustedFont).width(plainText()) + arrowWidth() + 4 * BorderWidth;
     return QSize(width, KUrlNavigatorButtonBase::sizeHint().height());
 }
 
+void KUrlNavigatorButton::setShowMnemonic(bool show)
+{
+    if (m_showMnemonic != show) {
+        m_showMnemonic = show;
+        update();
+    }
+}
+
+bool KUrlNavigatorButton::showMnemonic() const
+{
+    return m_showMnemonic;
+}
+
 void KUrlNavigatorButton::paintEvent(QPaintEvent* event)
 {
     Q_UNUSED(event);
 
     QPainter painter(this);
 
+    QFont adjustedFont(font());
+    adjustedFont.setBold(m_subDir.isEmpty());
+    painter.setFont(adjustedFont);
+
     int buttonWidth  = width();
     int preferredWidth = sizeHint().width();
     if (preferredWidth < minimumWidth()) {
@@ -190,7 +224,6 @@ void KUrlNavigatorButton::paintEvent(QPaintEvent* event)
 
     painter.setPen(fgColor);
     const bool clipped = isTextClipped();
-    const int align = clipped ? Qt::AlignVCenter : Qt::AlignCenter;
     const QRect textRect(textLeft, 0, textWidth, buttonHeight);
     if (clipped) {
         QColor bgColor = fgColor;
@@ -208,7 +241,14 @@ void KUrlNavigatorButton::paintEvent(QPaintEvent* event)
         pen.setBrush(QBrush(gradient));
         painter.setPen(pen);
     }
-    painter.drawText(textRect, align, text());
+
+    int textFlags = clipped ? Qt::AlignVCenter : Qt::AlignCenter;
+    if (m_showMnemonic) {
+        textFlags |= Qt::TextShowMnemonic;
+        painter.drawText(textRect, textFlags, text());
+    } else {
+        painter.drawText(textRect, textFlags, plainText());
+    }
 }
 
 void KUrlNavigatorButton::enterEvent(QEvent* event)
@@ -218,7 +258,7 @@ void KUrlNavigatorButton::enterEvent(QEvent* event)
     // if the text is clipped due to a small window width, the text should
     // be shown as tooltip
     if (isTextClipped()) {
-        setToolTip(text());
+        setToolTip(plainText());
     }
 }
 
@@ -233,6 +273,22 @@ void KUrlNavigatorButton::leaveEvent(QEvent* event)
     }
 }
 
+void KUrlNavigatorButton::keyPressEvent(QKeyEvent* event)
+{
+    switch (event->key()) {
+    case Qt::Key_Enter:
+    case Qt::Key_Return:
+        emit clicked(m_url, Qt::LeftButton);
+        break;
+    case Qt::Key_Down:
+    case Qt::Key_Space:
+        startSubDirsJob();
+        break;
+    default:
+        KUrlNavigatorButtonBase::keyPressEvent(event);
+    }
+}
+
 void KUrlNavigatorButton::dropEvent(QDropEvent* event)
 {
     const KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
@@ -329,10 +385,9 @@ void KUrlNavigatorButton::wheelEvent(QWheelEvent* event)
         m_wheelSteps = event->delta() / 120;
         m_replaceButton = true;
         startSubDirsJob();
-        event->accept();
-    } else {
-        KUrlNavigatorButtonBase::wheelEvent(event);
     }
+
+    KUrlNavigatorButtonBase::wheelEvent(event);
 }
 
 void KUrlNavigatorButton::requestSubDirs()
@@ -352,8 +407,8 @@ void KUrlNavigatorButton::startSubDirsJob()
     m_subDirsJob = KIO::listDir(url, KIO::HideProgressInfo, false /*no hidden files*/);
     m_subDirs.clear(); // just to be ++safe
 
-    connect(m_subDirsJob, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList &)),
-            this, SLOT(addEntriesToSubDirs(KIO::Job*, const KIO::UDSEntryList&)));
+    connect(m_subDirsJob, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
+            this, SLOT(addEntriesToSubDirs(KIO::Job*,KIO::UDSEntryList)));
 
     if (m_replaceButton) {
         connect(m_subDirsJob, SIGNAL(result(KJob*)), this, SLOT(replaceButton(KJob*)));
@@ -365,6 +420,7 @@ void KUrlNavigatorButton::startSubDirsJob()
 void KUrlNavigatorButton::addEntriesToSubDirs(KIO::Job* job, const KIO::UDSEntryList& entries)
 {
     Q_ASSERT(job == m_subDirsJob);
+    Q_UNUSED(job);
 
     foreach (const KIO::UDSEntry& entry, entries) {
         if (entry.isDir()) {
@@ -511,6 +567,33 @@ void KUrlNavigatorButton::cancelSubDirsRequest()
     }
 }
 
+QString KUrlNavigatorButton::plainText() const
+{
+    // Replace all "&&" by '&' and remove all single
+    // '&' characters
+    const QString source = text();
+    const int sourceLength = source.length();
+
+    QString dest;
+    dest.reserve(sourceLength);
+
+    int sourceIndex = 0;
+    int destIndex = 0;
+    while (sourceIndex < sourceLength) {
+        if (source.at(sourceIndex) == QLatin1Char('&')) {
+            ++sourceIndex;
+            if (sourceIndex >= sourceLength) {
+                break;
+            }
+        }
+        dest[destIndex] = source.at(sourceIndex);
+        ++sourceIndex;
+        ++destIndex;
+    }
+
+    return dest;
+}
+
 int KUrlNavigatorButton::arrowWidth() const
 {
     // if there isn't arrow then return 0
@@ -538,8 +621,9 @@ bool KUrlNavigatorButton::isTextClipped() const
         availableWidth -= arrowWidth() - BorderWidth;
     }
 
-    QFontMetrics fontMetrics(font());
-    return fontMetrics.width(text()) >= availableWidth;
+    QFont adjustedFont(font());
+    adjustedFont.setBold(m_subDir.isEmpty());
+    return QFontMetrics(adjustedFont).width(plainText()) >= availableWidth;
 }
 
 void KUrlNavigatorButton::updateMinimumWidth()
@@ -563,8 +647,8 @@ void KUrlNavigatorButton::initMenu(KUrlNavigatorMenu* menu, int startIndex)
 {
     connect(menu, SIGNAL(middleMouseButtonClicked(QAction*)),
             this, SLOT(slotMenuActionClicked(QAction*)));
-    connect(menu, SIGNAL(urlsDropped(QAction*, QDropEvent*)),
-            this, SLOT(urlsDropped(QAction*, QDropEvent*)));
+    connect(menu, SIGNAL(urlsDropped(QAction*,QDropEvent*)),
+            this, SLOT(urlsDropped(QAction*,QDropEvent*)));
 
     menu->setLayoutDirection(Qt::LeftToRight);
 
@@ -594,5 +678,6 @@ void KUrlNavigatorButton::initMenu(KUrlNavigatorMenu* menu, int startIndex)
     }
 }
 
+} // namespace KDEPrivate
 
 #include "kurlnavigatorbutton_p.moc"
