@@ -119,7 +119,73 @@ using namespace DOM;
                }
                return;
             }
-            style()->drawControl(element,option,painter,widget);
+
+            if (element == QStyle::CE_ComboBoxLabel) {
+                const QStyleOptionComboBox *o = qstyleoption_cast<const QStyleOptionComboBox*>(option);
+                if (o) {
+                    QStyleOptionComboBox comboOpt = *o;
+                    comboOpt.currentText = comboOpt.currentText.trimmed();
+                    // by default combobox label is drawn left justified, vertical centered
+                    // translate it to reflect padding values
+                    comboOpt.rect.translate(left, (top - bottom) / 2);
+                    if (noBorder) {
+                        // Need to expand a bit for some styles
+                        comboOpt.rect.adjust(-1, -2, 1, 2);
+                        comboOpt.rect.translate(-1, 0);
+                        comboOpt.state &= ~State_On;
+                    }
+                    return style()->drawControl(element, &comboOpt, painter, widget);
+                }
+            }
+
+            style()->drawControl(element, option, painter, widget);
+        }
+
+        void drawComplexControl(ComplexControl cc, const QStyleOptionComplex *opt, QPainter *painter, const QWidget *widget) const
+        {
+            if ((cc == QStyle::CC_ComboBox) && noBorder) {
+                if (const QStyleOptionComboBox *cbOpt = qstyleoption_cast<const QStyleOptionComboBox*>(opt)) {
+                    bool enabled = (cbOpt->state & State_Enabled);
+                    QColor color = cbOpt->palette.color(QPalette::ButtonText);
+                    if (color.value() < 128)
+                        color = color.lighter();
+                    painter->save();
+                    painter->setBackgroundMode(Qt::TransparentMode);
+                    painter->setPen(color);
+                    // Drop down indicator
+                    painter->setBrush(enabled ? QBrush(color, Qt::SolidPattern) : Qt::NoBrush);
+                    QRect arrowRect = style()->subControlRect(cc, opt, SC_ComboBoxArrow, widget);
+                    if (enabled && (cbOpt->state & State_On))
+                        arrowRect.translate(1, 1); // push effect
+                    const int arrowDown[] = { 5,-2, 0,3, -5,-2, -4,-3, -3,-3, 0,0, 3,-3, 4,-3 };
+                    QPolygon a(8);
+                    a.setPoints(8, arrowDown);
+                    a.translate((arrowRect.x() + (arrowRect.width() >> 1)), (arrowRect.y() + (arrowRect.height() >> 1)));
+                    painter->drawPolygon(a);
+                    // Focus rect
+                    if (enabled && (cbOpt->state & State_HasFocus)) {
+                        QRect focusRect = style()->subElementRect(SE_ComboBoxFocusRect, cbOpt, widget);
+                        focusRect.adjust(0, -2, 0, 2);
+                        // Begin drawing focus rect (from qcleanlooksstyle)
+                        painter->setBrush(QBrush(color, Qt::Dense4Pattern));
+                        painter->setBrushOrigin(focusRect.topLeft());
+                        painter->setPen(Qt::NoPen);
+                        const QRect rects[4] = {
+                            QRect(focusRect.left(), focusRect.top(), focusRect.width(), 1),    // Top
+                            QRect(focusRect.left(), focusRect.bottom(), focusRect.width(), 1), // Bottom
+                            QRect(focusRect.left(), focusRect.top(), 1, focusRect.height()),   // Left
+                            QRect(focusRect.right(), focusRect.top(), 1, focusRect.height())   // Right
+                        };
+                        painter->drawRects(rects, 4);
+                        // End drawing focus rect
+                    }
+                    painter->restore();
+
+                    return;
+                }
+            }
+
+            style()->drawComplexControl(cc, opt, painter, widget);
         }
 
         QRect subControlRect(ComplexControl cc, const QStyleOptionComplex* opt, SubControl sc, const QWidget* widget) const
@@ -269,7 +335,6 @@ void RenderFormElement::updateFromElement()
         document()->quietResetFocus();
 
     RenderWidget::updateFromElement();
-    setPadding();
 }
 
 void RenderFormElement::layout()
@@ -291,8 +356,7 @@ void RenderFormElement::layout()
     setNeedsLayout(false);
 }
 
-
-Qt::AlignmentFlag RenderFormElement::textAlignment() const
+Qt::Alignment RenderFormElement::textAlignment() const
 {
     switch (style()->textAlign()) {
         case LEFT:
@@ -1038,7 +1102,8 @@ void RenderLineEdit::setStyle(RenderStyle* _style)
 {
     RenderFormElement::setStyle( _style );
 
-    widget()->setAlignment(textAlignment());
+    if (widget()->alignment() != textAlignment())
+        widget()->setAlignment(textAlignment());
 
     bool showClearButton = (!shouldDisableNativeBorders() && !_style->hasBackgroundImage());
 
@@ -1661,6 +1726,15 @@ void RenderSelect::clearItemFlags(int index, Qt::ItemFlags flags)
     }
 }
 
+void RenderSelect::setStyle(RenderStyle *_style)
+{
+    RenderFormElement::setStyle(_style);
+    if (!m_useListBox) {
+        KHTMLProxyStyle* proxyStyle = static_cast<KHTMLProxyStyle*>(getProxyStyle());
+        proxyStyle->noBorder = shouldDisableNativeBorders();
+    }
+}
+
 void RenderSelect::updateFromElement()
 {
     m_ignoreSelectEvents = true;
@@ -1677,10 +1751,14 @@ void RenderSelect::updateFromElement()
     if (oldMultiple != m_multiple || oldSize != m_size) {
         if (m_useListBox != oldListbox) {
             // type of select has changed
-            if(m_useListBox)
+            if (m_useListBox)
                 setQWidget(createListBox());
             else
                 setQWidget(createComboBox());
+
+            // Call setStyle() to fix unwanted font size change (#142722)
+            // and to update our proxy style properties
+            setStyle(style());
         }
 
         if (m_useListBox && oldMultiple != m_multiple) {
@@ -1758,8 +1836,10 @@ void RenderSelect::updateFromElement()
                 if (disabled)
                     clearItemFlags(listIndex, Qt::ItemIsSelectable | Qt::ItemIsEnabled);
             }
-            else
+            else {
                 KHTMLAssert(false);
+            }
+
             m_selectionChanged = true;
         }
 
@@ -1787,9 +1867,8 @@ short RenderSelect::baselinePosition( bool f ) const
     if (m_useListBox)
         return RenderFormElement::baselinePosition(f);
 
-    bool hasFrame = static_cast<KComboBox*>(widget())->hasFrame();
-    int bTop = hasFrame ? 0 : borderTop();
-    int bBottom = hasFrame ? 0 : borderBottom();
+    int bTop = shouldDisableNativeBorders() ? borderTop() : 0;
+    int bBottom = shouldDisableNativeBorders() ? borderBottom() : 0;
     int ret = (height()-RenderWidget::paddingTop()-RenderWidget::paddingBottom()-bTop-bBottom+1)/2;
     ret += marginTop() + RenderWidget::paddingTop() + bTop;
     ret += ((fontMetrics( f ).ascent())/2)-2;
@@ -1882,8 +1961,17 @@ void RenderSelect::layout( )
     }
     else {
         QSize s(m_widget->sizeHint());
-        setIntrinsicWidth( s.width() );
-        setIntrinsicHeight( s.height() );
+        int w = s.width();
+        int h = s.height();
+
+        if (shouldDisableNativeBorders()) {
+            const int dfw = 2 * m_widget->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, m_widget);
+            w -= dfw;
+            h -= dfw;
+        }
+
+        setIntrinsicWidth(w);
+        setIntrinsicHeight(h);
     }
 
     /// uuh, ignore the following line..
@@ -1895,7 +1983,7 @@ void RenderSelect::layout( )
 
     bool foundOption = false;
     for (int i = 0; i < listItems.size() && !foundOption; i++)
-	foundOption = (listItems[i]->id() == ID_OPTION);
+         foundOption = (listItems[i]->id() == ID_OPTION);
 
     m_widget->setEnabled(foundOption && ! element()->disabled());
 }
@@ -1989,8 +2077,7 @@ void RenderSelect::setOptionsChanged(bool _optionsChanged)
 
 void RenderSelect::setPadding()
 {
-    if (m_size > 1 || m_multiple)
-        RenderFormElement::setPadding();
+    RenderFormElement::setPadding();
 }
 
 ListBoxWidget* RenderSelect::createListBox()
@@ -2065,21 +2152,17 @@ TextAreaWidget::TextAreaWidget(int wrap, QWidget* parent)
     KCursor::setAutoHideCursor(viewport(), true);
     setAcceptRichText (false);
     setMouseTracking(true);
-
-}
-
-void TextAreaWidget::scrollContentsBy( int dx, int dy )
-{
-    KTextEdit::scrollContentsBy(dx, dy);
-    update();
-
 }
 
 TextAreaWidget::~TextAreaWidget()
 {
 }
 
-
+void TextAreaWidget::scrollContentsBy( int dx, int dy )
+{
+    KTextEdit::scrollContentsBy(dx, dy);
+    update();
+}
 
 bool TextAreaWidget::event( QEvent *e )
 {
@@ -2140,6 +2223,7 @@ RenderTextArea::RenderTextArea(HTMLTextAreaElementImpl *element)
     connect(edit,SIGNAL(textChanged()),this,SLOT(slotTextChanged()));
 
     setText(element->value().string());
+    m_textAlignment = edit->alignment();
 }
 
 RenderTextArea::~RenderTextArea()
@@ -2200,18 +2284,29 @@ void RenderTextArea::calcMinMaxWidth()
 
 void RenderTextArea::setStyle(RenderStyle* _style)
 {
-    bool unsubmittedFormChange = element()->m_unsubmittedFormChange;
-
     RenderFormElement::setStyle(_style);
 
-    bool blocked = widget()->blockSignals(true);
-    widget()->setAlignment(textAlignment());
-    widget()->blockSignals(blocked);
+    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
+
+    if (m_textAlignment != textAlignment()) {
+        m_textAlignment = textAlignment();
+        bool unsubmittedFormChange = element()->m_unsubmittedFormChange;
+        bool blocked = w->blockSignals(true);
+        int cx = w->horizontalScrollBar()->value();
+        int cy = w->verticalScrollBar()->value();
+        QTextCursor tc = w->textCursor();
+        // Set alignment on all textarea's paragraphs
+        w->selectAll();
+        w->setAlignment(m_textAlignment);
+        w->setTextCursor(tc);
+        w->horizontalScrollBar()->setValue(cx);
+        w->verticalScrollBar()->setValue(cy);
+        w->blockSignals(blocked);
+        element()->m_unsubmittedFormChange = unsubmittedFormChange;
+    }
 
     scrollbarsStyled = false;
 
-    element()->m_unsubmittedFormChange = unsubmittedFormChange;
-    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
     if (style()->overflowX() == OSCROLL)
         w->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOn );
     else if (style()->overflowX() == OHIDDEN)
@@ -2259,33 +2354,31 @@ void RenderTextArea::setText(const QString& newText)
 
     // When this is called, m_value in the element must have just
     // been set to new value --- see if we have any work to do
-    if ( newText != text() ) {
+
+    QString oldText = text();
+    int oldTextLen = oldText.length();
+    int newTextLen = newText.length();
+    if (newTextLen != oldTextLen || newText != oldText) {
         bool blocked = w->blockSignals(true);
-        QTextCursor tc = w->textCursor();
-        bool atEnd = tc.atEnd();
-        bool atStart = tc.atStart();
         int cx = w->horizontalScrollBar()->value();
         int cy = w->verticalScrollBar()->value();
-        QString oldText = w->toPlainText();
+        // Not using setPlaintext as it resets text alignment property
+        int minLen = (newTextLen < oldTextLen) ? newTextLen : oldTextLen;
         int ex = 0;
-        int otl = oldText.length();
-        if (otl && newText.length() > otl) {
-            while (ex < otl && newText[ex] == oldText[ex])
-                ++ex;
-            QTextCursor tc(w->document());
-            tc.setPosition( ex, QTextCursor::MoveAnchor );
-            tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
-            tc.insertText(newText.right( newText.length()-ex ));
-        } else {
-            w->setPlainText( newText );
-        }
+        while (ex < minLen && (newText.at(ex) == oldText.at(ex)))
+               ++ex;
+        QTextCursor tc = w->textCursor();
+        tc.setPosition(ex, QTextCursor::MoveAnchor);
+        tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
+        tc.insertText(newText.right(newTextLen - ex));
+
+        if (oldTextLen == 0)
+            tc.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
+        else
+            tc.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);
         w->setTextCursor(tc);
-        if (atEnd)
-           tc.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);
-        else if (atStart)
-           tc.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
-        w->horizontalScrollBar()->setValue( cx );
-        w->verticalScrollBar()->setValue( cy );
+        w->horizontalScrollBar()->setValue(cx);
+        w->verticalScrollBar()->setValue(cy);
         w->blockSignals(blocked);
     }
 }
