@@ -33,6 +33,7 @@
 #include "misc/translator.h"
 #include <rendering/render_layer.h>
 #include <khtmlview.h>
+#include <khtml_part.h>
 
 #include <QtGui/QActionEvent>
 #include <kdebug.h>
@@ -362,6 +363,7 @@ khtml::IDTable<EventImpl>* EventImpl::initIdTable()
     s_idTable->addStaticMapping(KEYUP_EVENT, "keyup");
     s_idTable->addStaticMapping(KEYPRESS_EVENT, "keypress");
         //DOM3 ev. suggests textInput, but it's better for compat this way
+    s_idTable->addStaticMapping(HASHCHANGE_EVENT, "hashchange");
 
     //khtml extensions
     s_idTable->addStaticMapping(KHTML_ECMA_DBLCLICK_EVENT, "dblclick");
@@ -374,6 +376,8 @@ khtml::IDTable<EventImpl>* EventImpl::initIdTable()
         // idem
     s_idTable->addStaticMapping(KHTML_READYSTATECHANGE_EVENT, "readystatechange");
 
+    s_idTable->addStaticMapping(MESSAGE_EVENT, "message");
+
     return s_idTable;
 }
 
@@ -402,6 +406,16 @@ bool EventImpl::isKeyboardEvent() const
     return false;
 }
 
+bool EventImpl::isMessageEvent() const
+{
+    return false;
+}
+
+bool EventImpl::isHashChangeEvent() const
+{
+    return false;
+}
+
 // -----------------------------------------------------------------------------
 
 UIEventImpl::UIEventImpl(EventId _id, bool canBubbleArg, bool cancelableArg,
@@ -891,10 +905,32 @@ int KeyboardEventImpl::keyCode() const
 {
     //Keycode on key events always identifies the -key- and not the input,
     //so e.g. 'a' will get 'A'
-    if (m_virtKeyVal != DOM_VK_UNDEFINED)
+    if (m_virtKeyVal != DOM_VK_UNDEFINED) {
         return m_virtKeyVal;
-    else
-        return QChar((unsigned short)m_keyVal).toUpper().unicode();
+    } else {
+        unsigned char code = QChar((unsigned short)m_keyVal).toUpper().unicode();
+        // Some codes we get from Qt are things like ( which conflict with
+        // browser scancodes. Remap them to what's on their keycaps in a US
+        // layout.
+        if (virtKeyToQtKey()->hasLeft(code)) {
+            switch (code) {
+                case '!': return '1';
+                case '@': return '2';
+                case '#': return '3';
+                case '$': return '4';
+                case '%': return '5';
+                case '^': return '6';
+                case '&': return '7';
+                case '*': return '8';
+                case '(': return '9';
+                case ')': return '0';
+                default:
+                    kWarning(6000) << "Don't know how resolve conflict of code:" << code
+                                   << " with a virtKey";
+            }
+        }
+        return code;
+    }
 }
 
 int KeyboardEventImpl::charCode() const
@@ -1042,3 +1078,46 @@ bool MutationEventImpl::isMutationEvent() const
     return true;
 }
 
+// -----------------------------------------------------------------------------
+
+MessageEventImpl::MessageEventImpl()
+{}
+
+
+bool MessageEventImpl::isMessageEvent() const
+{
+    return true;
+}
+
+void MessageEventImpl::initMessageEvent(const DOMString& typeArg,
+                                        bool  canBubbleArg,
+                                        bool  cancelableArg,
+                                        const RefPtr<Data>& dataArg,
+                                        const DOMString& originArg,
+                                        const DOMString& lastEventIdArg,
+                                        KHTMLPart* sourceArg)
+{
+    EventImpl::initEvent(typeArg, canBubbleArg, cancelableArg);
+    m_data   = dataArg;
+    m_origin = originArg;
+    m_lastEventId = lastEventIdArg;
+    m_source      = sourceArg;
+}
+
+// -----------------------------------------------------------------------------
+
+HashChangeEventImpl::HashChangeEventImpl()
+{
+}
+
+bool HashChangeEventImpl::isHashChangeEvent() const
+{
+  return true;
+}
+
+void HashChangeEventImpl::initHashChangeEvent(const DOMString& eventTypeArg, bool canBubbleArg, bool cancelableArg, const DOMString& oldUrl, const DOMString& newUrl)
+{
+    EventImpl::initEvent(eventTypeArg, canBubbleArg, cancelableArg);
+    m_oldUrl = oldUrl;
+    m_newUrl = newUrl;
+}
