@@ -23,6 +23,7 @@
 
 #include <stdlib.h>
 #include <QtGui/QImage>
+#include <QtGui/QPainter>
 #include <QtCore/QIODevice>
 #include <QtCore/QStack>
 #include <QtCore/QVector>
@@ -31,8 +32,9 @@
 
 
 int XCFImageFormat::random_table[RANDOM_TABLE_SIZE];
+bool XCFImageFormat::random_table_initialized;
 
-//int XCFImageFormat::add_lut[256][256];
+QVector<QRgb> XCFImageFormat::grayTable;
 
 
 const XCFImageFormat::LayerModes XCFImageFormat::layer_modes[] = {
@@ -69,10 +71,16 @@ inline QRgb qRgba ( const QRgb& rgb, int a )
 
 
 /*!
- * The constructor for the XCF image loader. This initializes the
- * tables used in the layer merging routines.
+ * The constructor for the XCF image loader.
  */
 XCFImageFormat::XCFImageFormat()
+{
+}
+
+/*!
+ * This initializes the tables used in the layer dissolving routines.
+ */
+void XCFImageFormat::initializeRandomTable()
 {
 	// From GIMP "paint_funcs.c" v1.2
 	srand(RANDOM_SEED);
@@ -87,15 +95,6 @@ XCFImageFormat::XCFImageFormat()
 		random_table[i] = random_table[swap];
 		random_table[swap] = tmp;
 	}
-
-//	for (int j = 0; j < 256; j++) {
-//		for (int k = 0; k < 256; k++) {
-//			int tmp_sum = j + k;
-//			if (tmp_sum > 255)
-//				tmp_sum = 255;
-//			add_lut[j][k] = tmp_sum;
-//		}
-//	}
 }
 
 inline
@@ -114,6 +113,10 @@ bool XCFImageFormat::readXCF(QIODevice *device, QImage *outImage)
             kDebug(399) << "XCF: read failure on header tag";
             return false;
 	}
+	if (qstrncmp(tag, "gimp xcf", 8) != 0) {
+            kDebug(399) << "XCF: read called on non-XCF file";
+            return false;
+	}
 
 	xcf_io >> xcf_image.width >> xcf_image.height >> xcf_image.type;
 
@@ -492,6 +495,15 @@ bool XCFImageFormat::composeTiles(XCFImage& xcf_image)
 	layer.nrows = (layer.height + TILE_HEIGHT - 1) / TILE_HEIGHT;
 	layer.ncols = (layer.width + TILE_WIDTH - 1) / TILE_WIDTH;
 
+        //kDebug(399) << "IMAGE: height=" << xcf_image.height << ", width=" << xcf_image.width;
+        //kDebug(399) << "LAYER: height=" << layer.height << ", width=" << layer.width;
+        //kDebug(399) << "LAYER: rows=" << layer.nrows << ", columns=" << layer.ncols;
+        
+        // SANITY CHECK: Catch corrupted XCF image file where the width or height
+        // of a tile is reported are bogus. See Bug# 234030.
+        if (layer.height > xcf_image.height || layer.width > xcf_image.width)
+              return false;
+       
 	layer.image_tiles.resize(layer.nrows);
 
 	if (layer.type == GRAYA_GIMAGE || layer.type == INDEXEDA_GIMAGE)
@@ -602,8 +614,14 @@ bool XCFImageFormat::composeTiles(XCFImage& xcf_image)
  */
 void XCFImageFormat::setGrayPalette(QImage& image)
 {
-	for (int i = 0; i < 256; i++)
-		image.setColor(i, qRgb(i, i, i));
+	if (grayTable.isEmpty()) {
+		grayTable.resize(256);
+
+		for (int i = 0; i < 256; i++)
+			grayTable[i] = qRgb(i, i, i);
+	}
+
+	image.setColorTable(grayTable);
 }
 
 
@@ -614,8 +632,9 @@ void XCFImageFormat::setGrayPalette(QImage& image)
  */
 void XCFImageFormat::setPalette(XCFImage& xcf_image, QImage& image)
 {
-	for (int i = 0; i < xcf_image.num_colors; i++)
-		image.setColor(i, xcf_image.palette[i]);
+	Q_ASSERT (xcf_image.num_colors == xcf_image.palette.size());
+
+	image.setColorTable(xcf_image.palette);
 }
 
 
@@ -628,24 +647,29 @@ void XCFImageFormat::setPalette(XCFImage& xcf_image, QImage& image)
  */
 void XCFImageFormat::assignImageBytes(Layer& layer, uint i, uint j)
 {
+	QImage &image = layer.image_tiles[j][i];
 	uchar* tile = layer.tile;
+	const int width = image.width();
+	const int height = image.height();
+	const int bytesPerLine = image.bytesPerLine();
+	uchar *bits = image.bits();
 
 	switch (layer.type) {
 		case RGB_GIMAGE:
-			for (int l = 0; l < layer.image_tiles[j][i].height(); l++) {
-				for (int k = 0; k < layer.image_tiles[j][i].width(); k++) {
-					layer.image_tiles[j][i].setPixel(k, l,
-							qRgb(tile[0], tile[1], tile[2]));
+			for (int y = 0; y < height; y++) {
+				QRgb *dataPtr = (QRgb *) (bits + y * bytesPerLine);
+				for (int x = 0; x < width; x++) {
+					*dataPtr++ = qRgb(tile[0], tile[1], tile[2]);
 					tile += sizeof(QRgb);
 				}
 			}
 			break;
 
 		case RGBA_GIMAGE:
-			for ( int l = 0; l < layer.image_tiles[j][i].height(); l++ ) {
-				for ( int k = 0; k < layer.image_tiles[j][i].width(); k++ ) {
-					layer.image_tiles[j][i].setPixel(k, l,
-							qRgba(tile[0], tile[1], tile[2], tile[3]));
+			for (int y = 0; y < height; y++) {
+				QRgb *dataPtr = (QRgb *) (bits + y * bytesPerLine);
+				for (int x = 0; x < width; x++) {
+					*dataPtr++ = 	qRgba(tile[0], tile[1], tile[2], tile[3]);
 					tile += sizeof(QRgb);
 				}
 			}
@@ -653,9 +677,10 @@ void XCFImageFormat::assignImageBytes(Layer& layer, uint i, uint j)
 
 		case GRAY_GIMAGE:
 		case INDEXED_GIMAGE:
-			for (int l = 0; l < layer.image_tiles[j][i].height(); l++) {
-				for (int k = 0; k < layer.image_tiles[j][i].width(); k++) {
-					layer.image_tiles[j][i].setPixel(k, l, tile[0]);
+			for (int y = 0; y < height; y++) {
+				uchar *dataPtr = bits + y * bytesPerLine;
+				for (int x = 0; x < width; x++) {
+					*dataPtr++ = tile[0];
 					tile += sizeof(QRgb);
 				}
 			}
@@ -663,17 +688,21 @@ void XCFImageFormat::assignImageBytes(Layer& layer, uint i, uint j)
 
 		case GRAYA_GIMAGE:
 		case INDEXEDA_GIMAGE:
-			for (int l = 0; l < layer.image_tiles[j][i].height(); l++) {
-				for (int k = 0; k < layer.image_tiles[j][i].width(); k++) {
+			for (int y = 0; y < height; y++) {
+				uchar *dataPtr = bits + y * bytesPerLine;
+				uchar *alphaPtr = layer.alpha_tiles[j][i].scanLine(y);
+				for (int x = 0; x < width; x++) {
 
 				// The "if" here should not be necessary, but apparently there
 				// are some cases where the image can contain larger indices
 				// than there are colors in the palette. (A bug in The GIMP?)
 
-					if (tile[0] < layer.image_tiles[j][i].numColors())
-						layer.image_tiles[j][i].setPixel(k, l, tile[0]);
+					if (tile[0] < image.numColors())
+						*dataPtr = tile[0];
 
-					layer.alpha_tiles[j][i].setPixel(k, l, tile[1]);
+					*alphaPtr = tile[1];
+					dataPtr += 1;
+					alphaPtr += 1;
 					tile += sizeof(QRgb);
 				}
 			}
@@ -1001,11 +1030,17 @@ bool XCFImageFormat::loadChannelProperties(QDataStream& xcf_io, Layer& layer)
  */
 void XCFImageFormat::assignMaskBytes(Layer& layer, uint i, uint j)
 {
+	QImage &image = layer.mask_tiles[j][i];
 	uchar* tile = layer.tile;
+	const int width = image.width();
+	const int height = image.height();
+	const int bytesPerLine = image.bytesPerLine();
+	uchar *bits = image.bits();
 
-	for (int l = 0; l < layer.image_tiles[j][i].height(); l++) {
-		for (int k = 0; k < layer.image_tiles[j][i].width(); k++) {
-			layer.mask_tiles[j][i].setPixel(k, l, tile[0]);
+	for (int y = 0; y < height; y++) {
+		uchar *dataPtr = bits + y * bytesPerLine;
+		for (int x = 0; x < width; x++) {
+			*dataPtr++ = tile[0];
 			tile += sizeof(QRgb);
 		}
 	}
@@ -1210,6 +1245,10 @@ void XCFImageFormat::copyLayerToImage(XCFImage& xcf_image)
 			// single layer.
 
 			if (layer.mode == DISSOLVE_MODE) {
+				if (!random_table_initialized) {
+					initializeRandomTable();
+					random_table_initialized = true;
+				}
 				if (layer.type == RGBA_GIMAGE)
 					dissolveRGBPixels(layer.image_tiles[j][i], x, y);
 
@@ -1217,6 +1256,15 @@ void XCFImageFormat::copyLayerToImage(XCFImage& xcf_image)
 					dissolveAlphaPixels(layer.alpha_tiles[j][i], x, y);
 			}
 
+			// Shortcut for common case
+			if (copy == copyRGBToRGB && layer.apply_mask != 1) {
+				QPainter painter(&image);
+				painter.setOpacity(layer.opacity / 255.0);
+				painter.setCompositionMode(QPainter::CompositionMode_Source);
+				painter.drawImage(x + layer.x_offset, y + layer.y_offset, layer.image_tiles[j][i]);
+				continue;
+			}
+
 			for (int l = 0; l < layer.image_tiles[j][i].height(); l++) {
 				for (int k = 0; k < layer.image_tiles[j][i].width(); k++) {
 
@@ -1479,6 +1527,10 @@ void XCFImageFormat::mergeLayerIntoImage(XCFImage& xcf_image)
 			// single layer.
 
 			if (layer.mode == DISSOLVE_MODE) {
+				if (!random_table_initialized) {
+					initializeRandomTable();
+					random_table_initialized = true;
+				}
 				if (layer.type == RGBA_GIMAGE)
 					dissolveRGBPixels(layer.image_tiles[j][i], x, y);
 
@@ -1486,6 +1538,16 @@ void XCFImageFormat::mergeLayerIntoImage(XCFImage& xcf_image)
 					dissolveAlphaPixels(layer.alpha_tiles[j][i], x, y);
 			}
 
+			// Shortcut for common case
+			if (merge == mergeRGBToRGB && layer.apply_mask != 1
+			    && layer.mode == NORMAL_MODE) {
+				QPainter painter(&image);
+				painter.setOpacity(layer.opacity / 255.0);
+				painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
+				painter.drawImage(x + layer.x_offset, y + layer.y_offset, layer.image_tiles[j][i]);
+				continue;
+			}
+
 			for (int l = 0; l < layer.image_tiles[j][i].height(); l++) {
 				for (int k = 0; k < layer.image_tiles[j][i].width(); k++) {
 
