@@ -249,6 +249,13 @@ public:
      */
     bool isHidden() const;
 
+    /**
+     * @return true if the file is a remote URL, or a local file on a network mount.
+     * It will return false only for really-local file systems.
+     * @since 4.7.4
+     */
+    bool isSlow() const;
+
     /**
      * Checks whether the file is a readable local .desktop file,
      * i.e. a file whose path can be given to KDesktopFile
@@ -301,7 +308,9 @@ public:
      * @see timeString()
      */
     KDateTime time( FileTimes which ) const;
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED time_t time( unsigned int which ) const;
+#endif
 
     /**
      * Requests the modification, access or creation time as a string, depending
@@ -311,7 +320,9 @@ public:
      * @see time
      */
     QString timeString( FileTimes which = ModificationTime ) const;
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED QString timeString( unsigned int which) const;
+#endif
 
     /**
      * Returns true if the file is a local file.
@@ -398,6 +409,14 @@ public:
      */
     QStringList overlays() const;
 
+    /**
+     * A comment which can contain anything - even rich text. It will
+     * simply be displayed to the user as is.
+     *
+     * @since 4.6
+     */
+    QString comment() const;
+
     /**
      * Returns the string to be displayed in the statusbar,
      * e.g. when the mouse is over this item
@@ -412,8 +431,12 @@ public:
      *
      * @param maxcount the maximum number of entries shown
      * @return the tool tip string
+     *
+     * @deprecated File Managers implement more complete tooltips.
      */
-    QString getToolTipText(int maxcount = 6) const;
+#ifndef KDE_NO_DEPRECATED
+    KDE_DEPRECATED QString getToolTipText(int maxcount = 6) const;
+#endif
 
     /**
      * Returns true if files can be dropped over this item.
@@ -425,7 +448,9 @@ public:
      * mostly makes sense for file managers only.
      * KDirModel has setDropsAllowed for similar (but configurable) logic.
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED bool acceptsDrops() const;
+#endif
 
     /**
      * Let's "KRun" this file !
@@ -519,7 +544,9 @@ public:
      *
      * @deprecated use model/view (KDirModel) and you won't need this anymore
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED void setExtraData( const void *key, void *value );
+#endif
 
     /**
      * Retrieves the extra data with the given @p key.
@@ -530,7 +557,9 @@ public:
      *
      * @deprecated use model/view (KDirModel) and you won't need this anymore
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED const void * extraData( const void *key ) const;
+#endif
 
     /**
      * Removes the extra data associated with an item via @p key.
@@ -538,7 +567,9 @@ public:
      *
      * @deprecated use model/view (KDirModel) and you won't need this anymore
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED void removeExtraData( const void *key );
+#endif
 
     /**
      * Sets the metainfo of this item to @p info.
@@ -562,7 +593,9 @@ public:
     /**
      * @deprecated simply use '='
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED void assign( const KFileItem & item );
+#endif
 
     /**
      * Reinitialize KFileItem with a new UDSEntry.
@@ -583,9 +616,11 @@ public:
      * @deprecated why not just create another KFileItem and use operator=,
      * now that it's a value class?
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED void setUDSEntry( const KIO::UDSEntry& entry, const KUrl& url,
                                      bool delayedMimeTypes = false,
                                      bool urlIsDirectory = false );
+#endif
 
     /**
      * Tries to give a local URL for this file item if possible.
@@ -593,6 +628,13 @@ public:
      */
     KUrl mostLocalUrl(bool &local) const; // KDE4 TODO: bool* local = 0
 
+    /**
+     * Tries to give a local URL for this file item if possible.
+     *
+     * \since 4.6
+     */
+    KUrl mostLocalUrl() const; // KDE5: merge with above version
+
     /**
      * Return true if default-constructed
      */
