@@ -39,8 +39,8 @@
 #include <QtGui/QApplication>
 #include <QtGui/QDesktopWidget>
 
-#include "kmimetypetrader.h"
-#include "kmimetype.h"
+#include <kmimetypetrader.h>
+#include <kmimetype.h>
 #include "kio/jobclasses.h" // for KIO::JobFlags
 #include "kio/job.h"
 #include "kio/jobuidelegate.h"
@@ -67,6 +67,7 @@
 #include <QtCore/QTextIStream>
 #include <QtCore/QDate>
 #include <QtCore/QRegExp>
+#include <QDir>
 #include <kdesktopfile.h>
 #include <kmacroexpander.h>
 #include <kshell.h>
@@ -134,7 +135,7 @@ bool KRun::runUrl(const KUrl& u, const QString& _mimetype, QWidget* window, bool
     else if (isExecutableFile(u, _mimetype)) {
         if (u.isLocalFile() && runExecutables) {
             if (KAuthorized::authorize("shell_access")) {
-                return (KRun::runCommand(KShell::quoteArg(u.toLocalFile()), QString(), QString(), window, asn)); // just execute the url as a command
+                return (KRun::runCommand(KShell::quoteArg(u.toLocalFile()), QString(), QString(), window, asn, u.directory())); // just execute the url as a command
                 // ## TODO implement deleting the file if tempFile==true
             }
             else {
@@ -210,6 +211,7 @@ bool KRun::displayOpenWithDialog(const KUrl::List& lst, QWidget* window, bool te
     return false;
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KRun::shellQuote(QString &_str)
 {
     // Credits to Walter, says Bernd G. :)
@@ -219,6 +221,7 @@ void KRun::shellQuote(QString &_str)
     QChar q('\'');
     _str.replace(q, "'\\''").prepend(q).append(q);
 }
+#endif
 
 
 class KRunMX1 : public KMacroExpanderBase
@@ -248,7 +251,7 @@ KRunMX1::expandEscapedMacro(const QString &str, int pos, QStringList &ret)
         ret << service.entryPath().replace('%', "%%");
         break;
     case 'i':
-        ret << "-icon" << service.icon().replace('%', "%%");
+        ret << "--icon" << service.icon().replace('%', "%%");
         break;
     case 'm':
 //       ret << "-miniicon" << service.icon().replace( '%', "%%" );
@@ -297,20 +300,20 @@ KRunMX2::subst(int option, const KUrl &url, QStringList &ret)
     switch (option) {
     case 'u':
         ret << ((url.isLocalFile() && url.fragment().isNull() && url.encodedQuery().isNull()) ?
-                url.toLocalFile()  : url.url());
+                QDir::toNativeSeparators(url.toLocalFile())  : url.url());
         break;
     case 'd':
         ret << url.directory();
         break;
     case 'f':
-        ret << url.path();
+        ret << QDir::toNativeSeparators(url.toLocalFile());
         break;
     case 'n':
         ret << url.fileName();
         break;
     case 'v':
         if (url.isLocalFile() && QFile::exists(url.toLocalFile())) {
-            ret << KDesktopFile(url.path()).desktopGroup().readEntry("Dev");
+            ret << KDesktopFile(url.toLocalFile()).desktopGroup().readEntry("Dev");
         }
         break;
     }
@@ -370,7 +373,9 @@ static QStringList supportedProtocols(const KService& _service)
         if (supportedProtocols.isEmpty()) {
             // compat mode: assume KIO if not set and it's a KDE app (or a KDE service)
             const QStringList categories = _service.property("Categories").toStringList();
-            if (categories.contains("KDE") || !_service.isApplication()) {
+            if (categories.contains("KDE")
+                    || !_service.isApplication()
+                    || _service.entryPath().isEmpty() /*temp service*/) {
                 supportedProtocols.append("KIO");
             }
             else { // if no KDE app, be a bit over-generic
@@ -615,7 +620,7 @@ static bool runCommandInternal(KProcess* proc, const KService* service, const QS
         if (window) {
             data.setLaunchedBy(window->winId());
         }
-        if(service)
+        if(service && !service->entryPath().isEmpty())
             data.setApplicationId(service->entryPath());
         KStartupInfo::sendStartup(id, data);
     }
@@ -1046,7 +1051,14 @@ bool KRun::runCommand(const QString& cmd, QWidget* window, const QString& workin
         kWarning() << "Command was empty, nothing to run";
         return false;
     }
-    const QString bin = KShell::splitArgs(cmd).first();
+
+    const QStringList args = KShell::splitArgs(cmd);
+    if (args.isEmpty()) {
+        kWarning() << "Command could not be parsed.";
+        return false;
+    }
+
+    const QString bin = args.first();
     return KRun::runCommand(cmd, bin, bin /*iconName*/, window, QByteArray(), workingDirectory);
 }
 
@@ -1142,15 +1154,14 @@ void KRun::init()
         d->m_bIsLocalFile = true;
     }
 
-    QString exec;
-    if (d->m_strURL.protocol().startsWith(QLatin1String("http"))) {
-        exec = d->m_externalBrowser;
-    }
-
-    if (d->m_bIsLocalFile) {
+    if (!d->m_externalBrowser.isEmpty() && d->m_strURL.protocol().startsWith(QLatin1String("http"))) {
+        if (d->runExecutable(d->m_externalBrowser)) {
+            return;
+        }
+    } else if (d->m_bIsLocalFile) {
         if (d->m_mode == 0) {
             KDE_struct_stat buff;
-            if (KDE::stat(d->m_strURL.path(), &buff) == -1) {
+            if (KDE::stat(d->m_strURL.toLocalFile(), &buff) == -1) {
                 d->m_showingDialog = true;
                 KMessageBoxWrapper::error(d->m_window,
                                           i18n("<qt>Unable to run the command specified. "
@@ -1168,45 +1179,31 @@ void KRun::init()
         KMimeType::Ptr mime = KMimeType::findByUrl(d->m_strURL, d->m_mode, d->m_bIsLocalFile);
         assert(mime);
         kDebug(7010) << "MIME TYPE is " << mime->name();
-        mimeTypeDetermined(mime->name());
-        return;
-    }
-    else if (!exec.isEmpty() || KProtocolInfo::isHelperProtocol(d->m_strURL)) {
-        kDebug(7010) << "Helper protocol";
-
-        bool ok = false;
-        KUrl::List urls;
-        urls.append(d->m_strURL);
-        if (exec.isEmpty()) {
-            exec = KProtocolInfo::exec(d->m_strURL.protocol());
-            if (exec.isEmpty()) {
-                mimeTypeDetermined(KProtocolManager::defaultMimetype(d->m_strURL));
+        if (!d->m_externalBrowser.isEmpty() && (
+               mime->is(QLatin1String("text/html")) ||
+               mime->is(QLatin1String("application/xml")))) {
+            if (d->runExecutable(d->m_externalBrowser)) {
                 return;
             }
-            run(exec, urls, d->m_window, QString(), QString(), d->m_asn);
-            ok = true;
-        }
-        else if (exec.startsWith('!')) {
-            exec = exec.mid(1); // Literal command
-            exec += " %u";
-            run(exec, urls, d->m_window, QString(), QString(), d->m_asn);
-            ok = true;
-        }
-        else {
-            KService::Ptr service = KService::serviceByStorageId(exec);
-            if (service) {
-                run(*service, urls, d->m_window, false, QString(), d->m_asn);
-                ok = true;
-            }
-        }
-
-        if (ok) {
-            d->m_bFinished = true;
-            // will emit the error and autodelete this
-            d->startTimer();
+        } else {
+            mimeTypeDetermined(mime->name());
             return;
         }
     }
+    else if (KProtocolInfo::isHelperProtocol(d->m_strURL)) {
+        kDebug(7010) << "Helper protocol";
+        const QString exec = KProtocolInfo::exec(d->m_strURL.protocol());
+        if (exec.isEmpty()) {
+            mimeTypeDetermined(KProtocolManager::defaultMimetype(d->m_strURL));
+            return;
+        } else {
+            if (run(exec, KUrl::List() << d->m_strURL, d->m_window, QString(), QString(), d->m_asn)) {
+                d->m_bFinished = true;
+                d->startTimer();
+                return;
+            }
+        }
+    }
 
     // Did we already get the information that it is a directory ?
     if (S_ISDIR(d->m_mode)) {
@@ -1229,8 +1226,8 @@ void KRun::init()
     KIO::JobFlags flags = d->m_bProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo;
     KIO::StatJob *job = KIO::stat(d->m_strURL, KIO::StatJob::SourceSide, 0 /* no details */, flags);
     job->ui()->setWindow(d->m_window);
-    connect(job, SIGNAL(result(KJob *)),
-            this, SLOT(slotStatResult(KJob *)));
+    connect(job, SIGNAL(result(KJob*)),
+            this, SLOT(slotStatResult(KJob*)));
     d->m_job = job;
     kDebug(7010) << " Job " << job << " is about stating " << d->m_strURL.url();
 }
@@ -1245,6 +1242,30 @@ KRun::~KRun()
     delete d;
 }
 
+bool KRun::KRunPrivate::runExecutable(const QString& _exec)
+{
+    KUrl::List urls;
+    urls.append(m_strURL);
+    if (_exec.startsWith('!')) {
+        QString exec = _exec.mid(1); // Literal command
+        exec += " %u";
+        if (q->run(exec, urls, m_window, QString(), QString(), m_asn)) {
+            m_bFinished = true;
+            startTimer();
+            return true;
+        }
+    }
+    else {
+        KService::Ptr service = KService::serviceByStorageId(_exec);
+        if (service && q->run(*service, urls, m_window, false, QString(), m_asn)) {
+            m_bFinished = true;
+            startTimer();
+            return true;
+        }
+    }
+    return false;
+}
+
 void KRun::scanFile()
 {
     kDebug(7010) << d->m_strURL;
@@ -1276,10 +1297,10 @@ void KRun::scanFile()
     KIO::JobFlags flags = d->m_bProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo;
     KIO::TransferJob *job = KIO::get(d->m_strURL, KIO::NoReload /*reload*/, flags);
     job->ui()->setWindow(d->m_window);
-    connect(job, SIGNAL(result(KJob *)),
-            this, SLOT(slotScanFinished(KJob *)));
-    connect(job, SIGNAL(mimetype(KIO::Job *, const QString &)),
-            this, SLOT(slotScanMimeType(KIO::Job *, const QString &)));
+    connect(job, SIGNAL(result(KJob*)),
+            this, SLOT(slotScanFinished(KJob*)));
+    connect(job, SIGNAL(mimetype(KIO::Job*,QString)),
+            this, SLOT(slotScanMimeType(KIO::Job*,QString)));
     d->m_job = job;
     kDebug(7010) << " Job " << job << " is about getting from " << d->m_strURL.url();
 }
@@ -1323,28 +1344,36 @@ void KRun::slotTimeout()
 void KRun::slotStatResult(KJob * job)
 {
     d->m_job = 0L;
-    if (job->error()) {
-        d->m_showingDialog = true;
-        kError(7010) << this << "ERROR" << job->error() << job->errorString();
-        job->uiDelegate()->showErrorMessage();
-        //kDebug(7010) << this << " KRun returning from showErrorDialog, starting timer to delete us";
-        d->m_showingDialog = false;
+    const int errCode = job->error();
+    if (errCode) {
+        // ERR_NO_CONTENT is not an error, but an indication no further
+        // actions needs to be taken.
+        if (errCode != KIO::ERR_NO_CONTENT) {
+            d->m_showingDialog = true;
+            kError(7010) << this << "ERROR" << job->error() << job->errorString();
+            job->uiDelegate()->showErrorMessage();
+            //kDebug(7010) << this << " KRun returning from showErrorDialog, starting timer to delete us";
+            d->m_showingDialog = false;
+            d->m_bFault = true;
+        }
 
-        d->m_bFault = true;
         d->m_bFinished = true;
 
         // will emit the error and autodelete this
         d->startTimer();
-
     }
     else {
-
         kDebug(7010) << "Finished";
-        if (!qobject_cast<KIO::StatJob*>(job)) {
+
+        KIO::StatJob* statJob = qobject_cast<KIO::StatJob*>(job);
+        if (!statJob) {
             kFatal() << "job is a " << typeid(*job).name() << " should be a StatJob";
         }
 
-        const KIO::UDSEntry entry = ((KIO::StatJob*)job)->statResult();
+        // Update our URL in case of a redirection
+        setUrl(statJob->url());
+
+        const KIO::UDSEntry entry = statJob->statResult();
         const mode_t mode = entry.numberValue(KIO::UDSEntry::UDS_FILE_TYPE);
         if (S_ISDIR(mode)) {
             d->m_bIsDirectory = true; // it's a dir
@@ -1385,16 +1414,21 @@ void KRun::slotScanMimeType(KIO::Job *, const QString &mimetype)
 void KRun::slotScanFinished(KJob *job)
 {
     d->m_job = 0;
-    if (job->error()) {
-        d->m_showingDialog = true;
-        kError(7010) << this << "ERROR (stat):" << job->error() << ' ' << job->errorString();
-        job->uiDelegate()->showErrorMessage();
-        //kDebug(7010) << this << " KRun returning from showErrorDialog, starting timer to delete us";
-        d->m_showingDialog = false;
+    const int errCode = job->error();
+    if (errCode) {
+        // ERR_NO_CONTENT is not an error, but an indication no further
+        // actions needs to be taken.
+        if (errCode != KIO::ERR_NO_CONTENT) {
+            d->m_showingDialog = true;
+            kError(7010) << this << "ERROR (stat):" << job->error() << ' ' << job->errorString();
+            job->uiDelegate()->showErrorMessage();
+            //kDebug(7010) << this << " KRun returning from showErrorDialog, starting timer to delete us";
+            d->m_showingDialog = false;
+
+            d->m_bFault = true;
+        }
 
-        d->m_bFault = true;
         d->m_bFinished = true;
-
         // will emit the error and autodelete this
         d->startTimer();
     }
@@ -1431,16 +1465,11 @@ void KRun::foundMimeType(const QString& type)
 
     Q_ASSERT(!d->m_bFinished);
 
-    KMimeType::Ptr mime = KMimeType::mimeType(type, KMimeType::ResolveAliases);
-    if (!mime) {
-        kWarning(7010) << "Unknown mimetype " << type;
-    }
-
     // Support for preferred service setting, see setPreferredService
     if (!d->m_preferredService.isEmpty()) {
         kDebug(7010) << "Attempting to open with preferred service: " << d->m_preferredService;
         KService::Ptr serv = KService::serviceByDesktopName(d->m_preferredService);
-        if (serv && serv->hasMimeType(mime.data())) {
+        if (serv && serv->hasMimeType(type)) {
             KUrl::List lst;
             lst.append(d->m_strURL);
             if (KRun::run(*serv, lst, d->m_window, false, QString(), d->m_asn)) {
@@ -1455,6 +1484,10 @@ void KRun::foundMimeType(const QString& type)
     }
 
     // Resolve .desktop files from media:/, remote:/, applications:/ etc.
+    KMimeType::Ptr mime = KMimeType::mimeType(type, KMimeType::ResolveAliases);
+    if (!mime) {
+        kWarning(7010) << "Unknown mimetype " << type;
+    }
     if (mime && mime->is("application/x-desktop") && !d->m_localPath.isEmpty()) {
         d->m_strURL = KUrl();
         d->m_strURL.setPath(d->m_localPath);
@@ -1596,40 +1629,52 @@ KIO::Job* KRun::job()
     return d->m_job;
 }
 
+#ifndef KDE_NO_DEPRECATED
 QTimer& KRun::timer()
 {
     return d->m_timer;
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 void KRun::setDoScanFile(bool scanFile)
 {
     d->m_bScanFile = scanFile;
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 bool KRun::doScanFile() const
 {
     return d->m_bScanFile;
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 void KRun::setIsDirecory(bool isDirectory)
 {
     d->m_bIsDirectory = isDirectory;
 }
+#endif
 
 bool KRun::isDirectory() const
 {
     return d->m_bIsDirectory;
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KRun::setInitializeNextAction(bool initialize)
 {
     d->m_bInit = initialize;
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 bool KRun::initializeNextAction() const
 {
     return d->m_bInit;
 }
+#endif
 
 void KRun::setIsLocalFile(bool isLocalFile)
 {
@@ -1675,8 +1720,8 @@ KProcessRunner::KProcessRunner(KProcess * p, const QString & executable, const K
     m_pid = 0;
     process = p;
     m_executable = executable;
-    connect(process, SIGNAL(finished(int, QProcess::ExitStatus)),
-            this, SLOT(slotProcessExited(int, QProcess::ExitStatus)));
+    connect(process, SIGNAL(finished(int,QProcess::ExitStatus)),
+            this, SLOT(slotProcessExited(int,QProcess::ExitStatus)));
 
     process->start();
     if (!process->waitForStarted()) {
