@@ -25,9 +25,6 @@
 #include <kdebug.h>
 
 #include <sys/stat.h>
-#ifdef __FreeBSD__
-    #include <machine/param.h>
-#endif
 #include <sys/types.h>
 
 #ifdef Q_OS_UNIX
@@ -42,7 +39,6 @@
 #include <QtCore/QRegExp>
 
 #include <kfileitem.h>
-#include <kapplication.h>
 #include <kde_file.h>
 #include <ktemporaryfile.h>
 #include <kservicetypetrader.h>
@@ -52,6 +48,7 @@
 #include <kservice.h>
 #include <QtCore/QLinkedList>
 #include <kconfiggroup.h>
+#include <kprotocolinfo.h>
 
 #include "jobuidelegate.h"
 #include "job_p.h"
@@ -76,6 +73,8 @@ public:
 
     KFileItemList initialItems;
     QStringList enabledPlugins;
+    // Some plugins support remote URLs, <protocol, mimetypes>
+    QHash<QString, QStringList> m_remoteProtocolPlugins;
     // Our todo list :)
     // We remove the first item at every step, so use QLinkedList
     QLinkedList<PreviewItem> items;
@@ -131,6 +130,7 @@ public:
     Q_DECLARE_PUBLIC(PreviewJob)
 };
 
+#ifndef KDE_NO_DEPRECATED
 PreviewJob::PreviewJob( const KFileItemList &items, int width, int height,
     int iconSize, int iconAlpha, bool scale, bool save,
     const QStringList *enabledPlugins )
@@ -141,7 +141,7 @@ PreviewJob::PreviewJob( const KFileItemList &items, int width, int height,
     d->shmid = -1;
     d->shmaddr = 0;
     d->initialItems = items;
-    d->enabledPlugins = enabledPlugins ? *enabledPlugins : QStringList();
+    d->enabledPlugins = enabledPlugins ? *enabledPlugins : availablePlugins();
     d->width = width;
     d->height = height ? height : width;
     d->cacheWidth = d->width;
@@ -151,7 +151,46 @@ PreviewJob::PreviewJob( const KFileItemList &items, int width, int height,
     d->bScale = scale;
     d->bSave = save && scale;
     d->succeeded = false;
-    d->thumbRoot = QDir::homePath() + "/.thumbnails/";
+    d->thumbRoot = QDir::homePath() + QLatin1String("/.thumbnails/");
+    d->ignoreMaximumSize = false;
+    d->sequenceIndex = 0;
+    d->maximumLocalSize = 0;
+    d->maximumRemoteSize = 0;
+
+    // Return to event loop first, determineNextFile() might delete this;
+    QTimer::singleShot(0, this, SLOT(startPreview()));
+}
+#endif
+
+PreviewJob::PreviewJob(const KFileItemList &items,
+                       const QSize &size,
+                       const QStringList *enabledPlugins) :
+    KIO::Job(*new PreviewJobPrivate)
+{
+    Q_D(PreviewJob);
+    d->tOrig = 0;
+    d->shmid = -1;
+    d->shmaddr = 0;
+    d->initialItems = items;
+    if (enabledPlugins) {
+        d->enabledPlugins = *enabledPlugins;
+    } else {
+        const KConfigGroup globalConfig(KGlobal::config(), "PreviewSettings");
+        d->enabledPlugins = globalConfig.readEntry("Plugins", QStringList()
+                                                              << "directorythumbnail"
+                                                              << "imagethumbnail"
+                                                              << "jpegthumbnail");
+    }
+    d->width = size.width();
+    d->height = size.height();
+    d->cacheWidth = d->width;
+    d->cacheHeight = d->height;
+    d->iconSize = 0;
+    d->iconAlpha = 70;
+    d->bScale = true;
+    d->bSave = true;
+    d->succeeded = false;
+    d->thumbRoot = QDir::homePath() + QLatin1String("/.thumbnails/");
     d->ignoreMaximumSize = false;
     d->sequenceIndex = 0;
     d->maximumLocalSize = 0;
@@ -172,20 +211,96 @@ PreviewJob::~PreviewJob()
 #endif
 }
 
+void PreviewJob::setOverlayIconSize(int size)
+{
+    Q_D(PreviewJob);
+    d->iconSize = size;
+}
+
+int PreviewJob::overlayIconSize() const
+{
+    Q_D(const PreviewJob);
+    return d->iconSize;
+}
+
+void PreviewJob::setOverlayIconAlpha(int alpha)
+{
+    Q_D(PreviewJob);
+    d->iconAlpha = qBound(0, alpha, 255);
+}
+
+int PreviewJob::overlayIconAlpha() const
+{
+    Q_D(const PreviewJob);
+    return d->iconAlpha;
+}
+
+void PreviewJob::setScaleType(ScaleType type)
+{
+    Q_D(PreviewJob);
+    switch (type) {
+    case Unscaled:
+        d->bScale = false;
+        d->bSave = false;
+        break;
+    case Scaled:
+        d->bScale = true;
+        d->bSave = false;
+        break;
+    case ScaledAndCached:
+        d->bScale = true;
+        d->bSave = true;
+        break;
+    default:
+        break;
+    }
+}
+
+PreviewJob::ScaleType PreviewJob::scaleType() const
+{
+    Q_D(const PreviewJob);
+    if (d->bScale) {
+        return d->bSave ? ScaledAndCached : Scaled;
+    }
+    return Unscaled;
+}
+
 void PreviewJobPrivate::startPreview()
 {
     Q_Q(PreviewJob);
     // Load the list of plugins to determine which mimetypes are supported
     const KService::List plugins = KServiceTypeTrader::self()->query("ThumbCreator");
     QMap<QString, KService::Ptr> mimeMap;
-
+    QHash<QString, QHash<QString, KService::Ptr> > protocolMap;
     for (KService::List::ConstIterator it = plugins.constBegin(); it != plugins.constEnd(); ++it) {
-        if (enabledPlugins.isEmpty() || enabledPlugins.contains((*it)->desktopEntryName()))
-    {
-        const QStringList mimeTypes = (*it)->serviceTypes();
-        for (QStringList::ConstIterator mt = mimeTypes.constBegin(); mt != mimeTypes.constEnd(); ++mt)
-            mimeMap.insert(*mt, *it);
-    }
+        QStringList protocols = (*it)->property("X-KDE-Protocols").toStringList();
+        const QString p = (*it)->property("X-KDE-Protocol").toString();
+        if (!p.isEmpty()) {
+            protocols.append(p);
+        }
+        foreach (const QString &protocol, protocols) {
+            QStringList mtypes = (*it)->serviceTypes();
+            // Filter out non-mimetype servicetypes
+            // TODO KDE5: use KService::mimeTypes()
+            foreach (const QString &_mtype, mtypes) {
+                if (!((*it)->hasMimeType(_mtype))) {
+                    mtypes.removeAll(_mtype);
+                }
+            }
+            // Add supported mimetype for this protocol
+            QStringList &_ms = m_remoteProtocolPlugins[protocol];
+            foreach (const QString &_m, mtypes) {
+                protocolMap[protocol].insert(_m, *it);
+                if (!_ms.contains(_m)) {
+                    _ms.append(_m);
+                }
+            }
+        }
+        if (enabledPlugins.contains((*it)->desktopEntryName())) {
+            const QStringList mimeTypes = (*it)->serviceTypes();
+            for (QStringList::ConstIterator mt = mimeTypes.constBegin(); mt != mimeTypes.constEnd(); ++mt)
+                mimeMap.insert(*mt, *it);
+        }
     }
 
     // Look for images and store the items in our todo list :)
@@ -197,60 +312,50 @@ void PreviewJobPrivate::startPreview()
         PreviewItem item;
         item.item = *kit;
         const QString mimeType = item.item.mimetype();
-        QMap<QString, KService::Ptr>::ConstIterator plugin = mimeMap.constFind(mimeType);
-        if (plugin == mimeMap.constEnd())
+        KService::Ptr plugin(0);
 
-        {
-            QString groupMimeType = mimeType;
-            groupMimeType.replace(QRegExp("/.*"), "/*");
-            plugin = mimeMap.constFind(groupMimeType);
+        // look for protocol-specific thumbnail plugins first
+        QHash<QString, QHash<QString, KService::Ptr> >::const_iterator it = protocolMap.constFind(item.item.url().protocol());
+        if (it != protocolMap.constEnd()) {
+            plugin = it.value().value(mimeType);
+        }
 
-            if (plugin == mimeMap.constEnd())
-            {
-                // check mime type inheritance, resolve aliases
-                const KMimeType::Ptr mimeInfo = KMimeType::mimeType(mimeType, KMimeType::ResolveAliases);
-                if (mimeInfo) {
-                    const QStringList parentMimeTypes = mimeInfo->allParentMimeTypes();
-                    Q_FOREACH(const QString& parentMimeType, parentMimeTypes) {
-                        plugin = mimeMap.constFind(parentMimeType);
-                        if (plugin != mimeMap.constEnd()) break;
-                    }
-                }
-            }
-#if 0 // KDE4: should be covered by inheritance above, all text mimetypes inherit from text/plain
-            // if that's not enough, we need to invent something else
-            if (plugin == mimeMap.end())
-            {
-                // check X-KDE-Text property
-                KMimeType::Ptr mimeInfo = KMimeType::mimeType(mimeType);
-                QVariant textProperty = mimeInfo->property("X-KDE-text");
-                if (textProperty.isValid() && textProperty.type() == QVariant::Bool)
-                {
-                    if (textProperty.toBool())
-                    {
-                        plugin = mimeMap.find("text/plain");
-                        if (plugin == mimeMap.end())
-                        {
-                            plugin = mimeMap.find( "text/*" );
+        if (!plugin) {
+            QMap<QString, KService::Ptr>::ConstIterator pluginIt = mimeMap.constFind(mimeType);
+            if (pluginIt == mimeMap.constEnd()) {
+                QString groupMimeType = mimeType;
+                groupMimeType.replace(QRegExp("/.*"), "/*");
+                pluginIt = mimeMap.constFind(groupMimeType);
+
+                if (pluginIt == mimeMap.constEnd()) {
+                    // check mime type inheritance, resolve aliases
+                    const KMimeType::Ptr mimeInfo = KMimeType::mimeType(mimeType);
+                    if (mimeInfo) {
+                        const QStringList parentMimeTypes = mimeInfo->allParentMimeTypes();
+                        Q_FOREACH(const QString& parentMimeType, parentMimeTypes) {
+                            pluginIt = mimeMap.constFind(parentMimeType);
+                            if (pluginIt != mimeMap.constEnd())
+                                break;
                         }
                     }
                 }
             }
-#endif
+
+            if (pluginIt != mimeMap.constEnd()) {
+                plugin = *pluginIt;
+            }
         }
 
-        if (plugin != mimeMap.constEnd())
-        {
-            item.plugin = *plugin;
+        if (plugin) {
+            item.plugin = plugin;
             items.append(item);
             if (!bNeedCache && bSave &&
                 ((*kit).url().protocol() != "file" ||
                  !(*kit).url().directory( KUrl::AppendTrailingSlash ).startsWith(thumbRoot)) &&
-                (*plugin)->property("CacheThumbnail").toBool())
+                plugin->property("CacheThumbnail").toBool()) {
                 bNeedCache = true;
-        }
-        else
-        {
+            }
+        } else {
             emit q->failed( *kit );
         }
     }
@@ -353,8 +458,9 @@ void PreviewJob::slotResult( KJob *job )
 
             bool skipCurrentItem = false;
             const KIO::filesize_t size = (KIO::filesize_t)entry.numberValue( KIO::UDSEntry::UDS_SIZE, 0 );
-            bool local = false;
-            if (d->currentItem.item.mostLocalUrl(local).isLocalFile())
+            const KUrl itemUrl = d->currentItem.item.mostLocalUrl();
+
+            if (itemUrl.isLocalFile() || KProtocolInfo::protocolClass(itemUrl.protocol()) == QLatin1String(":local"))
             {
                 skipCurrentItem = !d->ignoreMaximumSize && size > d->maximumLocalSize
                                   && !d->currentItem.plugin->property("IgnoreMaximumSize").toBool();
@@ -380,7 +486,6 @@ void PreviewJob::slotResult( KJob *job )
             }
 
             bool pluginHandlesSequences = d->currentItem.plugin->property("HandleSequences", QVariant::Bool).toBool();
-
             if ( !d->currentItem.plugin->property( "CacheThumbnail" ).toBool()  || (d->sequenceIndex && pluginHandlesSequences) )
             {
                 // This preview will not be cached, no need to look for a saved thumbnail
@@ -424,8 +529,7 @@ bool PreviewJobPrivate::statResultThumbnail()
     if ( thumbPath.isEmpty() )
         return false;
 
-    bool local = false;
-    KUrl url = currentItem.item.mostLocalUrl(local);
+    KUrl url = currentItem.item.mostLocalUrl();
     // Don't include the password if any
     url.setPass(QString());
     origName = url.url();
@@ -477,18 +581,34 @@ void PreviewJobPrivate::getOrCreateThumbnail()
     // We still need to load the orig file ! (This is getting tedious) :)
     const KFileItem& item = currentItem.item;
     const QString localPath = item.localPath();
-    if ( !localPath.isEmpty() )
+    if (!localPath.isEmpty()) {
         createThumbnail( localPath );
-    else
-    {
+    } else {
+        const KUrl fileUrl = item.url();
+        // heuristics for remote URL support
+        bool supportsProtocol = false;
+        if (m_remoteProtocolPlugins.value(fileUrl.scheme()).contains(item.mimetype())) {
+            // There's a plugin supporting this protocol and mimetype
+            supportsProtocol = true;
+        } else if (m_remoteProtocolPlugins.value("KIO").contains(item.mimetype())) {
+            // Assume KIO understands any URL, ThumbCreator slaves who have
+            // X-KDE-Protocols=KIO will get fed the remote URL directly.
+            supportsProtocol = true;
+        }
+
+        if (supportsProtocol) {
+            createThumbnail(fileUrl.url());
+            return;
+        }
+        // No plugin support access to this remote content, copy the file
+        // to the local machine, then create the thumbnail
         state = PreviewJobPrivate::STATE_GETORIG;
         KTemporaryFile localFile;
         localFile.setAutoRemove(false);
         localFile.open();
         KUrl localURL;
         localURL.setPath( tempName = localFile.fileName() );
-        bool local = false;
-        const KUrl currentURL = item.mostLocalUrl(local);
+        const KUrl currentURL = item.mostLocalUrl();
         KIO::Job * job = KIO::file_copy( currentURL, localURL, -1, KIO::Overwrite | KIO::HideProgressInfo /* No GUI */ );
         job->addMetaData("thumbnail","1");
         q->addSubjob(job);
@@ -504,7 +624,7 @@ void PreviewJobPrivate::createThumbnail( const QString &pixPath )
     thumbURL.setPath(pixPath);
     KIO::TransferJob *job = KIO::get(thumbURL, NoReload, HideProgressInfo);
     q->addSubjob(job);
-    q->connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)), SLOT(slotThumbData(KIO::Job *, const QByteArray &)));
+    q->connect(job, SIGNAL(data(KIO::Job*,QByteArray)), SLOT(slotThumbData(KIO::Job*,QByteArray)));
     bool save = bSave && currentItem.plugin->property("CacheThumbnail").toBool() && !sequenceIndex;
     job->addMetaData("mimeType", currentItem.item.mimetype());
     job->addMetaData("width", QString().setNum(save ? cacheWidth : width));
@@ -631,6 +751,7 @@ QStringList PreviewJob::supportedMimeTypes()
     return result;
 }
 
+#ifndef KDE_NO_DEPRECATED
 PreviewJob *KIO::filePreview( const KFileItemList &items, int width, int height,
     int iconSize, int iconAlpha, bool scale, bool save,
     const QStringList *enabledPlugins )
@@ -651,11 +772,19 @@ PreviewJob *KIO::filePreview( const KUrl::List &items, int width, int height,
     return new PreviewJob(fileItems, width, height, iconSize, iconAlpha,
                           scale, save, enabledPlugins);
 }
+#endif
 
+PreviewJob *KIO::filePreview(const KFileItemList &items, const QSize &size, const QStringList *enabledPlugins)
+{
+    return new PreviewJob(items, size, enabledPlugins);
+}
+
+#ifndef KDE_NO_DEPRECATED
 KIO::filesize_t PreviewJob::maximumFileSize()
 {
     KConfigGroup cg( KGlobal::config(), "PreviewSettings" );
     return cg.readEntry( "MaximumSize", 5*1024*1024LL /* 5MB */ );
 }
+#endif
 
 #include "previewjob.moc"
