@@ -28,35 +28,22 @@
 
 #include <config.h>
 
-#include <sys/types.h>
-#include <sys/uio.h>
-#include <sys/time.h>
-#include <sys/socket.h>
-
-#include <netinet/in.h>
-
-#include <time.h>
-#include <netdb.h>
-#include <unistd.h>
-#include <errno.h>
-
 #include <kdebug.h>
+#include <kconfiggroup.h>
 #include <ksslcertificatemanager.h>
 #include <ksslsettings.h>
 #include <kmessagebox.h>
+#include <klocale.h>
+#include <ktoolinvocation.h>
 #include <network/ktcpsocket.h>
 
-#include <klocale.h>
 #include <QtCore/QDataStream>
 #include <QtCore/QTime>
 #include <QtNetwork/QTcpSocket>
 #include <QtNetwork/QHostInfo>
+#include <QtNetwork/QSslConfiguration>
 #include <QtDBus/QtDBus>
 
-#include <kapplication.h>
-#include <ktoolinvocation.h>
-#include <ksocketfactory.h>
-#include <kprotocolmanager.h>
 
 using namespace KIO;
 //using namespace KNetwork;
@@ -155,14 +142,14 @@ public:
         sslMetaData.insert("ssl_peer_chain", peerCertChain);
         sendSslMetaData();
     }
-    
+
     void clearSslMetaData()
     {
         sslMetaData.clear();
         sslMetaData.insert("ssl_in_use", "FALSE");
         sendSslMetaData();
     }
-    
+
     void sendSslMetaData()
     {
         MetaData::ConstIterator it = sslMetaData.constBegin();
@@ -171,6 +158,10 @@ public:
         }
     }
 
+    SslResult startTLSInternal(KTcpSocket::SslVersion sslVersion,
+                               const QSslConfiguration& configuration = QSslConfiguration(),
+                               int waitForEncryptedTimeout = -1);
+
     TCPSlaveBase* q;
 
     bool isBlocking;
@@ -188,7 +179,7 @@ public:
     bool sslNoUi; // If true, we just drop the connection silently
                   // if SSL certificate check fails in some way.
     QList<KSslError> sslErrors;
-    
+
     MetaData sslMetaData;
 };
 
@@ -267,18 +258,19 @@ ssize_t TCPSlaveBase::read(char* data, ssize_t len)
     }
 
     if (!d->socket.bytesAvailable()) {
-        if (d->isBlocking) {
-            d->socket.waitForReadyRead(-1);
-        } else {
-            d->socket.waitForReadyRead(readTimeout());
-        }
-    } else if (d->socket.encryptionMode() != KTcpSocket::SslClientMode ||
+        const int timeout = d->isBlocking ? -1 : (readTimeout() * 1000);
+        d->socket.waitForReadyRead(timeout);
+    }
+#if 0
+    // Do not do this because its only benefit is to cause a nasty side effect
+    // upstream in Qt. See BR# 260769.
+    else if (d->socket.encryptionMode() != KTcpSocket::SslClientMode ||
                QNetworkProxy::applicationProxy().type() == QNetworkProxy::NoProxy) {
         // we only do this when it doesn't trigger Qt socket bugs. When it doesn't break anything
         // it seems to help performance.
         d->socket.waitForReadyRead(0);
     }
-
+#endif
     return d->socket.read(data, len);
 }
 
@@ -291,7 +283,7 @@ ssize_t TCPSlaveBase::readLine(char *data, ssize_t len)
         return -1;
     }
 
-    const int timeout = (d->isBlocking ? -1: readTimeout());
+    const int timeout = (d->isBlocking ? -1: (readTimeout() * 1000));
     ssize_t readTotal = 0;
     do {
         if (!d->socket.bytesAvailable())
@@ -310,9 +302,26 @@ ssize_t TCPSlaveBase::readLine(char *data, ssize_t len)
 bool TCPSlaveBase::connectToHost(const QString &/*protocol*/,
                                  const QString &host,
                                  quint16 port)
+{
+    QString errorString;
+    const int errCode = connectToHost(host, port, &errorString);
+    if (errCode == 0)
+        return true;
+
+    error(errCode, errorString);
+    return false;
+}
+
+int TCPSlaveBase::connectToHost(const QString& host, quint16 port, QString* errorString)
 {
     d->clearSslMetaData(); //We have separate connection and SSL setup phases
 
+    if (errorString) {
+        errorString->clear();  // clear prior error messages.
+    }
+
+    d->socket.setVerificationPeerName(host); // Used for ssl certificate verification (SNI)
+
     //  - leaving SSL - warn before we even connect
     //### see if it makes sense to move this into the HTTP ioslave which is the only
     //    user.
@@ -333,49 +342,65 @@ bool TCPSlaveBase::connectToHost(const QString &/*protocol*/,
                                     "WarnOnLeaveSSLMode");
 
             if (result == KMessageBox::Cancel) {
-                error(ERR_USER_CANCELED, host);
-                return false;
+                if (errorString)
+                    *errorString = host;
+                return ERR_USER_CANCELED;
             }
         }
     }
 
-    KTcpSocket::SslVersion trySslVersion = KTcpSocket::TlsV1;
+    /*
+      By default the SSL handshake attempt uses these settings in the order shown:
+
+      1.) Protocol: KTcpSocket::SecureProtocols   SSL compression: ON  (DEFAULT)
+      2.) Protocol: KTcpSocket::SecureProtocols   SSL compression: OFF
+      3.) Protocol: KTcpSocket::TlsV1             SSL compression: ON
+      4.) Protocol: KTcpSocket::TlsV1             SSL compression: OFF
+      5.) Protocol: KTcpSocket::SslV3             SSL compression: ON
+      6.) Protocol: KTcpSocket::SslV3             SSL compression: OFF
+
+      If any combination other than the one marked DEFAULT is used to complete
+      the SSL handshake, then that combination will be cached using KIO's internal
+      meta-data mechanism in order to speed up future connections to the same host.
+    */
+
+    QSslConfiguration sslConfig = d->socket.sslConfiguration();
+#if QT_VERSION >= 0x040800
+    const bool isSslCompressionDisabled = sslConfig.testSslOption(QSsl::SslOptionDisableCompression);
+    const bool shouldSslCompressBeDisabled = config()->readEntry("LastUsedSslDisableCompressionFlag", isSslCompressionDisabled);
+    sslConfig.setSslOption(QSsl::SslOptionDisableCompression, shouldSslCompressBeDisabled);
+#endif
+    
+    const int lastSslVerson = config()->readEntry("LastUsedSslVersion", static_cast<int>(KTcpSocket::SecureProtocols));
+    KTcpSocket::SslVersion trySslVersion = static_cast<KTcpSocket::SslVersion>(lastSslVerson);
+    KTcpSocket::SslVersions alreadyTriedSslVersions = trySslVersion;
+
+    const int timeout = (connectTimeout() * 1000);
     while (true) {
         disconnectFromHost();  //Reset some state, even if we are already disconnected
         d->host = host;
 
-        //FIXME! KTcpSocket doesn't know or care about protocol ports! Fix it there, then use it here.
+        d->socket.connectToHost(host, port);
+        const bool connectOk = d->socket.waitForConnected(timeout > -1 ? timeout : -1);
 
-        QList<QHostAddress> addresses;
+        kDebug(7027) << "Socket: state=" << d->socket.state()
+                     << ", error=" << d->socket.error()
+                     << ", connected?" << connectOk;
 
-        QHostAddress address;
-        if (address.setAddress(host)) {
-            addresses.append(address);
-        } else {
-            QHostInfo info;
-            lookupHost(host);
-            waitForHostInfo(info);
-            if (info.error() != QHostInfo::NoError) {
-                error(ERR_UNKNOWN_HOST, host);
-                return false;
-            }
-            addresses = info.addresses();
-        }
-
-        QListIterator<QHostAddress> it(addresses);
-        int timeout = connectTimeout() * 1000;
-        QTime time;
-        forever {
-            time.start();
-            d->socket.connectToHost(it.next(), port);
-            if (d->socket.waitForConnected(timeout)) {
-                break;
-            }
-            timeout -= time.elapsed();
-            if (!it.hasNext() || (timeout < 0)) {
-                error(ERR_COULD_NOT_CONNECT,
-                      host + QLatin1String(": ") + d->socket.errorString());
-                return false;
+        if (d->socket.state() != KTcpSocket::ConnectedState) {
+            if (errorString)
+                *errorString = host + QLatin1String(": ") + d->socket.errorString();
+            switch (d->socket.error()) {
+            case KTcpSocket::UnsupportedSocketOperationError:
+                return ERR_UNSUPPORTED_ACTION;
+            case KTcpSocket::RemoteHostClosedError:
+                return ERR_CONNECTION_BROKEN;
+            case KTcpSocket::SocketTimeoutError:
+                return ERR_SERVER_TIMEOUT;
+            case KTcpSocket::HostNotFoundError:
+                return ERR_UNKNOWN_HOST;
+            default:
+                return ERR_COULD_NOT_CONNECT;
             }
         }
 
@@ -385,22 +410,71 @@ bool TCPSlaveBase::connectToHost(const QString &/*protocol*/,
         d->port = d->socket.peerPort();
 
         if (d->autoSSL) {
-            SslResult res = startTLSInternal(trySslVersion);
-            if ((res & ResultFailed) && (res & ResultFailedEarly)
-                && (trySslVersion == KTcpSocket::TlsV1)) {
-                trySslVersion = KTcpSocket::SslV3;
-                continue;
-                //### SSL 2.0 is (close to) dead and it's a good thing, too.
+            SslResult res = d->startTLSInternal(trySslVersion, sslConfig, 30000 /*30 secs timeout*/);
+            if ((res & ResultFailed) && (res & ResultFailedEarly)) {
+#if QT_VERSION >= 0x040800
+                if (!sslConfig.testSslOption(QSsl::SslOptionDisableCompression)) {
+                    sslConfig.setSslOption(QSsl::SslOptionDisableCompression, true);
+                    continue;
+                }
+#endif
+
+                if (!(alreadyTriedSslVersions & KTcpSocket::SecureProtocols)) {
+                    trySslVersion = KTcpSocket::SecureProtocols;
+                    alreadyTriedSslVersions |= trySslVersion;
+#if QT_VERSION >= 0x040800
+                    sslConfig.setSslOption(QSsl::SslOptionDisableCompression, false);
+#endif
+                    continue;
+                }
+
+                if (!(alreadyTriedSslVersions & KTcpSocket::TlsV1)) {
+                    trySslVersion = KTcpSocket::TlsV1;
+                    alreadyTriedSslVersions |= trySslVersion;
+#if QT_VERSION >= 0x040800
+                    sslConfig.setSslOption(QSsl::SslOptionDisableCompression, false);
+#endif
+                    continue;
+                }
+
+                if (!(alreadyTriedSslVersions & KTcpSocket::SslV3)) {
+                    trySslVersion = KTcpSocket::SslV3;
+                    alreadyTriedSslVersions |= trySslVersion;
+#if QT_VERSION >= 0x040800
+                    sslConfig.setSslOption(QSsl::SslOptionDisableCompression, false);
+#endif
+                    continue;
+                }
             }
+
+            //### SSL 2.0 is (close to) dead and it's a good thing, too.
             if (res & ResultFailed) {
-                error(ERR_COULD_NOT_CONNECT,
-                      i18nc("%1 is a host name", "%1: SSL negotiation failed", host));
-                return false;
+                if (errorString)
+                    *errorString = i18nc("%1 is a host name", "%1: SSL negotiation failed", host);
+                return ERR_COULD_NOT_CONNECT;
             }
         }
-        return true;
+        // If the SSL handshake was done with anything protocol other than the default,
+        // save that information so that any subsequent requests do not have to do thesame thing.
+        if (trySslVersion != KTcpSocket::SecureProtocols && lastSslVerson == KTcpSocket::SecureProtocols) {
+            setMetaData(QLatin1String("{internal~currenthost}LastUsedSslVersion"),
+                        QString::number(trySslVersion));
+        }
+#if QT_VERSION >= 0x040800
+        if (sslConfig.testSslOption(QSsl::SslOptionDisableCompression) && !shouldSslCompressBeDisabled) {
+            setMetaData(QLatin1String("{internal~currenthost}LastUsedSslDisableCompressionFlag"),
+                        QString::number(true));
+        }
+#endif
+        return 0;
     }
     Q_ASSERT(false);
+    // Code flow never gets here but let's make the compiler happy.
+    // More: the stack allocation of QSslSettings seems to be confusing the compiler;
+    //       in fact, any non-POD allocation does. 
+    //       even a 'return 0;' directly after the allocation (so before the while(true))
+    //       is ignored. definitely seems to be a compiler bug? - aseigo
+    return 0;
 }
 
 void TCPSlaveBase::disconnectFromHost()
@@ -450,7 +524,7 @@ bool TCPSlaveBase::startSsl()
 {
     if (d->usingSSL)
         return false;
-    return startTLSInternal(KTcpSocket::TlsV1) & ResultOk;
+    return d->startTLSInternal(KTcpSocket::TlsV1) & ResultOk;
 }
 
 // Find out if a hostname matches an SSL certificate's Common Name (including wildcards)
@@ -497,46 +571,57 @@ static bool isMatchingHostname(const QString &cnIn, const QString &hostnameIn)
     return true;
 }
 
-TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
+TCPSlaveBase::SslResult TCPSlaveBase::TcpSlaveBasePrivate::startTLSInternal (KTcpSocket::SslVersion version,
+                                                                             const QSslConfiguration& sslConfig,
+                                                                             int waitForEncryptedTimeout)
 {
-    KTcpSocket::SslVersion sslVersion = static_cast<KTcpSocket::SslVersion>(v_);
-    selectClientCertificate();
+    q->selectClientCertificate();
 
     //setMetaData("ssl_session_id", d->kssl->session()->toString());
     //### we don't support session reuse for now...
+    usingSSL = true;
+#if QT_VERSION >= 0x040800
+    kDebug(7027) << "Trying SSL handshake with protocol:" << version
+                 << ", SSL compression ON:" << sslConfig.testSslOption(QSsl::SslOptionDisableCompression);
+#endif
+    // Set the SSL version to use...
+    socket.setAdvertisedSslVersion(version);
 
-    d->usingSSL = true;
-
-    d->socket.setAdvertisedSslVersion(sslVersion);
+    // Set SSL configuration information
+    if (!sslConfig.isNull())
+        socket.setSslConfiguration(sslConfig);
 
     /* Usually ignoreSslErrors() would be called in the slot invoked by the sslErrors()
        signal but that would mess up the flow of control. We will check for errors
        anyway to decide if we want to continue connecting. Otherwise ignoreSslErrors()
        before connecting would be very insecure. */
-    d->socket.ignoreSslErrors();
-    d->socket.startClientEncryption();
-    const bool encryptionStarted = d->socket.waitForEncrypted(-1);
+    socket.ignoreSslErrors();
+    socket.startClientEncryption();
+    const bool encryptionStarted = socket.waitForEncrypted(waitForEncryptedTimeout);
 
     //Set metadata, among other things for the "SSL Details" dialog
-    KSslCipher cipher = d->socket.sessionCipher();
+    KSslCipher cipher = socket.sessionCipher();
 
-    if (!encryptionStarted || d->socket.encryptionMode() != KTcpSocket::SslClientMode
-        || cipher.isNull() || cipher.usedBits() == 0 || d->socket.peerCertificateChain().isEmpty()) {
-        d->usingSSL = false;
-        d->clearSslMetaData();
+    if (!encryptionStarted || socket.encryptionMode() != KTcpSocket::SslClientMode
+        || cipher.isNull() || cipher.usedBits() == 0 || socket.peerCertificateChain().isEmpty()) {
+        usingSSL = false;
+        clearSslMetaData();
         kDebug(7029) << "Initial SSL handshake failed. encryptionStarted is"
                      << encryptionStarted << ", cipher.isNull() is" << cipher.isNull()
                      << ", cipher.usedBits() is" << cipher.usedBits()
-                     << ", length of certificate chain is" << d->socket.peerCertificateChain().count()
-                     << ", the socket says:" << d->socket.errorString()
+                     << ", length of certificate chain is" << socket.peerCertificateChain().count()
+                     << ", the socket says:" << socket.errorString()
                      << "and the list of SSL errors contains"
-                     << d->socket.sslErrors().count() << "items.";
+                     << socket.sslErrors().count() << "items.";
+        Q_FOREACH(const KSslError& sslError, socket.sslErrors()) {
+            kDebug(7029) << "SSL ERROR: (" << sslError.error() << ")" << sslError.errorString();
+        }
         return ResultFailed | ResultFailedEarly;
     }
 
     kDebug(7029) << "Cipher info - "
-                 << " advertised SSL protocol version" << d->socket.advertisedSslVersion()
-                 << " negotiated SSL protocol version" << d->socket.negotiatedSslVersion()
+                 << " advertised SSL protocol version" << socket.advertisedSslVersion()
+                 << " negotiated SSL protocol version" << socket.negotiatedSslVersion()
                  << " authenticationMethod:" << cipher.authenticationMethod()
                  << " encryptionMethod:" << cipher.encryptionMethod()
                  << " keyExchangeMethod:" << cipher.keyExchangeMethod()
@@ -547,25 +632,36 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
     // Since we connect by IP (cf. KIO::HostInfo) the SSL code will not recognize
     // that the site certificate belongs to the domain. We therefore do the
     // domain<->certificate matching here.
-    d->sslErrors = d->socket.sslErrors();
-    QSslCertificate peerCert = d->socket.peerCertificateChain().first();
-    QStringList domainPatterns(peerCert.subjectInfo(QSslCertificate::CommonName));
-    domainPatterns += peerCert.alternateSubjectNames().values(QSsl::DnsEntry);
-    QMutableListIterator<KSslError> it(d->sslErrors);
+    sslErrors = socket.sslErrors();
+    QSslCertificate peerCert = socket.peerCertificateChain().first();
+    QMutableListIterator<KSslError> it(sslErrors);
     while (it.hasNext()) {
         // As of 4.4.0 Qt does not assign a certificate to the QSslError it emits
         // *in the case of HostNameMismatch*. A HostNameMismatch, however, will always
         // be an error of the peer certificate so we just don't check the error's
         // certificate().
-        if (it.next().error() != KSslError::HostNameMismatch) {
-            continue;
+
+        // Remove all HostNameMismatch, we have to redo name checking later.
+        if (it.next().error() == KSslError::HostNameMismatch) {
+            it.remove();
         }
-        Q_FOREACH (const QString &dp, domainPatterns) {
-            if (isMatchingHostname(dp, d->host)) {
-                it.remove();
-            }
+    }
+    // Redo name checking here and (re-)insert HostNameMismatch to sslErrors if
+    // host name does not match any of the names in server certificate.
+    // QSslSocket may not report HostNameMismatch error, when server
+    // certificate was issued for the IP we are connecting to.
+    QStringList domainPatterns(peerCert.subjectInfo(QSslCertificate::CommonName));
+    domainPatterns += peerCert.alternateSubjectNames().values(QSsl::DnsEntry);
+    bool names_match = false;
+    foreach (const QString &dp, domainPatterns) {
+        if (isMatchingHostname(dp, host)) {
+            names_match = true;
+            break;
         }
     }
+    if (!names_match) {
+        sslErrors.insert(0, KSslError(KSslError::HostNameMismatch, peerCert));
+    }
 
     // TODO: review / rewrite / remove the comment
     // The app side needs the metadata now for the SSL error dialog (if any) but
@@ -576,26 +672,26 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
     // from here, for example. And Konqi will be the second application to connect
     // to the slave.
     // Therefore we choose to have our metadata and send it, too :)
-    d->setSslMetaData();
-    sendAndKeepMetaData();
+    setSslMetaData();
+    q->sendAndKeepMetaData();
 
-    SslResult rc = verifyServerCertificate();
+    SslResult rc = q->verifyServerCertificate();
     if (rc & ResultFailed) {
-        d->usingSSL = false;
-        d->clearSslMetaData();
+        usingSSL = false;
+        clearSslMetaData();
         kDebug(7029) << "server certificate verification failed.";
-        d->socket.disconnectFromHost();     //Make the connection fail (cf. ignoreSslErrors())
+        socket.disconnectFromHost();     //Make the connection fail (cf. ignoreSslErrors())
         return ResultFailed;
     } else if (rc & ResultOverridden) {
         kDebug(7029) << "server certificate verification failed but continuing at user's request.";
     }
 
     //"warn" when starting SSL/TLS
-    if (metaData("ssl_activate_warnings") == "TRUE"
-        && metaData("ssl_was_in_use") == "FALSE"
-        && d->sslSettings.warnOnEnter()) {
+    if (q->metaData("ssl_activate_warnings") == "TRUE"
+        && q->metaData("ssl_was_in_use") == "FALSE"
+        && sslSettings.warnOnEnter()) {
 
-        int msgResult = messageBox(i18n("You are about to enter secure mode. "
+        int msgResult = q->messageBox(i18n("You are about to enter secure mode. "
                                         "All transmissions will be encrypted "
                                         "unless otherwise noted.\nThis means "
                                         "that no third party will be able to "
@@ -606,7 +702,7 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
                                    i18n("C&onnect"),
                                    "WarnOnEnterSSLMode");
         if (msgResult == KMessageBox::Yes) {
-            messageBox(SSLMessageBox /*==the SSL info dialog*/, d->host);
+            q->messageBox(SSLMessageBox /*==the SSL info dialog*/, host);
         }
     }
 
