@@ -37,7 +37,7 @@ QTEST_KDEMAIN( KDirListerTest, NoGUI )
 void MyDirLister::handleError(KIO::Job* job)
 {
     // Currently we don't expect any errors.
-    kFatal() << job << job->errorString();
+    kFatal() << "KDirLister called handleError!" << job << job->error() << job->errorString();
 }
 
 void KDirListerTest::initTestCase()
@@ -101,12 +101,14 @@ void KDirListerTest::testOpenUrl()
     QCOMPARE(m_dirLister.spyClear.count(), 1);
     QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
     QCOMPARE(m_dirLister.spyRedirection.count(), 0);
+    //qDebug() << m_items;
+    //qDebug() << "In dir" << QDir(path).entryList( QDir::AllEntries | QDir::NoDotAndDotDot);
     QCOMPARE(m_items.count(), fileCount());
     QVERIFY(m_dirLister.isFinished());
     disconnect(&m_dirLister, 0, this, 0);
 
     const QString fileName = "toplevelfile_3";
-    const KUrl itemUrl = path + fileName;
+    const KUrl itemUrl(path + fileName);
     KFileItem byName = m_dirLister.findByName(fileName);
     QVERIFY(!byName.isNull());
     QCOMPARE(byName.url().url(), itemUrl.url());
@@ -122,7 +124,7 @@ void KDirListerTest::testOpenUrl()
 
     KFileItem rootByUrl = m_dirLister.findByUrl(path);
     QVERIFY(!rootByUrl.isNull());
-    QCOMPARE(rootByUrl.url().path() + '/', path);
+    QCOMPARE(QString(rootByUrl.url().toLocalFile() + '/'), path);
 
     m_dirLister.clearSpies(); // for the tests that call testOpenUrl for setup
 }
@@ -198,7 +200,7 @@ void KDirListerTest::testNewItems()
     QCOMPARE(m_dirLister.spyClear.count(), 0);
     QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
 
-    const KUrl itemUrl = path + fileName;
+    const KUrl itemUrl(path + fileName);
     KFileItem itemForUrl = KDirLister::cachedItemForUrl(itemUrl);
     QVERIFY(!itemForUrl.isNull());
     QCOMPARE(itemForUrl.url().url(), itemUrl.url());
@@ -218,7 +220,7 @@ void KDirListerTest::testNewItemByCopy()
 
     const QString fileName = "toplevelfile_copy";
     const KUrl itemUrl(path + fileName);
-    KIO::CopyJob* job = KIO::copyAs(path+"toplevelfile_3", itemUrl, KIO::HideProgressInfo);
+    KIO::CopyJob* job = KIO::copyAs(QString(path+"toplevelfile_3"), itemUrl, KIO::HideProgressInfo);
     job->exec();
 
     int numTries = 0;
@@ -316,7 +318,7 @@ void KDirListerTest::testNewItemsInSymlink() // #213799
         }
         QCOMPARE(dirLister2.spyDeleteItem.count(), 1);
         const KFileItem item = dirLister2.spyDeleteItem[0][0].value<KFileItem>();
-        QCOMPARE(item.url().path(), symPath + '/' + fileName);
+        QCOMPARE(item.url().toLocalFile(), QString(symPath + '/' + fileName));
     }
 
     // TODO: test file update.
@@ -334,8 +336,8 @@ void KDirListerTest::testRefreshItems()
     QVERIFY(!cachedItem.isNull());
     QCOMPARE(cachedItem.mimetype(), QString("application/octet-stream"));
 
-    connect(&m_dirLister, SIGNAL(refreshItems(const QList<QPair<KFileItem, KFileItem> > &)),
-            this, SLOT(slotRefreshItems(const QList<QPair<KFileItem, KFileItem> > &)));
+    connect(&m_dirLister, SIGNAL(refreshItems(QList<QPair<KFileItem,KFileItem> >)),
+            this, SLOT(slotRefreshItems(QList<QPair<KFileItem,KFileItem> >)));
 
     QFile file(fileName);
     QVERIFY(file.open(QIODevice::Append));
@@ -354,10 +356,10 @@ void KDirListerTest::testRefreshItems()
     QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
     QCOMPARE(m_refreshedItems.count(), 1);
     QPair<KFileItem, KFileItem> entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), fileName);
+    QCOMPARE(entry.first.url().toLocalFile(), fileName);
     QCOMPARE(entry.first.size(), KIO::filesize_t(11));
     QCOMPARE(entry.first.mimetype(), QString("application/octet-stream"));
-    QCOMPARE(entry.second.url().path(), fileName);
+    QCOMPARE(entry.second.url().toLocalFile(), fileName);
     QCOMPARE(entry.second.size(), KIO::filesize_t(11 /*Hello world*/ + 6 /*<html>*/));
     QCOMPARE(entry.second.mimetype(), QString("text/html"));
 
@@ -381,8 +383,8 @@ void KDirListerTest::testRefreshRootItem()
     MyDirLister dirLister2;
     fillDirLister2(dirLister2, path);
 
-    connect(&m_dirLister, SIGNAL(refreshItems(const QList<QPair<KFileItem, KFileItem> > &)),
-            this, SLOT(slotRefreshItems(const QList<QPair<KFileItem, KFileItem> > &)));
+    connect(&m_dirLister, SIGNAL(refreshItems(QList<QPair<KFileItem,KFileItem> >)),
+            this, SLOT(slotRefreshItems(QList<QPair<KFileItem,KFileItem> >)));
 
     org::kde::KDirNotify::emitFilesChanged(QStringList() << KUrl(path).url());
     waitForRefreshedItems();
@@ -396,15 +398,15 @@ void KDirListerTest::testRefreshRootItem()
     QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
     QCOMPARE(m_refreshedItems.count(), 1);
     QPair<KFileItem, KFileItem> entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), path);
     QCOMPARE(entry.first.name(), QString("subdir"));
-    QCOMPARE(entry.second.url().path(), path);
+    QCOMPARE(entry.second.url().toLocalFile(), path);
     QCOMPARE(entry.second.name(), QString("subdir"));
 
     QCOMPARE(m_refreshedItems2.count(), 1);
     entry = m_refreshedItems2.first();
-    QCOMPARE(entry.first.url().path(), path);
-    QCOMPARE(entry.second.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), path);
+    QCOMPARE(entry.second.url().toLocalFile(), path);
     // item name() doesn't matter here, it's the root item.
 
     m_refreshedItems.clear();
@@ -422,8 +424,8 @@ void KDirListerTest::testRefreshRootItem()
     waitForRefreshedItems();
     QCOMPARE(m_refreshedItems.count(), 1);
     entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), path);
-    QCOMPARE(entry.second.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), path);
+    QCOMPARE(entry.second.url().toLocalFile(), path);
 
     m_refreshedItems.clear();
     m_refreshedItems2.clear();
@@ -439,7 +441,7 @@ void KDirListerTest::testDeleteItem()
     const int origItemCount = m_items.count();
     QCOMPARE(fileCount(), origItemCount);
     const QString path = m_tempDir.name();
-    connect(&m_dirLister, SIGNAL(deleteItem(const KFileItem&)), this, SLOT(exitLoop()));
+    connect(&m_dirLister, SIGNAL(deleteItem(KFileItem)), this, SLOT(exitLoop()));
 
     //kDebug() << "Removing " << path+"toplevelfile_1";
     QFile::remove(path+"toplevelfile_1");
@@ -471,8 +473,8 @@ void KDirListerTest::testRenameItem()
 {
     m_refreshedItems.clear();
     const QString dirPath = m_tempDir.name();
-    connect(&m_dirLister, SIGNAL(refreshItems(const QList<QPair<KFileItem, KFileItem> > &)),
-            this, SLOT(slotRefreshItems(const QList<QPair<KFileItem, KFileItem> > &)));
+    connect(&m_dirLister, SIGNAL(refreshItems(QList<QPair<KFileItem,KFileItem> >)),
+            this, SLOT(slotRefreshItems(QList<QPair<KFileItem,KFileItem> >)));
     const QString path = dirPath+"toplevelfile_2";
     const QString newPath = dirPath+"toplevelfile_2.renamed.html";
 
@@ -486,16 +488,16 @@ void KDirListerTest::testRenameItem()
 
     QCOMPARE(m_refreshedItems.count(), 1);
     QPair<KFileItem, KFileItem> entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), path);
     QCOMPARE(entry.first.mimetype(), QString("application/octet-stream"));
-    QCOMPARE(entry.second.url().path(), newPath);
+    QCOMPARE(entry.second.url().toLocalFile(), newPath);
     QCOMPARE(entry.second.mimetype(), QString("text/html"));
     disconnect(&m_dirLister, 0, this, 0);
 
     // Let's see what KDirLister has in cache now
     KFileItem cachedItem = m_dirLister.findByUrl(KUrl(newPath));
     QVERIFY(!cachedItem.isNull());
-    QCOMPARE(cachedItem.url().path(), newPath);
+    QCOMPARE(cachedItem.url().toLocalFile(), newPath);
     KFileItem oldCachedItem = m_dirLister.findByUrl(KUrl(path));
     QVERIFY(oldCachedItem.isNull());
     m_refreshedItems.clear();
@@ -515,11 +517,11 @@ void KDirListerTest::testRenameAndOverwrite() // has to be run after testRenameI
         QTest::qWait(100);
         existingItem = m_dirLister.findByUrl(KUrl(path));
     };
-    QCOMPARE(existingItem.url().path(), path);
+    QCOMPARE(existingItem.url().toLocalFile(), path);
 
     m_refreshedItems.clear();
-    connect(&m_dirLister, SIGNAL(refreshItems(const QList<QPair<KFileItem, KFileItem> > &)),
-            this, SLOT(slotRefreshItems(const QList<QPair<KFileItem, KFileItem> > &)));
+    connect(&m_dirLister, SIGNAL(refreshItems(QList<QPair<KFileItem,KFileItem> >)),
+            this, SLOT(slotRefreshItems(QList<QPair<KFileItem,KFileItem> >)));
     const QString newPath = dirPath+"toplevelfile_2.renamed.html";
 
     KIO::SimpleJob* job = KIO::rename(newPath, path, KIO::Overwrite | KIO::HideProgressInfo);
@@ -536,13 +538,13 @@ void KDirListerTest::testRenameAndOverwrite() // has to be run after testRenameI
 
     QCOMPARE(m_refreshedItems.count(), 1);
     QPair<KFileItem, KFileItem> entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), newPath);
-    QCOMPARE(entry.second.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), newPath);
+    QCOMPARE(entry.second.url().toLocalFile(), path);
     disconnect(&m_dirLister, 0, this, 0);
 
     // Let's see what KDirLister has in cache now
     KFileItem cachedItem = m_dirLister.findByUrl(KUrl(path));
-    QCOMPARE(cachedItem.url().path(), path);
+    QCOMPARE(cachedItem.url().toLocalFile(), path);
     KFileItem oldCachedItem = m_dirLister.findByUrl(KUrl(newPath));
     QVERIFY(oldCachedItem.isNull());
     m_refreshedItems.clear();
@@ -613,7 +615,9 @@ void KDirListerTest::testConcurrentListing()
     QCOMPARE(dirLister2.spyClear.count(), 1);
     QCOMPARE(dirLister2.spyClearKUrl.count(), 0);
     QCOMPARE(m_items2.count(), origItemCount);
-    //QVERIFY(m_dirLister.isFinished()); // false when an update is running because subdir is already in cache
+    if (!m_dirLister.isFinished()) { // false when an update is running because subdir is already in cache
+      QTest::kWaitForSignal(&m_dirLister, SIGNAL(canceled()), 1000);
+    }
 
     disconnect(&m_dirLister, 0, this, 0);
     disconnect(&dirLister2, 0, this, 0);
@@ -674,12 +678,104 @@ void KDirListerTest::testConcurrentHoldingListing()
     QCOMPARE(m_dirLister.spyClear.count(), 1);
     QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
     QVERIFY(dirLister2.isFinished());
-    disconnect(&dirLister2, 0, this, 0);
     QVERIFY(m_dirLister.isFinished());
     disconnect(&m_dirLister, 0, this, 0);
     QCOMPARE(m_items.count(), origItemCount);
 }
 
+void KDirListerTest::testConcurrentListingAndStop()
+{
+    m_items.clear();
+    m_items2.clear();
+
+    MyDirLister dirLister2;
+
+    // Use a new tempdir for this test, so that we don't use the cache at all.
+    KTempDir tempDir;
+    const QString path = tempDir.name();
+    createTestFile(path+"file_1");
+    createTestFile(path+"file_2");
+    createTestFile(path+"file_3");
+
+    connect(&m_dirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
+    connect(&dirLister2, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems2(KFileItemList)));
+
+    // Before m_dirLister has time to emit the items, let's make dirLister2 call stop().
+    // This should not stop the list job for m_dirLister (#267709).
+    dirLister2.openUrl(KUrl(path), KDirLister::Reload);
+    m_dirLister.openUrl(KUrl(path)/*, KDirLister::Reload*/);
+
+    QCOMPARE(m_dirLister.spyStarted.count(), 1);
+    QCOMPARE(m_dirLister.spyCompleted.count(), 0);
+    QCOMPARE(m_dirLister.spyCompletedKUrl.count(), 0);
+    QCOMPARE(m_dirLister.spyCanceled.count(), 0);
+    QCOMPARE(m_dirLister.spyCanceledKUrl.count(), 0);
+    QCOMPARE(m_dirLister.spyClear.count(), 1);
+    QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
+    QCOMPARE(m_items.count(), 0);
+
+    QCOMPARE(dirLister2.spyStarted.count(), 1);
+    QCOMPARE(dirLister2.spyCompleted.count(), 0);
+    QCOMPARE(dirLister2.spyCompletedKUrl.count(), 0);
+    QCOMPARE(dirLister2.spyCanceled.count(), 0);
+    QCOMPARE(dirLister2.spyCanceledKUrl.count(), 0);
+    QCOMPARE(dirLister2.spyClear.count(), 1);
+    QCOMPARE(dirLister2.spyClearKUrl.count(), 0);
+    QCOMPARE(m_items2.count(), 0);
+    QVERIFY(!m_dirLister.isFinished());
+    QVERIFY(!dirLister2.isFinished());
+
+    dirLister2.stop();
+
+    QCOMPARE(dirLister2.spyStarted.count(), 1);
+    QCOMPARE(dirLister2.spyCompleted.count(), 0);
+    QCOMPARE(dirLister2.spyCompletedKUrl.count(), 0);
+    QCOMPARE(dirLister2.spyCanceled.count(), 1);
+    QCOMPARE(dirLister2.spyCanceledKUrl.count(), 1);
+    QCOMPARE(dirLister2.spyClear.count(), 1);
+    QCOMPARE(dirLister2.spyClearKUrl.count(), 0);
+    QCOMPARE(m_items2.count(), 0);
+
+    // then wait for completed
+    qDebug("waiting for completed");
+    connect(&m_dirLister, SIGNAL(completed()), this, SLOT(exitLoop()));
+    enterLoop();
+
+    QCOMPARE(m_items.count(), 3);
+    QCOMPARE(m_items2.count(), 0);
+
+    //QCOMPARE(m_dirLister.spyStarted.count(), 1); // 2 when in cache
+    QCOMPARE(m_dirLister.spyCompleted.count(), 1);
+    QCOMPARE(m_dirLister.spyCompletedKUrl.count(), 1);
+    QCOMPARE(m_dirLister.spyCanceled.count(), 0);
+    QCOMPARE(m_dirLister.spyCanceledKUrl.count(), 0);
+    QCOMPARE(m_dirLister.spyClear.count(), 1);
+    QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
+
+    disconnect(&m_dirLister, 0, this, 0);
+}
+
+void KDirListerTest::testDeleteListerEarly()
+{
+    // Do the same again, it should behave the same, even with the items in the cache
+    testOpenUrl();
+
+    // Start a second lister, it will get a cached items job, but delete it before the job can run
+    //kDebug() << "==========================================";
+    {
+        m_items.clear();
+        const QString path = m_tempDir.name();
+        MyDirLister secondDirLister;
+        secondDirLister.openUrl(KUrl(path), KDirLister::NoFlags);
+        QVERIFY(!secondDirLister.isFinished());
+    }
+    //kDebug() << "==========================================";
+
+    // Check if we didn't keep the deleted dirlister in one of our lists.
+    // I guess the best way to do that is to just list the same dir again.
+    testOpenUrl();
+}
+
 void KDirListerTest::testOpenUrlTwice()
 {
     // Calling openUrl(reload)+openUrl(normal) before listing even starts.
@@ -707,7 +803,9 @@ void KDirListerTest::testOpenUrlTwice()
     QCOMPARE(secondDirLister.spyCanceledKUrl.count(), 0);
     QCOMPARE(secondDirLister.spyClear.count(), 2);
     QCOMPARE(secondDirLister.spyClearKUrl.count(), 0);
-    QCOMPARE(m_items.count(), origItemCount);
+    if (origItemCount) { // 0 if running this test separately
+      QCOMPARE(m_items.count(), origItemCount);
+    }
     QVERIFY(secondDirLister.isFinished());
     disconnect(&secondDirLister, 0, this, 0);
 }
@@ -728,6 +826,7 @@ void KDirListerTest::testOpenUrlTwiceWithKeep()
 
     secondDirLister.openUrl(KUrl(path)); // will start a list job
     QCOMPARE(secondDirLister.spyStarted.count(), 1);
+    QCOMPARE(secondDirLister.spyCanceled.count(), 0);
     QCOMPARE(secondDirLister.spyCompleted.count(), 0);
 
     qDebug("calling openUrl again");
@@ -793,7 +892,7 @@ void KDirListerTest::testRedirection()
 
     // then wait for the redirection signal
     qDebug("waiting for redirection");
-    connect(&m_dirLister, SIGNAL(redirection(KUrl, KUrl)), this, SLOT(exitLoop()));
+    connect(&m_dirLister, SIGNAL(redirection(KUrl,KUrl)), this, SLOT(exitLoop()));
     enterLoop();
     QCOMPARE(m_dirLister.spyStarted.count(), 1);
     QCOMPARE(m_dirLister.spyCompleted.count(), 0); // we stopped before the listing.
@@ -896,8 +995,8 @@ void KDirListerTest::fillDirLister2(MyDirLister& lister, const QString& path)
 {
     m_items2.clear();
     connect(&lister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems2(KFileItemList)));
-    connect(&lister, SIGNAL(refreshItems(QList<QPair<KFileItem, KFileItem> >)),
-            this, SLOT(slotRefreshItems2(QList<QPair<KFileItem, KFileItem> >)));
+    connect(&lister, SIGNAL(refreshItems(QList<QPair<KFileItem,KFileItem> >)),
+            this, SLOT(slotRefreshItems2(QList<QPair<KFileItem,KFileItem> >)));
     lister.openUrl(KUrl(path), KDirLister::NoFlags);
     connect(&lister, SIGNAL(completed()), this, SLOT(exitLoop()));
     enterLoop();
