@@ -96,6 +96,7 @@ void KDirModelTest::recreateTestData()
      * PATH/.hidden2
      * PATH/subdir
      * PATH/subdir/testfile
+     * PATH/subdir/testsymlink
      * PATH/subdir/subsubdir
      * PATH/subdir/subsubdir/testfile
      */
@@ -302,25 +303,25 @@ void KDirModelTest::testItemForIndex()
     QVERIFY(!fileItem.isNull());
     QCOMPARE(fileItem.name(), QString("toplevelfile_1"));
     QVERIFY(!fileItem.isDir());
-    QCOMPARE(fileItem.url().path(), m_tempDir->name() + "toplevelfile_1");
+    QCOMPARE(fileItem.url().toLocalFile(), QString(m_tempDir->name() + "toplevelfile_1"));
 
     KFileItem dirItem = m_dirModel->itemForIndex(m_dirIndex);
     QVERIFY(!dirItem.isNull());
     QCOMPARE(dirItem.name(), QString("subdir"));
     QVERIFY(dirItem.isDir());
-    QCOMPARE(dirItem.url().path(), m_tempDir->name() + "subdir");
+    QCOMPARE(dirItem.url().toLocalFile(), QString(m_tempDir->name() + "subdir"));
 
     KFileItem fileInDirItem = m_dirModel->itemForIndex(m_fileInDirIndex);
     QVERIFY(!fileInDirItem.isNull());
     QCOMPARE(fileInDirItem.name(), QString("testfile"));
     QVERIFY(!fileInDirItem.isDir());
-    QCOMPARE(fileInDirItem.url().path(), m_tempDir->name() + "subdir/testfile");
+    QCOMPARE(fileInDirItem.url().toLocalFile(), QString(m_tempDir->name() + "subdir/testfile"));
 
     KFileItem fileInSubdirItem = m_dirModel->itemForIndex(m_fileInSubdirIndex);
     QVERIFY(!fileInSubdirItem.isNull());
     QCOMPARE(fileInSubdirItem.name(), QString("testfile"));
     QVERIFY(!fileInSubdirItem.isDir());
-    QCOMPARE(fileInSubdirItem.url().path(), m_tempDir->name() + "subdir/subsubdir/testfile");
+    QCOMPARE(fileInSubdirItem.url().toLocalFile(), QString(m_tempDir->name() + "subdir/subsubdir/testfile"));
 }
 
 void KDirModelTest::testIndexForItem()
@@ -395,7 +396,7 @@ void KDirModelTest::testModifyFile()
     const KUrl url(file);
 
 #if 1
-    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
+    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
 #else
     ModelSpy modelSpy(m_dirModel);
 #endif
@@ -434,7 +435,7 @@ void KDirModelTest::testRenameFile()
     const KUrl url(m_tempDir->name() + "toplevelfile_2");
     const KUrl newUrl(m_tempDir->name() + "toplevelfile_2_renamed");
 
-    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
+    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
     connect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
              &m_eventLoop, SLOT(exitLoop()) );
 
@@ -499,8 +500,8 @@ void KDirModelTest::testMoveDirectory(const QString& dir /*just a dir name, no s
     disconnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
                &m_eventLoop, SLOT(exitLoop()));
 
-    QVERIFY(!m_dirModel->indexForUrl(path + "subdir").isValid());
-    QVERIFY(!m_dirModel->indexForUrl(path + "subdir_renamed").isValid());
+    QVERIFY(!m_dirModel->indexForUrl(KUrl(path + "subdir")).isValid());
+    QVERIFY(!m_dirModel->indexForUrl(KUrl(path + "subdir_renamed")).isValid());
 
     connect(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
             &m_eventLoop, SLOT(exitLoop()));
@@ -537,7 +538,7 @@ void KDirModelTest::testRenameDirectory() // #172945, #174703, (and #180156)
     enterLoop();
 
     // Now do the renaming
-    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
+    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
     connect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
              &m_eventLoop, SLOT(exitLoop()) );
     KIO::SimpleJob* job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
@@ -555,13 +556,13 @@ void KDirModelTest::testRenameDirectory() // #172945, #174703, (and #180156)
     // check renaming happened
     QCOMPARE(m_dirModel->itemForIndex(m_dirIndex).url().url(), newUrl.url());
     QCOMPARE(m_dirModel->indexForUrl(newUrl), m_dirIndex);
-    QVERIFY(m_dirModel->indexForUrl(path + "subdir_renamed").isValid());
-    QVERIFY(m_dirModel->indexForUrl(path + "subdir_renamed/testfile").isValid());
-    QVERIFY(m_dirModel->indexForUrl(path + "subdir_renamed/subsubdir").isValid());
-    QVERIFY(m_dirModel->indexForUrl(path + "subdir_renamed/subsubdir/testfile").isValid());
+    QVERIFY(m_dirModel->indexForUrl(KUrl(path + "subdir_renamed")).isValid());
+    QVERIFY(m_dirModel->indexForUrl(KUrl(path + "subdir_renamed/testfile")).isValid());
+    QVERIFY(m_dirModel->indexForUrl(KUrl(path + "subdir_renamed/subsubdir")).isValid());
+    QVERIFY(m_dirModel->indexForUrl(KUrl(path + "subdir_renamed/subsubdir/testfile")).isValid());
 
     // Check the other kdirmodel got redirected
-    QCOMPARE(dirListerForExpand->url().path(), path+"subdir_renamed");
+    QCOMPARE(dirListerForExpand->url().toLocalFile(), QString(path+"subdir_renamed"));
 
     kDebug() << "calling testMoveDirectory(subdir_renamed)";
 
@@ -581,17 +582,17 @@ void KDirModelTest::testRenameDirectory() // #172945, #174703, (and #180156)
 
     QCOMPARE(m_dirModel->itemForIndex(m_dirIndex).url().url(), url.url());
     QCOMPARE(m_dirModel->indexForUrl(url), m_dirIndex);
-    QVERIFY(m_dirModel->indexForUrl(path + "subdir").isValid());
-    QVERIFY(m_dirModel->indexForUrl(path + "subdir/testfile").isValid());
-    QVERIFY(m_dirModel->indexForUrl(path + "subdir/subsubdir").isValid());
-    QVERIFY(m_dirModel->indexForUrl(path + "subdir/subsubdir/testfile").isValid());
-    QVERIFY(!m_dirModel->indexForUrl(path + "subdir_renamed").isValid());
-    QVERIFY(!m_dirModel->indexForUrl(path + "subdir_renamed/testfile").isValid());
-    QVERIFY(!m_dirModel->indexForUrl(path + "subdir_renamed/subsubdir").isValid());
-    QVERIFY(!m_dirModel->indexForUrl(path + "subdir_renamed/subsubdir/testfile").isValid());
+    QVERIFY(m_dirModel->indexForUrl(KUrl(path + "subdir")).isValid());
+    QVERIFY(m_dirModel->indexForUrl(KUrl(path + "subdir/testfile")).isValid());
+    QVERIFY(m_dirModel->indexForUrl(KUrl(path + "subdir/subsubdir")).isValid());
+    QVERIFY(m_dirModel->indexForUrl(KUrl(path + "subdir/subsubdir/testfile")).isValid());
+    QVERIFY(!m_dirModel->indexForUrl(KUrl(path + "subdir_renamed")).isValid());
+    QVERIFY(!m_dirModel->indexForUrl(KUrl(path + "subdir_renamed/testfile")).isValid());
+    QVERIFY(!m_dirModel->indexForUrl(KUrl(path + "subdir_renamed/subsubdir")).isValid());
+    QVERIFY(!m_dirModel->indexForUrl(KUrl(path + "subdir_renamed/subsubdir/testfile")).isValid());
 
     // TODO INVESTIGATE
-    // QCOMPARE(dirListerForExpand->url().path(), path+"subdir");
+    // QCOMPARE(dirListerForExpand->url().toLocalFile(), path+"subdir");
 
     delete m_dirModelForExpand;
     m_dirModelForExpand = 0;
@@ -612,7 +613,7 @@ void KDirModelTest::testRenameDirectoryInCache() // #188807
     const KUrl url(path);
     KUrl newUrl(path);
     newUrl.adjustPath(KUrl::RemoveTrailingSlash);
-    newUrl.setPath(newUrl.path() + "_renamed");
+    newUrl.setPath(newUrl.toLocalFile() + "_renamed");
     kDebug() << newUrl;
     KIO::SimpleJob* job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
     QVERIFY(job->exec());
@@ -635,7 +636,7 @@ void KDirModelTest::testRenameDirectoryInCache() // #188807
 
 void KDirModelTest::testChmodDirectory() // #53397
 {
-    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
+    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
     connect( m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
              &m_eventLoop, SLOT(exitLoop()) );
     const QString path = m_tempDir->name();
@@ -669,8 +670,9 @@ enum {
     NoFlag = 0,
     NewDir = 1, // whether to re-create a new KTempDir completely, to avoid cached fileitems
     ListFinalDir = 2, // whether to list the target dir at the same time, like k3b, for #193364
-    Recreate = 4
-    // flags, next item is 8!
+    Recreate = 4,
+    CacheSubdir = 8, // put subdir in the cache before expandToUrl
+    // flags, next item is 16!
 };
 
 void KDirModelTest::testExpandToUrl_data()
@@ -712,7 +714,12 @@ void KDirModelTest::testExpandToUrl_data()
         << int(NewDir) << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
 
     QTest::newRow("hold dest dir") // #193364
-        << int(NewDir|ListFinalDir|Recreate) << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
+        << int(NewDir|ListFinalDir) << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
+
+    // Put subdir in cache too (#175035)
+    QTest::newRow("hold subdir and dest dir")
+      << int(NewDir|CacheSubdir|ListFinalDir|Recreate) << subsubdirfile
+                           << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
 
     // Make sure the last test has the Recreate option set, for the subsequent test methods.
 }
@@ -730,9 +737,14 @@ void KDirModelTest::testExpandToUrl()
     }
 
     const QString path = m_tempDir->name();
+    if (flags & CacheSubdir) {
+        // This way, the listDir for subdir will find items in cache, and will schedule a CachedItemsJob
+        m_dirModel->dirLister()->openUrl(KUrl(path + "subdir"));
+        QTest::kWaitForSignal(m_dirModel->dirLister(), SIGNAL(completed()), 2000);
+    }
     if (flags & ListFinalDir) {
         // This way, the last listDir will find items in cache, and will schedule a CachedItemsJob
-        m_dirModel->dirLister()->openUrl(path + "subdir/subsubdir");
+        m_dirModel->dirLister()->openUrl(KUrl(path + "subdir/subsubdir"));
         QTest::kWaitForSignal(m_dirModel->dirLister(), SIGNAL(completed()), 2000);
     }
 
@@ -790,8 +802,8 @@ void KDirModelTest::slotExpand(const QModelIndex& index)
     const QString path = m_tempDir->name();
     KFileItem item = m_dirModelForExpand->itemForIndex(index);
     QVERIFY(!item.isNull());
-    kDebug() << item.url().path();
-    QCOMPARE(item.url().path(), path + m_expectedExpandSignals[m_nextExpectedExpandSignals++]);
+    kDebug() << item.url().toLocalFile();
+    QCOMPARE(item.url().toLocalFile(), QString(path + m_expectedExpandSignals[m_nextExpectedExpandSignals++]));
 
     // if rowsInserted wasn't emitted yet, then any proxy model would be unable to do anything with index at this point
     if (item.url() == m_urlToExpandTo) {
@@ -837,7 +849,28 @@ void KDirModelTest::testFilter()
     QCOMPARE(m_dirModel->rowCount(), 4); // 3 toplevel* files, one subdir
     QCOMPARE(m_dirModel->rowCount(m_dirIndex), 1); // the files get filtered out, the subdir remains
 
-    QCOMPARE(spyRowsRemoved.count(), 3); // once for every dir
+    // In the subdir, we can get rowsRemoved signals like (1,2) or (0,0)+(2,2),
+    // depending on the order of the files in the model.
+    // So QCOMPARE(spyRowsRemoved.count(), 3) is fragile, we rather need
+    // to sum up the removed rows per parent directory.
+    QMap<QString, int> rowsRemovedPerDir;
+    for (int i = 0; i < spyRowsRemoved.count(); ++i) {
+        const QVariantList args = spyRowsRemoved[i];
+        const QModelIndex parentIdx = args[0].value<QModelIndex>();
+        QString dirName;
+        if (parentIdx.isValid()) {
+            const KFileItem item = m_dirModel->itemForIndex(parentIdx);
+            dirName = item.name();
+        } else {
+            dirName = "root";
+        }
+        rowsRemovedPerDir[dirName] += args[2].toInt() - args[1].toInt() + 1;
+        //kDebug() << parentIdx << args[1].toInt() << args[2].toInt();
+    }
+    QCOMPARE(rowsRemovedPerDir.count(), 3); // once for every dir
+    QCOMPARE(rowsRemovedPerDir.value("root"     ), 1); // one from toplevel ('specialchars')
+    QCOMPARE(rowsRemovedPerDir.value("subdir"   ), 2); // two from subdir
+    QCOMPARE(rowsRemovedPerDir.value("subsubdir"), 1); // one from subsubdir
     QCOMPARE(spyItemsDeleted.count(), 3); // once for every dir
     QCOMPARE(spyItemsDeleted[0][0].value<KFileItemList>().count(), 1); // one from toplevel ('specialchars')
     QCOMPARE(spyItemsDeleted[1][0].value<KFileItemList>().count(), 2); // two from subdir
@@ -905,7 +938,7 @@ void KDirModelTest::testShowHiddenFiles() // #174788
 {
     KDirLister* dirLister = m_dirModel->dirLister();
 
-    QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex, int, int)));
+    QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
     QSignalSpy spyNewItems(dirLister, SIGNAL(newItems(KFileItemList)));
     QSignalSpy spyRowsInserted(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)));
     dirLister->setShowingDotFiles(true);
@@ -932,10 +965,10 @@ void KDirModelTest::testMultipleSlashes()
     QModelIndex index = m_dirModel->indexForUrl(KUrl(path+"subdir//testfile"));
     QVERIFY(index.isValid());
 
-    index = m_dirModel->indexForUrl(path+"subdir//subsubdir//");
+    index = m_dirModel->indexForUrl(KUrl(path+"subdir//subsubdir//"));
     QVERIFY(index.isValid());
 
-    index = m_dirModel->indexForUrl(path+"subdir///subsubdir////testfile");
+    index = m_dirModel->indexForUrl(KUrl(path+"subdir///subsubdir////testfile"));
     QVERIFY(index.isValid());
 }
 
@@ -994,7 +1027,7 @@ void KDirModelTest::testZipFile() // # 171721
 
     KUrl zipUrl(path);
     zipUrl.addPath("wronglocalsizes.zip"); // just a zip file lying here for other reasons
-    QVERIFY(QFile::exists(zipUrl.path()));
+    QVERIFY(QFile::exists(zipUrl.toLocalFile()));
     zipUrl.setProtocol("zip");
     QModelIndex index = m_dirModel->indexForUrl(zipUrl);
     QVERIFY(!index.isValid()); // protocol mismatch, can't find it!
@@ -1033,6 +1066,27 @@ void KDirModelTest::testSmb()
     }
 }
 
+class MyDirLister : public KDirLister
+{
+public:
+    void emitItemsDeleted(const KFileItemList& items) { emit itemsDeleted(items); }
+};
+
+void KDirModelTest::testBug196695()
+{
+    KFileItem rootItem(KUrl(m_tempDir->name()), QString(), KFileItem::Unknown);
+    KFileItem childItem(KUrl(QString(m_tempDir->name() + "toplevelfile_1")), QString(), KFileItem::Unknown);
+
+    KFileItemList list;
+    // Important: the root item must not be first in the list to trigger bug 196695
+    list << childItem << rootItem;
+
+    MyDirLister* dirLister = static_cast<MyDirLister*>(m_dirModel->dirLister());
+    dirLister->emitItemsDeleted(list);
+
+    fillModel(true);
+}
+
 void KDirModelTest::testDeleteFile()
 {
     fillModel(false);
@@ -1062,7 +1116,7 @@ void KDirModelTest::testDeleteFile()
     disconnect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
                 &m_eventLoop, SLOT(exitLoop()) );
 
-    QModelIndex fileIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
+    QModelIndex fileIndex = m_dirModel->indexForUrl(KUrl(path + "toplevelfile_1"));
     QVERIFY(!fileIndex.isValid());
 
     // Recreate the file, for consistency in the next tests
@@ -1105,7 +1159,7 @@ void KDirModelTest::testDeleteFileWhileListing() // doesn't really test that yet
     QCOMPARE(spyRowsRemoved[0][1].toInt(), m_fileIndex.row());
     QCOMPARE(spyRowsRemoved[0][2].toInt(), m_fileIndex.row());
 
-    QModelIndex fileIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
+    QModelIndex fileIndex = m_dirModel->indexForUrl(KUrl(path + "toplevelfile_1"));
     QVERIFY(!fileIndex.isValid());
 
     kDebug() << "Test done, recreating file";
@@ -1148,7 +1202,7 @@ void KDirModelTest::testOverwriteFileWithDir() // #151851 c4
     QCOMPARE(topLevelRowCount, oldTopLevelRowCount - 1); // one less than before
 
     QVERIFY(!m_dirModel->indexForUrl(dir).isValid());
-    QModelIndex newIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
+    QModelIndex newIndex = m_dirModel->indexForUrl(KUrl(path + "toplevelfile_1"));
     QVERIFY(newIndex.isValid());
     KFileItem newItem = m_dirModel->itemForIndex(newIndex);
     QVERIFY(newItem.isDir()); // yes, the file is a dir now ;-)
@@ -1197,7 +1251,7 @@ void KDirModelTest::testRenameFileToHidden() // #174721
     const KUrl url(m_tempDir->name() + "toplevelfile_2");
     const KUrl newUrl(m_tempDir->name() + ".toplevelfile_2");
 
-    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
+    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
     QSignalSpy spyRowsInserted(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)));
     connect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
@@ -1259,9 +1313,9 @@ void KDirModelTest::testDeleteDirectory()
     disconnect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
                 &m_eventLoop, SLOT(exitLoop()) );
 
-    QModelIndex deletedDirIndex = m_dirModel->indexForUrl(path + "subdir/subsubdir");
+    QModelIndex deletedDirIndex = m_dirModel->indexForUrl(KUrl(path + "subdir/subsubdir"));
     QVERIFY(!deletedDirIndex.isValid());
-    QModelIndex dirIndex = m_dirModel->indexForUrl(path + "subdir");
+    QModelIndex dirIndex = m_dirModel->indexForUrl(KUrl(path + "subdir"));
     QVERIFY(dirIndex.isValid());
 
     // TODO!!! Bug in KDirWatch? ###
@@ -1310,7 +1364,7 @@ void KDirModelTest::testDeleteCurrentDirectory()
     disconnect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
                 &m_eventLoop, SLOT(exitLoop()) );
 
-    QModelIndex fileIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
+    QModelIndex fileIndex = m_dirModel->indexForUrl(KUrl(path + "toplevelfile_1"));
     QVERIFY(!fileIndex.isValid());
 }
 
