@@ -49,7 +49,6 @@
 #include <QtCore/QDir>
 #include <QtCore/QRegExp>
 #include <QtCore/QTextStream>
-#include <QtCore/QStringBuilder>
 
 // BR87227
 // Waba: Should the number of cookies be limited?
@@ -69,6 +68,37 @@
 #define QL1S(x)   QLatin1String(x)
 #define QL1C(x)   QLatin1Char(x)
 
+
+static KDateTime parseDate(const QString& value)
+{
+    KTimeZones *zones = KSystemTimeZones::timeZones();
+    // Check for the most common cookie expire date format: Thu, 01-Jan-1970 00:00:00 GMT
+    KDateTime dt = KDateTime::fromString(value, QL1S("%:A,%t%d-%:B-%Y%t%H:%M:%S%t%Z"), zones);
+    if (dt.isValid())
+        return dt;
+
+    // Check for incorrect formats (amazon.com): Thu Jan 01 1970 00:00:00 GMT
+    dt = KDateTime::fromString(value, QL1S("%:A%t%:B%t%d%t%Y%t%H:%M:%S%t%Z"), zones);
+    if (dt.isValid())
+        return dt;
+
+    // Check for a variation of the above format: Thu Jan 01 00:00:00 1970 GMT (BR# 145244)
+    dt = KDateTime::fromString(value, QL1S("%:A%t%:B%t%d%t%H:%M:%S%t%Y%t%Z"), zones);
+    if (dt.isValid())
+        return dt;
+
+    // Finally we try the RFC date formats as last resort
+    return KDateTime::fromString(value, KDateTime::RFCDate);
+}
+
+static qint64 epoch()
+{
+    KDateTime epoch;
+    epoch.setTime_t(0);
+    return epoch.secsTo_long(KDateTime::currentUtcDateTime());
+}
+
+
 QString KCookieJar::adviceToStr(KCookieAdvice _advice)
 {
     switch( _advice )
@@ -131,11 +161,8 @@ KHttpCookie::KHttpCookie(const QString &_host,
 //
 bool KHttpCookie::isExpired(qint64 currentDate) const
 {
-    if (currentDate == -1) {
-        KDateTime epoch;
-        epoch.setTime_t(0);
-        currentDate = epoch.secsTo_long(KDateTime::currentUtcDateTime());
-    }
+    if (currentDate == -1)
+        currentDate = epoch();
 
     return (mExpireDate != 0) && (mExpireDate < currentDate);
 }
@@ -151,21 +178,21 @@ QString KHttpCookie::cookieStr(bool useDOMFormat) const
         if ( mName.isEmpty() )
            result = mValue;
         else
-           result = mName % QL1C('=') % mValue;
+           result = mName + QL1C('=') + mValue;
     } else {
-        result = mName % QL1C('=') % mValue;
+        result = mName + QL1C('=') + mValue;
         if (mExplicitPath)
-            result += QL1S("; $Path=\"") % mPath % QL1C('"');
+            result += QL1S("; $Path=\"") + mPath + QL1C('"');
         if (!mDomain.isEmpty())
-            result += QL1S("; $Domain=\"") % mDomain % QL1C('"');
+            result += QL1S("; $Domain=\"") + mDomain + QL1C('"');
         if (!mPorts.isEmpty()) {
             if (mPorts.length() == 2 && mPorts.at(0) == -1)
                 result += QL1S("; $Port");
             else {
                 QString portNums;
                 Q_FOREACH(int port, mPorts)
-                    portNums += QString::number(port) % QL1C(' ');
-                result += QL1S("; $Port=\"") % portNums.trimmed() % QL1C('"');
+                    portNums += QString::number(port) + QL1C(' ');
+                result += QL1S("; $Port=\"") + portNums.trimmed() + QL1C('"');
             }
         }
     }
@@ -189,7 +216,7 @@ bool KHttpCookie::match(const QString &fqdn, const QStringList &domains,
             return false;
 
         // Maybe the domain needs an extra dot.
-        QString domain = QL1C('.') % mDomain;
+        const QString domain = QL1C('.') + mDomain;
         if ( !domains.contains( domain ) )
           if ( fqdn != mDomain )
             return false;
@@ -204,16 +231,15 @@ bool KHttpCookie::match(const QString &fqdn, const QStringList &domains,
     if (mPath.isEmpty())
         return true;
 
-    // According to the netscape spec both http://www.acme.com/foobar,
+    // According to the netscape spec http://www.acme.com/foobar,
     // http://www.acme.com/foo.bar and http://www.acme.com/foo/bar
-    // match http://www.acme.com/foo.
+    // should all match http://www.acme.com/foo...
     // We only match http://www.acme.com/foo/bar
-
     if( path.startsWith(mPath) &&
         (
          (path.length() == mPath.length() ) || 	// Paths are exact match
-          mPath.endsWith('/') || 	        // mPath ended with a slash
-         (path[mPath.length()] == '/')		// A slash follows
+          mPath.endsWith(QL1C('/')) || 	        // mPath ended with a slash
+         (path[mPath.length()] == QL1C('/'))		// A slash follows
          ))
         return true; // Path of URL starts with cookie-path
 
@@ -300,21 +326,12 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
     const bool secureRequest = (_url.startsWith(QL1S("https://"), Qt::CaseInsensitive) ||
                                 _url.startsWith(QL1S("webdavs://"), Qt::CaseInsensitive));
     if (port == -1)
-    {
-      if (secureRequest)
-        port = 443;
-      else
-        port = 80;
-    }
+        port = (secureRequest ? 443 : 80);
 
     extractDomains(fqdn, domains);
 
     KHttpCookieList allCookies;
-
-    QStringList::ConstIterator itEnd = domains.constEnd();
-    for(QStringList::ConstIterator it = domains.constBegin();
-        true;
-        ++it)
+    for (QStringList::ConstIterator it = domains.constBegin(), itEnd = domains.constEnd();;++it)
     {
        KHttpCookieList *cookieList = 0;
        if (it == itEnd)
@@ -339,79 +356,68 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
           advice = cookieList->getAdvice();
 
        QMutableListIterator<KHttpCookie> cookieIt (*cookieList);
-       while (cookieIt.hasNext()) {
+       while (cookieIt.hasNext()) 
+       {
           KHttpCookie& cookie = cookieIt.next();
           // If the we are setup to automatically accept all session cookies and to
           // treat all cookies as session cookies or the current cookie is a session
           // cookie, then send the cookie back regardless of domain policy.
           if (advice == KCookieReject &&
               !(m_autoAcceptSessionCookies &&
-                (m_ignoreCookieExpirationDate || cookie.expireDate() == 0))) {
+                (m_ignoreCookieExpirationDate || cookie.expireDate() == 0)))
               continue;
-          }
 
           if (!cookie.match(fqdn, domains, path, port))
              continue;
 
-          if( cookie.isSecure() && !secureRequest ) {
+          if( cookie.isSecure() && !secureRequest )
              continue;
-          }
 
-          if( cookie.isHttpOnly() && useDOMFormat ) {
+          if( cookie.isHttpOnly() && useDOMFormat )
              continue;
-          }
 
           // Do not send expired cookies.
           if ( cookie.isExpired())
           {
-             // Note there is no need to actually delete the cookie here
-             // since the cookieserver will invoke ::saveCookieJar because
-             // of the state change below. This will then do the job of
-             // deleting the cookie for us.
+             // NOTE: there is no need to delete the cookie here because the
+             // cookieserver will invoke its saveCookieJar function as a result
+             // of the state change below. This will then result in the cookie
+             // being deleting at that point.
              m_cookiesChanged = true;
              continue;
           }
 
           if (windowId && (cookie.windowIds().indexOf(windowId) == -1))
-          {
              cookie.windowIds().append(windowId);
-          }
 
           if (it == itEnd) // Only needed when processing pending cookies
              removeDuplicateFromList(&allCookies, cookie);
 
           allCookies.append(cookie);
        }
+
        if (it == itEnd)
           break; // Finished.
     }
 
-    int cookieCount = 0;
     int protVersion = 0;
     Q_FOREACH(const KHttpCookie& cookie, allCookies) {
         if (cookie.protocolVersion() > protVersion)
             protVersion = cookie.protocolVersion();
     }
 
-    Q_FOREACH(const KHttpCookie& cookie, allCookies) {
-        if (useDOMFormat) {
-            if (cookieCount == 0) {
-                if (protVersion > 0)
-                    cookieStr += QL1S("$Version=") % QString::number(protVersion) % QL1S("; ");
-            }
-            else
-                cookieStr += QL1S("; ");
-            cookieStr += cookie.cookieStr(true);
-        } else {
-            if (cookieCount == 0) {
-                cookieStr += QL1S("Cookie: ");
-                if (protVersion > 0)
-                    cookieStr += QL1S("$Version=") % QString::number(protVersion) % QL1S("; ");
-            } else
-                cookieStr += QL1S("; ");
-            cookieStr += cookie.cookieStr(false);
-        }
-        cookieCount++;
+    if (!allCookies.isEmpty())
+    {
+        if (!useDOMFormat)
+            cookieStr = QL1S("Cookie: ");
+
+        if (protVersion > 0)
+            cookieStr = cookieStr + QL1S("$Version=") + QString::number(protVersion) + QL1S("; ");
+
+        Q_FOREACH(const KHttpCookie& cookie, allCookies)
+            cookieStr = cookieStr + cookie.cookieStr(useDOMFormat) + QL1S("; ");
+
+        cookieStr.truncate(cookieStr.length() - 2); // Remove the trailing ';'
     }
 
     return cookieStr;
@@ -531,7 +537,7 @@ QString KCookieJar::stripDomain(const KHttpCookie& cookie)
     if (cookie.domain().isEmpty())
        stripDomain( cookie.host(), domain);
     else
-       stripDomain( cookie.domain(), domain);
+       domain = cookie.domain();
     return domain;
 }
 
@@ -545,10 +551,10 @@ bool KCookieJar::parseUrl(const QString &_url,
        return false;
 
     _fqdn = kurl.host().toLower();
-    // Cookie spoofing protection.  Since there is no way a path separator
-    // or escape encoded character is allowed in the hostname according
-    // to RFC 2396, reject attempts to include such things there!
-    if(_fqdn.contains('/') || _fqdn.contains('%'))
+    // Cookie spoofing protection.  Since there is no way a path separator,
+    // a space or the escape encoding character is allowed in the hostname
+    // according to RFC 2396, reject attempts to include such things there!
+    if (_fqdn.contains(QL1C('/')) || _fqdn.contains(QL1C('%')))
         return false;  // deny everything!!
 
     // Set the port number from the protocol when one is found...
@@ -587,9 +593,9 @@ void KCookieJar::extractDomains(const QString &_fqdn,
     // Always add the FQDN at the start of the list for
     // hostname == cookie-domainname checks!
     _domains.append(_fqdn);
-    _domains.append(QL1C('.') % _fqdn);
+    _domains.append(QL1C('.') + _fqdn);
 
-    QStringList partList = _fqdn.split('.', QString::SkipEmptyParts);
+    QStringList partList = _fqdn.split(QL1C('.'), QString::SkipEmptyParts);
 
     if (partList.count())
         partList.erase(partList.begin()); // Remove hostname
@@ -621,7 +627,7 @@ void KCookieJar::extractDomains(const QString &_fqdn,
 
        QString domain = partList.join(QL1S("."));
        _domains.append(domain);
-       _domains.append(QL1C('.') % domain);
+       _domains.append(QL1C('.') + domain);
        partList.erase(partList.begin()); // Remove part
     }
 }
@@ -651,9 +657,9 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
     const char *cookieStr = cookie_headers.constData();    
 
     QString defaultPath;
-    const int i = path.lastIndexOf('/');
-    if (i > 0)
-       defaultPath = path.left(i);
+    const int index = path.lastIndexOf(QL1C('/'));
+    if (index > 0)
+       defaultPath = path.left(index);
 
     KDateTime epoch;
     epoch.setTime_t(0);
@@ -738,7 +744,7 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
                 if(dom.length() > 2 && dom[dom.length()-1] == '.')
                     dom = dom.left(dom.length()-1);
 
-                if(dom.count('.') > 1 || dom == ".local")
+                if(dom.count(QL1C('.')) > 1 || dom == ".local")
                     lastCookie.mDomain = dom;
             }
             else if (Name.compare(QL1S("max-age"), Qt::CaseInsensitive) == 0)
@@ -751,25 +757,7 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
             }
             else if (Name.compare(QL1S("expires"), Qt::CaseInsensitive) == 0)
             {
-                KTimeZones *zones = KSystemTimeZones::timeZones();
-                // Check for the most common cookie expire date format: Thu, 01-Jan-1970 00:00:00 GMT
-                KDateTime dt = KDateTime::fromString(Value, QL1S("%:A,%t%d-%:B-%Y%t%H:%M:%S%t%Z"), zones);
-                if (!dt.isValid()) {
-                    // Check for a variation of the above format: Thu, 01 Jan 1970 00:00:00 GMT
-                    dt = KDateTime::fromString(Value, QL1S("%:A,%t%d%t%:B%t%Y%t%H:%M:%S%t%Z"), zones);
-                    if (!dt.isValid()) {
-                        // Check for incorrect formats (amazon.com): Thu Jan 01 1970 00:00:00 GMT
-                        dt = KDateTime::fromString(Value, QL1S("%:A%t%:B%t%d%t%Y%t%H:%M:%S%t%Z"), zones);
-                        if (!dt.isValid()) {
-                            // Check for a variation of the above format: Thu Jan 01 00:00:00 1970 GMT (BR# 145244)
-                            dt = KDateTime::fromString(Value, QL1S("%:A%t%:B%t%d%t%H:%M:%S%t%Y%t%Z"), zones);
-                            if (!dt.isValid()) {
-                                // Finally we try the RFC date formats as last resort
-                                dt = KDateTime::fromString(Value, KDateTime::RFCDate);
-                            }
-                        }
-                    }
-                }
+                const KDateTime dt = parseDate(Value);
 
                 if (dt.isValid()) {
                     lastCookie.mExpireDate = epoch.secsTo_long(dt);
@@ -801,11 +789,11 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
             }
             else if (isRFC2965 && (Name.compare(QL1S("port"), Qt::CaseInsensitive) == 0 ||
                      (Name.isEmpty() && Value.compare(QL1S("port"), Qt::CaseInsensitive) == 0)))
-            {              
+            {
                 // Based on the port selection rule of RFC 2965 section 3.3.4...
                 if (Name.isEmpty())
                 {
-                    // We intentionally append a -1 first in orer to distinguish
+                    // We intentionally append a -1 first in order to distinguish
                     // between only a 'Port' vs a 'Port="80 443"' in the sent cookie.
                     lastCookie.mPorts.append(-1);
                     const bool secureRequest = (_url.startsWith(QL1S("https://"), Qt::CaseInsensitive) ||
@@ -931,10 +919,11 @@ void KCookieJar::addCookie(KHttpCookie &cookie)
     // that cookies of type hostname == cookie-domainname
     // are properly removed and/or updated as necessary!
     extractDomains( cookie.host(), domains );
+
     QStringListIterator it (domains);
     while (it.hasNext())
     {
-        const QString key = it.next();
+        const QString& key = it.next();
         KHttpCookieList* list;
 
         if (key.isNull())
@@ -1003,7 +992,7 @@ KCookieAdvice KCookieJar::cookieAdvice(KHttpCookie& cookie)
     // cross-site cookie injection.
     if (!cookie.domain().isEmpty()) {
       if (!domains.contains(cookie.domain()) &&
-          !cookie.domain().endsWith('.'+cookie.host()))
+          !cookie.domain().endsWith(QL1C('.') + cookie.host()))
          cookie.fixDomain(QString());
     }
 
@@ -1013,10 +1002,9 @@ KCookieAdvice KCookieJar::cookieAdvice(KHttpCookie& cookie)
 
     KCookieAdvice advice = KCookieDunno;
     QStringListIterator it (domains);
-
     while(advice == KCookieDunno && it.hasNext()) {
-       const QString domain = it.next();
-       if (domain.startsWith('.') || cookie.host() == domain) {
+       const QString& domain = it.next();
+       if (domain.startsWith(QL1C('.')) || cookie.host() == domain) {
               KHttpCookieList *cookieList = m_cookieDomains.value(domain);
           if (cookieList)
              advice = cookieList->getAdvice();
@@ -1122,7 +1110,7 @@ KHttpCookieList *KCookieJar::getCookieList(const QString & _domain,
     if (_domain.isEmpty())
         stripDomain(_fqdn, domain);
     else
-        stripDomain (_domain, domain);
+        domain = _domain;
 
     return m_cookieDomains.value(domain);
 }
@@ -1134,7 +1122,7 @@ KHttpCookieList *KCookieJar::getCookieList(const QString & _domain,
 void KCookieJar::eatCookie(KHttpCookieList::iterator cookieIterator)
 {
     const KHttpCookie& cookie = *cookieIterator;
-    QString domain = stripDomain(cookie); // We file the cookie under this domain.
+    const QString domain = stripDomain(cookie); // We file the cookie under this domain.
     KHttpCookieList *cookieList = m_cookieDomains.value(domain);
 
     if (cookieList) {
@@ -1146,7 +1134,6 @@ void KCookieJar::eatCookie(KHttpCookieList::iterator cookieIterator)
         {
             // This deletes cookieList!
             delete m_cookieDomains.take(domain);
-
             m_domainList.removeAll(domain);
         }
     }
@@ -1172,17 +1159,14 @@ void KCookieJar::eatSessionCookies( long windowId )
     if (!windowId)
         return;
 
-    QStringList::const_iterator it=m_domainList.constBegin();
-    for ( ; it != m_domainList.constEnd(); ++it )
-        eatSessionCookies( *it, windowId, false );
+    Q_FOREACH(const QString& domain, m_domainList)
+        eatSessionCookies( domain, windowId, false );
 }
 
 void KCookieJar::eatAllCookies()
 {
-    Q_FOREACH(const QString& domain, m_domainList) {
-        // This might remove domain from m_domainList!
-        eatCookiesForDomain(domain);
-    }
+    Q_FOREACH(const QString& domain, m_domainList)
+        eatCookiesForDomain(domain);    // This might remove domain from m_domainList!
 }
 
 void KCookieJar::eatSessionCookies( const QString& fqdn, long windowId,
@@ -1210,9 +1194,9 @@ void KCookieJar::eatSessionCookies( const QString& fqdn, long windowId,
 #ifndef NDEBUG
             if (ids.contains(windowId)) {
                 if (ids.count() > 1)
-                    kDebug() << "removing window id" << windowId << "from session cookie";
+                    kDebug(7104) << "removing window id" << windowId << "from session cookie";
                 else
-                    kDebug() << "deleting session cookie";
+                    kDebug(7104) << "deleting session cookie";
             }
 #endif
             if (!ids.removeAll(windowId) || !ids.isEmpty()) {
@@ -1234,7 +1218,7 @@ static QString hostWithPort(const KHttpCookie* cookie)
     Q_FOREACH(int port, ports)
         portList << QString::number(port);
 
-    return (cookie->host() % QL1C(':') % portList.join(QL1S(",")));
+    return (cookie->host() + QL1C(':') + portList.join(QL1S(",")));
 }
 
 //
@@ -1262,25 +1246,30 @@ bool KCookieJar::saveCookies(const QString &_filename)
     QStringListIterator it(m_domainList);
     while (it.hasNext())
     {
-        const QString &domain = it.next();
+        const QString& domain = it.next();
         bool domainPrinted = false;
 
         KHttpCookieList *cookieList = m_cookieDomains.value(domain);
+        if (!cookieList)
+            continue;
+
         QMutableListIterator<KHttpCookie> cookieIterator(*cookieList);
         while (cookieIterator.hasNext()) {
             const KHttpCookie& cookie = cookieIterator.next();
             if (cookie.isExpired()) {
                 // Delete expired cookies
                 cookieIterator.remove();
-            } else if (cookie.expireDate() != 0 && !m_ignoreCookieExpirationDate) {
+                continue;
+            }
+            if (cookie.expireDate() != 0 && !m_ignoreCookieExpirationDate) {
                 // Only save cookies that are not "session-only cookies"
                 if (!domainPrinted) {
                     domainPrinted = true;
                     ts << '[' << domain.toLocal8Bit().data() << "]\n";
                 }
                 // Store persistent cookies
-                const QString path = QL1S("\"") % cookie.path() % QL1C('"');
-                const QString domain = QL1S("\"") % cookie.domain() % QL1C('"');
+                const QString path = QL1S("\"") + cookie.path() + QL1C('"');
+                const QString domain = QL1S("\"") + cookie.domain() + QL1C('"');
                 const QString host = hostWithPort(&cookie);
 
                 // TODO: replace with direct QTextStream output ?
@@ -1390,9 +1379,7 @@ bool KCookieJar::loadCookies(const QString &_filename)
 
     if (success)
     {
-        KDateTime epoch;
-        epoch.setTime_t(0); // epoch
-        const qint64 currentTime = epoch.secsTo_long(KDateTime::currentUtcDateTime());
+        const qint64 currentTime = epoch();
         QList<int> ports;
 
         while(cookieFile.readLine(buffer, READ_BUFFER_SIZE-1) != -1)
@@ -1479,11 +1466,11 @@ void KCookieJar::saveConfig(KConfig *_config)
     QStringListIterator it (m_domainList);    
     while (it.hasNext())
     {
-         const QString &domain = it.next();
+         const QString& domain = it.next();
          KCookieAdvice advice = getDomainAdvice( domain);
          if (advice != KCookieDunno)
          {
-             const QString value = domain % QL1C(':') % adviceToStr(advice);
+             const QString value = domain + QL1C(':') + adviceToStr(advice);
              domainSettings.append(value);
          }
     }
@@ -1515,25 +1502,19 @@ void KCookieJar::loadConfig(KConfig *_config, bool reparse )
     m_globalAdvice = strToAdvice(policyGroup.readEntry("CookieGlobalAdvice", QString(QL1S("Accept"))));
 
     // Reset current domain settings first.
-    //  (must make a copy because setDomainAdvice() might delete the domain from m_domainList inside the for loop)
-    const QStringList domains = m_domainList;
-    Q_FOREACH( const QString &domain, domains )
-    {
-         setDomainAdvice(domain, KCookieDunno);
-    }
+    Q_FOREACH( const QString &domain, m_domainList )
+        setDomainAdvice(domain, KCookieDunno);
 
     // Now apply the domain settings read from config file...
-    for ( QStringList::const_iterator it=domainSettings.begin();
-          it != domainSettings.end(); )
+    for (QStringList::ConstIterator it = domainSettings.constBegin(), itEnd = domainSettings.constEnd();
+         it != itEnd; ++it)
     {
-        const QString &value = *it++;
-
-        int sepPos = value.lastIndexOf(':');
-
+        const QString& value = *it;
+        const int sepPos = value.lastIndexOf(QL1C(':'));
         if (sepPos <= 0)
           continue;
 
-        QString domain(value.left(sepPos));
+        const QString domain(value.left(sepPos));
         KCookieAdvice advice = strToAdvice( value.mid(sepPos + 1) );
         setDomainAdvice(domain, advice);
     }
