@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
     Copyright (C) 2008 Andreas Hartmetz <ahartmetz@gmail.com>
+    Copyright (C) 2010,2011 Rolf Eike Beer <kde@opensource.sf-tec.de>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,11 +18,15 @@
     Boston, MA 02110-1301, USA.
 */
 
+#include <ctype.h>
+
+#include <QDir>
 #include <QMap>
 #include <QTextCodec>
 #include <QUrl>
 
 #include <kcodecs.h>
+#include <kdebug.h>
 
 // Advance *pos beyond spaces / tabs
 static void skipSpace(const char input[], int *pos, int end)
@@ -61,24 +66,26 @@ static bool nextLine(const char input[], int *pos, int end)
     return idx < end && rCount < 2 && nCount < 2;
 }
 
-//Return true if the term was found, false otherwise. Advance *pos.
-//If (*pos + strlen(term) >= end) just advance *pos to end and return false.
-//This means that users should always search for the shortest terms first.
-static bool consume(const char input[], int *pos, int end, const char *term)
+// QByteArray::fromPercentEncoding() does not notify us about encoding errors so we need
+// to check here if this is valid at all.
+static bool isValidPercentEncoding(const QByteArray &data)
 {
-    // note: gcc/g++ is quite good at optimizing away redundant strlen()s
-    int idx = *pos;
-    if (idx + (int)strlen(term) >= end) {
-        *pos = end;
-        return false;
-    }
-    if (strncasecmp(&input[idx], term, strlen(term)) == 0) {
-        *pos = idx + strlen(term);
-        return true;
-    }
-    return false;
-}
+    int i = 0;
+    const int last = data.length() - 1;
+    const char *d = data.constData();
 
+    while ( (i = data.indexOf('%', i)) != -1) {
+        if ( i >= last - 2 )
+            return false;
+        if ( ! isxdigit(d[i + 1]) )
+            return false;
+        if ( ! isxdigit(d[i + 2]) )
+            return false;
+        i++;
+    }
+
+    return true;
+}
 
 QByteArray TokenIterator::next()
 {
@@ -109,7 +116,7 @@ QList<QByteArray> TokenIterator::all() const
 
 
 HeaderTokenizer::HeaderTokenizer(char *buffer)
- : m_buffer(buffer)
+    : m_buffer(buffer)
 {
     // add information about available headers and whether they have one or multiple,
     // comma-separated values.
@@ -133,7 +140,7 @@ HeaderTokenizer::HeaderTokenizer(char *buffer)
         {"dav", true}, //RFC 2518
         {"etag", false},
         {"expires", false},
-        {"keep-alive", false}, //RFC 2068
+        {"keep-alive", true}, //RFC 2068
         {"last-modified", false},
         {"link", false}, //RFC 2068, multi-valued with ";" separator
         {"location", false},
@@ -281,23 +288,32 @@ TokenIterator HeaderTokenizer::iterator(const char *key) const
 
 static void skipLWS(const QString &str, int &pos)
 {
-    while (pos < str.length() && (str[pos] == QLatin1Char(' ') || str[pos] == QLatin1Char('\t')))
+    while (pos < str.length() && (str[pos] == QLatin1Char(' ') || str[pos] == QLatin1Char('\t'))) {
         ++pos;
+    }
 }
 
 // keep the common ending, this allows the compiler to join them
-static const char typeSpecials[] =  "*'%()<>@,;:\\\"/[]?=";
-static const char attrSpecials[] =   "'%()<>@,;:\\\"/[]?=";
-static const char valueSpecials[] =    "()<>@,;:\\\"/[]?=";
+static const char typeSpecials[] =  "{}*'%()<>@,;:\\\"/[]?=";
+static const char attrSpecials[] =     "'%()<>@,;:\\\"/[]?=";
+static const char valueSpecials[] =      "()<>@,;:\\\"/[]?=";
 
 static bool specialChar(const QChar &ch, const char *specials)
 {
-    if( (ch.unicode() < 32) || (ch.unicode() >= 128) )
+    // WORKAROUND: According to RFC 2616, any character other than ascii
+    // characters should NOT be allowed in unquoted content-disposition file
+    // names. However, since none of the major browsers follow this rule, we do
+    // the same thing here and allow all printable unicode characters. See
+    // https://bugs.kde.org/show_bug.cgi?id=261223 for the detials.
+    if (!ch.isPrint()) {
         return true;
+    }
 
-    for( int i = strlen(specials) - 1; i>= 0; i--)
-       if( ch == QLatin1Char(specials[i]) )
-           return true;
+    for (int i = qstrlen(specials) - 1; i >= 0; i--) {
+        if (ch == QLatin1Char(specials[i])) {
+            return true;
+        }
+    }
 
     return false;
 }
@@ -325,29 +341,36 @@ static QString extractUntil(const QString &str, QChar term, int &pos, const char
 
     while (pos < str.length() && (str[pos] != term)) {
         out += str[pos];
-        valid &= !specialChar(str[pos], specials);
+        valid = (valid && !specialChar(str[pos], specials));
         ++pos;
     }
 
-    if (pos < str.length()) // Stopped due to finding term
+    if (pos < str.length()) { // Stopped due to finding term
         ++pos;
+    }
 
-    if( !valid )
+    if (!valid) {
         return QString();
+    }
 
     // Remove trailing linear whitespace...
-    while (out.endsWith(QLatin1Char(' ')) || out.endsWith(QLatin1Char('\t')))
+    while (out.endsWith(QLatin1Char(' ')) || out.endsWith(QLatin1Char('\t'))) {
         out.chop(1);
+    }
 
-    if( out.contains(QLatin1Char(' ')) )
+    if (out.contains(QLatin1Char(' '))) {
         out.clear();
+    }
 
     return out;
 }
 
 // As above, but also handles quotes..
-static QString extractMaybeQuotedUntil(const QString &str, QChar term, int &pos)
+// pos is set to -1 on parse error
+static QString extractMaybeQuotedUntil(const QString &str, int &pos)
 {
+    const QChar term = QLatin1Char(';');
+
     skipLWS(str, pos);
 
     // Are we quoted?
@@ -357,6 +380,9 @@ static QString extractMaybeQuotedUntil(const QString &str, QChar term, int &pos)
         // Skip the quote...
         ++pos;
 
+        // when quoted we also need an end-quote
+        bool endquote = false;
+
         // Parse until trailing quote...
         while (pos < str.length()) {
             if (str[pos] == QLatin1Char('\\') && pos + 1 < str.length()) {
@@ -365,19 +391,33 @@ static QString extractMaybeQuotedUntil(const QString &str, QChar term, int &pos)
                 pos += 2; // Skip both...
             } else if (str[pos] == QLatin1Char('"')) {
                 ++pos;
+                endquote = true;
                 break;
-            }  else {
+            } else if (!str[pos].isPrint()) { // Don't allow CTL's RFC 2616 sec 2.2
+                break;
+            } else {
                 out += str[pos];
                 ++pos;
             }
         }
 
-        // Skip until term..
-        while (pos < str.length() && (str[pos] != term))
-            ++pos;
+        if (!endquote) {
+            pos = -1;
+            return QString();
+        }
 
-        if (pos < str.length()) // Stopped due to finding term
+        // Skip until term..
+        while (pos < str.length() && (str[pos] != term)) {
+            if ((str[pos] != QLatin1Char(' ')) && (str[pos] != QLatin1Char('\t'))) {
+                pos = -1;
+                return QString();
+            }
             ++pos;
+        }
+
+        if (pos < str.length()) {  // Stopped due to finding term
+            ++pos;
+        }
 
         return out;
     } else {
@@ -385,7 +425,7 @@ static QString extractMaybeQuotedUntil(const QString &str, QChar term, int &pos)
     }
 }
 
-static QMap<QString, QString> contentDispositionParser(const QString &disposition)
+static QMap<QString, QString> contentDispositionParserInternal(const QString &disposition)
 {
     kDebug(7113) << "disposition: " << disposition;
     int pos = 0;
@@ -395,53 +435,69 @@ static QMap<QString, QString> contentDispositionParser(const QString &dispositio
     QMap<QString, QString> contparams;   // all parameters that contain continuations
     QMap<QString, QString> encparams;    // all parameters that have character encoding
 
-    if( !strDisposition.isEmpty() )
-        parameters.insert(QLatin1String("type"), strDisposition);
+    // the type is invalid, the complete header is junk
+    if (strDisposition.isEmpty()) {
+        return parameters;
+    }
+
+    parameters.insert(QLatin1String("type"), strDisposition);
 
     while (pos < disposition.length()) {
         QString key = extractUntil(disposition, QLatin1Char('='), pos, attrSpecials).toLower();
 
-        if( key.isEmpty() ) {
-            extractMaybeQuotedUntil(disposition, QLatin1Char(';'), pos);
+        if (key.isEmpty()) {
+            // parse error in this key: do not parse more, but add up
+            // everything we already got
+            kDebug(7113) << "parse error in key, abort parsing";
+            break;
+        }
+
+        QString val;
+        if (key.endsWith(QLatin1Char('*'))) {
+            val = extractUntil(disposition, QLatin1Char(';'), pos, valueSpecials);
+        } else {
+            val = extractMaybeQuotedUntil(disposition, pos);
+        }
+
+        if (val.isEmpty()) {
+            if (pos == -1) {
+                kDebug(7113) << "parse error in value, abort parsing";
+                break;
+            }
             continue;
         }
 
-        int spos = key.indexOf(QLatin1Char('*'));
-        QString val;
-        if( key.endsWith(QLatin1Char('*')) )
-            val = extractUntil(disposition, QLatin1Char(';'), pos, valueSpecials).toLower();
-        else
-            val = extractMaybeQuotedUntil(disposition, QLatin1Char(';'), pos);
-
-        if( val.isEmpty() )
-            continue;
-
-        if( spos == key.length() - 1 ) {
+        const int spos = key.indexOf(QLatin1Char('*'));
+        if (spos == key.length() - 1) {
             key.chop(1);
             encparams.insert(key, val);
-        } else if( spos >= 0 ) {
+        } else if (spos >= 0) {
             contparams.insert(key, val);
+        } else if (parameters.contains(key)) {
+            kDebug(7113) << "duplicate key" << key << "found, ignoring everything more";
+            parameters.remove(key);
+            return parameters;
         } else {
             parameters.insert(key, val);
         }
     }
 
     QMap<QString, QString>::iterator i = contparams.begin();
-    while( i != contparams.end() ) {
+    while (i != contparams.end()) {
         QString key = i.key();
         int spos = key.indexOf(QLatin1Char('*'));
         bool hasencoding = false;
 
-        if( key.at(spos + 1) != QLatin1Char('0') ) {
+        if (key.at(spos + 1) != QLatin1Char('0')) {
             ++i;
             continue;
         }
 
         // no leading zeros allowed, so delete the junk
         int klen = key.length();
-        if( klen > spos + 2 ) {
+        if (klen > spos + 2) {
             // nothing but continuations and encodings may insert * into parameter name
-            if( (klen > spos + 3) || ((klen == spos + 3) && (key.at(spos + 2) != QLatin1Char('*'))) ) {
+            if ((klen > spos + 3) || ((klen == spos + 3) && (key.at(spos + 2) != QLatin1Char('*')))) {
                 kDebug(7113) << "removing invalid key " << key << "with val" << i.value() << key.at(spos + 2);
                 i = contparams.erase(i);
                 continue;
@@ -455,76 +511,87 @@ static QMap<QString, QString> contentDispositionParser(const QString &dispositio
         // part is allowed to have one
         QString val = i.value();
 
-        if (hasencoding)
-            key.chop(2);
-        else
-            key.chop(1);
+        key.chop(hasencoding ? 2 : 1);
 
-        while( (partsi = contparams.find(key + QString::number(seqnum))) != contparams.end() )
-        {
+        while ((partsi = contparams.find(key + QString::number(seqnum))) != contparams.end()) {
             val += partsi.value();
             contparams.erase(partsi);
         }
 
         i = contparams.erase(i);
 
-        if( parameters.contains(key) )
-            continue;
-
         key.chop(1);
         if (hasencoding) {
             encparams.insert(key, val);
         } else {
+            if (parameters.contains(key)) {
+                kDebug(7113) << "duplicate key" << key << "found, ignoring everything more";
+                parameters.remove(key);
+                return parameters;
+            }
+
             parameters.insert(key, val);
         }
     }
 
-    for( QMap<QString, QString>::iterator i = encparams.begin();
-         i != encparams.end(); ++i )
-    {
+    for (QMap<QString, QString>::iterator i = encparams.begin(); i != encparams.end(); ++i) {
         QString val = i.value();
 
         // RfC 2231 encoded character set in filename
         int spos = val.indexOf(QLatin1Char('\''));
-        if (spos == -1)
+        if (spos == -1) {
             continue;
+        }
         int npos = val.indexOf(QLatin1Char('\''), spos + 1);
-        if (npos == -1)
+        if (npos == -1) {
             continue;
-
-        const QString charset = val.left( spos );
-        const QString lang = val.mid( spos + 1, npos - spos - 1 );
-        const QByteArray rawval = QByteArray::fromPercentEncoding( val.mid(npos + 1).toAscii() );
-        if( charset.isEmpty() || (charset == QLatin1String("us-ascii")) ) {
-            bool valid = true;
-            for( int j = rawval.length() - 1; (j >= 0) && valid; j-- )
-                valid = (rawval.at(j) >= 32);
-
-            if( valid )
-                val = QString::fromAscii(rawval.constData());
-            else
-                val.clear();
-        } else {
-            QTextCodec *codec = QTextCodec::codecForName( charset.toAscii() );
-            if( codec )
-                val = codec->toUnicode(rawval);
-            else
-                val.clear();
         }
 
-        if( !val.isEmpty() )
-            parameters.insert( i.key(), val );
-    }
+        const QString charset = val.left(spos);
+        const QString lang = val.mid(spos + 1, npos - spos - 1);
+        const QByteArray encodedVal = val.mid(npos + 1).toAscii();
 
-    const QLatin1String fn("filename");
-    if( parameters.contains(fn) ) {
-        // Content-Disposition is not allowed to dictate directory
-        // path, thus we extract the filename only.
-        QString val = parameters[fn];
-        int slpos = val.lastIndexOf( QLatin1Char('/') );
+        if ( ! isValidPercentEncoding(encodedVal) )
+            continue;
 
-        if( slpos > -1 )
-            parameters.insert(fn, val.mid( slpos + 1 ));
+        const QByteArray rawval = QByteArray::fromPercentEncoding(encodedVal);
+
+        if (charset.isEmpty() || (charset == QLatin1String("us-ascii"))) {
+            bool valid = true;
+            for (int j = rawval.length() - 1; (j >= 0) && valid; j--) {
+                valid = (rawval.at(j) >= 32);
+            }
+
+            if (!valid)
+                continue;
+            val = QString::fromAscii(rawval.constData());
+        } else {
+            QTextCodec *codec = QTextCodec::codecForName(charset.toAscii());
+            if (!codec)
+                continue;
+            val = codec->toUnicode(rawval);
+        }
+
+        parameters.insert(i.key(), val);
+    }
+
+    return parameters;
+}
+
+static QMap<QString, QString> contentDispositionParser(const QString &disposition)
+{
+    QMap<QString, QString> parameters = contentDispositionParserInternal(disposition);
+
+    const QLatin1String fn("filename");
+    if (parameters.contains(fn)) {
+        // Content-Disposition is not allowed to dictate directory
+        // path, thus we extract the filename only.
+        const QString val = QDir::toNativeSeparators(parameters[fn]);
+        int slpos = val.lastIndexOf(QDir::separator());
+
+        if (slpos > -1) {
+            parameters.insert(fn, val.mid(slpos + 1));
+        }
     }
 
     return parameters;
