@@ -1,6 +1,6 @@
 /*
  * This file is part of the Nepomuk KDE project.
- * Copyright (C) 2008-2010 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2008-2012 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,8 +26,6 @@
 #include <Soprano/StatementIterator>
 #include <Soprano/NodeIterator>
 #include <Soprano/QueryResultIterator>
-#include <Soprano/Client/DBusModel>
-#include <Soprano/Client/DBusClient>
 #include <Soprano/Client/LocalSocketClient>
 #include <Soprano/Query/QueryLanguage>
 #include <Soprano/Util/DummyModel>
@@ -42,8 +40,6 @@
 #include <QtCore/QTimer>
 #include <QtCore/QMutex>
 #include <QtCore/QMutexLocker>
-#include <QtDBus/QDBusConnection>
-#include <QtDBus/QDBusConnectionInterface>
 
 
 // FIXME: disconnect localSocketClient after n seconds of idling (but take care of not
@@ -57,23 +53,18 @@ class GlobalModelContainer
 {
 public:
     GlobalModelContainer()
-        : dbusClient( "org.kde.nepomuk.services.nepomukstorage" ),
-          dbusModel( 0 ),
-          localSocketModel( 0 ),
+        : localSocketModel( 0 ),
           dummyModel( 0 ),
           m_socketConnectFailed( false ),
           m_initMutex( QMutex::Recursive ) {
     }
 
     ~GlobalModelContainer() {
-        delete dbusModel;
         delete localSocketModel;
         delete dummyModel;
     }
 
-    Soprano::Client::DBusClient dbusClient;
     Soprano::Client::LocalSocketClient localSocketClient;
-    Soprano::Client::DBusModel* dbusModel;
     Soprano::Model* localSocketModel;
 
     Soprano::Util::DummyModel* dummyModel;
@@ -83,13 +74,6 @@ public:
 
         if( forced ) {
             m_socketConnectFailed = false;
-            delete dbusModel;
-            dbusModel = 0;
-        }
-
-        // TODO: check if the service is also initialized
-        if ( !dbusModel ) {
-            dbusModel = dbusClient.createModel( "main" );
         }
 
         // we may get disconnected from the server but we don't want to try
@@ -97,7 +81,7 @@ public:
         if ( !m_socketConnectFailed && !localSocketClient.isConnected() ) {
             delete localSocketModel;
             localSocketModel = 0;
-            QString socketName = KGlobal::dirs()->locateLocal( "data", "nepomuk/socket" );
+            QString socketName = KGlobal::dirs()->locateLocal( "socket", "nepomuk-socket" );
             kDebug() << "Connecting to local socket" << socketName;
             if ( localSocketClient.connect( socketName ) ) {
                 localSocketModel = localSocketClient.createModel( "main" );
@@ -118,9 +102,6 @@ public:
         if ( localSocketModel ) {
             return localSocketModel;
         }
-        else if ( dbusModel ) {
-            return dbusModel;
-        }
         else {
             if ( !dummyModel ) {
                 dummyModel = new Soprano::Util::DummyModel();
@@ -167,24 +148,13 @@ Nepomuk::MainModel::~MainModel()
 
 bool Nepomuk::MainModel::isValid() const
 {
-    return s_modelContainer->dbusClient.isValid() || s_modelContainer->localSocketClient.isConnected();
+    return s_modelContainer->localSocketClient.isConnected();
 }
 
 
 bool Nepomuk::MainModel::init()
 {
     s_modelContainer->init( true );
-    if ( s_modelContainer->dbusModel ) {
-        // we have to use the dbus model for signals in any case
-        connect( s_modelContainer->dbusModel, SIGNAL( statementsAdded() ),
-                 this, SIGNAL( statementsAdded() ) );
-        connect( s_modelContainer->dbusModel, SIGNAL( statementsRemoved() ),
-                 this, SIGNAL( statementsRemoved() ) );
-        connect( s_modelContainer->dbusModel, SIGNAL( statementAdded(const Soprano::Statement&) ),
-                 this, SIGNAL( statementAdded(const Soprano::Statement&) ) );
-        connect( s_modelContainer->dbusModel, SIGNAL( statementRemoved(const Soprano::Statement&) ),
-                 this, SIGNAL( statementRemoved(const Soprano::Statement&) ) );
-    }
     return isValid();
 }
 
