@@ -27,27 +27,28 @@
 
 #include <QAction>
 #include <QApplication>
-#include <QPainter>
 #include <QGraphicsSceneMouseEvent>
 #include <QGraphicsView>
 #include <QMenu>
+#include <QPainter>
 #include <QStyleOptionGraphicsItem>
 #include <QTextLayout>
+#include <QTimer>
 
+#include <kcolorscheme.h>
+#include <kdebug.h>
 #include <kglobalsettings.h>
+#include <kicon.h>
 #include <kiconeffect.h>
 #include <kiconloader.h>
-#include <kicon.h>
-#include <kurl.h>
-#include <krun.h>
 #include <kmimetype.h>
-#include <kdebug.h>
-#include <kcolorscheme.h>
+#include <kurl.h>
 
-#include <plasma/animator.h>
-#include <plasma/animations/animation.h>
-#include <plasma/paintutils.h>
-#include <plasma/theme.h>
+#include "animator.h"
+#include "animations/animation.h"
+#include "paintutils.h"
+#include "private/themedwidgetinterface_p.h"
+#include "theme.h"
 
 #include "svg.h"
 
@@ -100,7 +101,6 @@ void IconHoverAnimation::setAnimation(QPropertyAnimation *animation)
 
 IconWidgetPrivate::IconWidgetPrivate(IconWidget *i)
     : ActionWidgetInterface<IconWidget>(i),
-      q(i),
       iconSvg(0),
       hoverAnimation(new IconHoverAnimation(q)),
       iconSize(48, 48),
@@ -114,8 +114,7 @@ IconWidgetPrivate::IconWidgetPrivate(IconWidget *i)
       iconSvgElementChanged(false),
       invertLayout(false),
       drawBg(false),
-      textBgCustomized(false),
-      customFont(false)
+      textBgCustomized(false)
 {
 }
 
@@ -159,15 +158,6 @@ void IconWidgetPrivate::iconConfigChanged()
     }
 }
 
-QFont IconWidgetPrivate::widgetFont() const
-{
-    if (customFont) {
-        return q->font();
-    } else {
-        return Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-    }
-}
-
 IconAction::IconAction(IconWidget *icon, QAction *action)
     : m_icon(icon),
       m_action(action),
@@ -419,6 +409,7 @@ void IconWidgetPrivate::init()
 
     setActiveMargins();
     currentSize = QSizeF(-1, -1);
+    initTheming();
 }
 
 void IconWidget::addIconAction(QAction *action)
@@ -548,7 +539,7 @@ QSizeF IconWidgetPrivate::displaySizeHint(const QStyleOptionGraphicsItem *option
 
     //allow only five lines of text
     const qreal maxHeight =
-        numDisplayLines * QFontMetrics(widgetFont()).lineSpacing();
+        numDisplayLines * QFontMetrics(q->font()).lineSpacing();
 
     // To compute the nominal size for the label + info, we'll just append
     // the information string to the label
@@ -558,8 +549,8 @@ QSizeF IconWidgetPrivate::displaySizeHint(const QStyleOptionGraphicsItem *option
 
     QTextLayout layout;
     setLayoutOptions(layout, option, q->orientation());
-    layout.setFont(widgetFont());
-    QSizeF size = layoutText(layout, option, label, QSizeF(textWidth, maxHeight));
+    layout.setFont(q->font());
+    QSizeF size = layoutText(layout, label, QSizeF(textWidth, maxHeight));
 
     return addMargin(size, TextMargin);
 }
@@ -571,6 +562,17 @@ void IconWidgetPrivate::layoutIcons(const QStyleOptionGraphicsItem *option)
     }
 
     currentSize = option->rect.size();
+    iconSize = iconSizeForWidgetSize(option, currentSize);
+
+    int count = 0;
+    foreach (IconAction *iconAction, cornerActions) {
+        iconAction->setRect(actionRect((IconWidgetPrivate::ActionPosition)count));
+        ++count;
+    }
+}
+
+QSizeF IconWidgetPrivate::iconSizeForWidgetSize(const QStyleOptionGraphicsItem *option, const QSizeF &rect)
+{
     setActiveMargins();
 
     //calculate icon size based on the available space
@@ -580,20 +582,20 @@ void IconWidgetPrivate::layoutIcons(const QStyleOptionGraphicsItem *option)
         qreal heightAvail;
         //if there is text resize the icon in order to make room for the text
         if (text.isEmpty() && infoText.isEmpty()) {
-            heightAvail = currentSize.height();
+            heightAvail = rect.height();
         } else {
-            heightAvail = currentSize.height() -
-                          displaySizeHint(option, currentSize.width()).height() -
+            heightAvail = rect.height() -
+                          displaySizeHint(option, rect.width()).height() -
                           verticalMargin[IconWidgetPrivate::TextMargin].top -
                           verticalMargin[IconWidgetPrivate::TextMargin].bottom;
             //never make a label higher than half the total height
-            heightAvail = qMax(heightAvail, currentSize.height() / 2);
+            heightAvail = qMax(heightAvail, rect.height() / 2);
         }
 
         //aspect ratio very "tall"
         if (!text.isEmpty() || !infoText.isEmpty()) {
-            if (currentSize.width() < heightAvail) {
-                iconWidth = currentSize.width() -
+            if (rect.width() < heightAvail) {
+                iconWidth = rect.width() -
                             verticalMargin[IconWidgetPrivate::IconMargin].left -
                             verticalMargin[IconWidgetPrivate::IconMargin].right;
             } else {
@@ -602,7 +604,7 @@ void IconWidgetPrivate::layoutIcons(const QStyleOptionGraphicsItem *option)
                             verticalMargin[IconWidgetPrivate::IconMargin].bottom;
             }
         } else {
-            iconWidth = qMin(heightAvail, currentSize.width());
+            iconWidth = qMin(heightAvail, rect.width());
         }
 
         iconWidth -= verticalMargin[IconWidgetPrivate::ItemMargin].left + verticalMargin[IconWidgetPrivate::ItemMargin].right;
@@ -611,25 +613,22 @@ void IconWidgetPrivate::layoutIcons(const QStyleOptionGraphicsItem *option)
         //if there is text resize the icon in order to make room for the text
         if (text.isEmpty() && infoText.isEmpty()) {
             // with no text, we just take up the whole geometry
-            iconWidth = qMin(currentSize.height(), currentSize.width());
+            iconWidth = qMin(rect.height(), rect.width());
         } else {
-            iconWidth = currentSize.height() -
+            iconWidth = rect.height() -
                         horizontalMargin[IconWidgetPrivate::IconMargin].top -
                         horizontalMargin[IconWidgetPrivate::IconMargin].bottom;
         }
         iconWidth -= horizontalMargin[IconWidgetPrivate::ItemMargin].top + horizontalMargin[IconWidgetPrivate::ItemMargin].bottom;
     }
-    iconSize = QSizeF(iconWidth, iconWidth);
+
+    QSizeF iconRect(iconWidth, iconWidth);
 
     if (maximumIconSize.isValid()) {
-        iconSize = iconSize.boundedTo(maximumIconSize);
+        iconRect = iconRect.boundedTo(maximumIconSize);
     }
 
-    int count = 0;
-    foreach (IconAction *iconAction, cornerActions) {
-        iconAction->setRect(actionRect((IconWidgetPrivate::ActionPosition)count));
-        ++count;
-    }
+    return iconRect;
 }
 
 void IconWidget::setSvg(const QString &svgFilePath, const QString &elementId)
@@ -645,15 +644,25 @@ void IconWidget::setSvg(const QString &svgFilePath, const QString &elementId)
     if (!d->iconSvg) {
         d->iconSvg = new Plasma::Svg(this);
         connect(d->iconSvg, SIGNAL(repaintNeeded()), this, SLOT(svgChanged()));
+        d->oldIcon = d->icon;
+    } else {
+        d->oldIcon = d->iconSvg->pixmap(d->iconSvgElement);
     }
 
     d->iconSvg->setImagePath(svgFilePath);
     d->iconSvg->setContainsMultipleImages(!elementId.isNull());
     d->iconSvgElement = elementId;
     d->iconSvgElementChanged = true;
-    d->icon = QIcon();
     updateGeometry();
-    update();
+
+    if (!(d->states & IconWidgetPrivate::HoverState) && !d->iconChangeTimer->isActive() && !d->oldIcon.isNull()) {
+        d->animateMainIcon(true, d->states);
+    } else {
+        d->oldIcon = QIcon();
+        update();
+    }
+    d->iconChangeTimer->start(300);
+    d->icon = QIcon();
 }
 
 QString IconWidget::svg() const
@@ -672,11 +681,11 @@ QString IconWidget::svg() const
 QSizeF IconWidget::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
 {
     if (which == Qt::PreferredSize) {
+        int iconSize;
+
         if (d->preferredIconSize.isValid()) {
-            return sizeFromIconSize(qMax(d->preferredIconSize.height(), d->preferredIconSize.width()));
-        }
-        int iconSize = KIconLoader::SizeMedium;
-        if (d->iconSvg) {
+            iconSize = qMax(d->preferredIconSize.height(), d->preferredIconSize.width());
+        } else if (d->iconSvg) {
             QSizeF oldSize = d->iconSvg->size();
             d->iconSvg->resize();
             if (d->iconSvgElement.isNull()) {
@@ -685,17 +694,48 @@ QSizeF IconWidget::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
                 iconSize = qMax(d->iconSvg->elementSize(d->iconSvgElement).width(), d->iconSvg->elementSize(d->iconSvgElement).height());
             }
             d->iconSvg->resize(oldSize);
+        } else {
+            iconSize = KIconLoader::SizeMedium;
         }
+
+        if (constraint.width() > 0 || constraint.height() > 0) {
+            QSizeF constrainedWidgetSize(constraint);
+            QSizeF maximumWidgetSize;
+
+            if (d->maximumIconSize.isValid()) {
+                maximumWidgetSize =
+                    sizeFromIconSize(qMax(d->maximumIconSize.height(), d->maximumIconSize.width()));
+            } else {
+                maximumWidgetSize =
+                    QGraphicsWidget::sizeHint(Qt::MaximumSize);
+            }
+
+            if (constrainedWidgetSize.width() <= 0) {
+                constrainedWidgetSize.setWidth(maximumWidgetSize.width());
+            }
+            if (constrainedWidgetSize.height() <= 0) {
+                constrainedWidgetSize.setHeight(maximumWidgetSize.height());
+            }
+
+            QStyleOptionGraphicsItem option;
+            QSizeF iconRect =
+                d->iconSizeForWidgetSize(&option, constrainedWidgetSize);
+            iconSize =
+                qMin(iconSize, qMax<int>(iconRect.width(), iconRect.height()));
+        }
+
         return sizeFromIconSize(iconSize);
     } else if (which == Qt::MinimumSize) {
         if (d->minimumIconSize.isValid()) {
             return sizeFromIconSize(qMax(d->minimumIconSize.height(), d->minimumIconSize.width()));
         }
+
         return sizeFromIconSize(KIconLoader::SizeSmall);
     } else {
         if (d->maximumIconSize.isValid()) {
             return sizeFromIconSize(qMax(d->maximumIconSize.height(), d->maximumIconSize.width()));
         }
+
         return QGraphicsWidget::sizeHint(which, constraint);
     }
 }
@@ -711,20 +751,19 @@ void IconWidgetPrivate::animateMainIcon(bool show, const IconWidgetStates state)
     QPropertyAnimation *animation = hoverAnimation->animation();
     if (!animation) {
         animation = new QPropertyAnimation(hoverAnimation, "value");
-        animation->setProperty("duration", 150);
-        animation->setProperty("easingCurve", QEasingCurve::OutQuad);
-        animation->setProperty("startValue", 0.0);
-        animation->setProperty("endValue", 1.0);
+        animation->setDuration(150);
+        animation->setEasingCurve(QEasingCurve::OutQuad);
+        animation->setStartValue(0.0);
+        animation->setEndValue(1.0);
         hoverAnimation->setAnimation(animation);
         q->connect(animation, SIGNAL(finished()), q, SLOT(hoverAnimationFinished()));
     } else if (animation->state() == QAbstractAnimation::Running) {
         animation->pause();
     }
 
-    animation->setProperty("direction", show ?
-            QAbstractAnimation::Forward : QAbstractAnimation::Backward);
-    animation->start(show ?
-            QAbstractAnimation::KeepWhenStopped : QAbstractAnimation::DeleteWhenStopped);
+    animation->setDirection(show ? QAbstractAnimation::Forward : QAbstractAnimation::Backward);
+    animation->start(show ? QAbstractAnimation::KeepWhenStopped : QAbstractAnimation::DeleteWhenStopped);
+    q->update();
 }
 
 void IconWidgetPrivate::hoverAnimationFinished()
@@ -784,7 +823,7 @@ QPixmap IconWidgetPrivate::decoration(const QStyleOptionGraphicsItem *option, bo
     }
 
     if (usePressedEffect) {
-        result = result.scaled(result.size() * 0.9, Qt::KeepAspectRatio);
+        result = result.scaled(result.size() * 0.9, Qt::KeepAspectRatio, Qt::SmoothTransformation);
     }
 
     if (!result.isNull() && useHoverEffect) {
@@ -823,7 +862,6 @@ QPointF IconWidgetPrivate::iconPosition(const QStyleOptionGraphicsItem *option,
     // Compute the nominal decoration rectangle
     const QSizeF size = addMargin(iconSize, IconWidgetPrivate::IconMargin);
 
-
     Qt::LayoutDirection direction = iconDirection(option);
 
     //alignment depends from orientation and option->direction
@@ -877,8 +915,9 @@ QRectF IconWidgetPrivate::labelRectangle(const QStyleOptionGraphicsItem *option,
 }
 
 // Lays the text out in a rectangle no larger than constraints, eliding it as necessary
-QSizeF IconWidgetPrivate::layoutText(QTextLayout &layout, const QStyleOptionGraphicsItem *option,
-                                     const QString &text, const QSizeF &constraints) const
+QSizeF IconWidgetPrivate::layoutText(QTextLayout &layout,
+                                     const QString &text,
+                                     const QSizeF &constraints) const
 {
     const QSizeF size = layoutText(layout, text, constraints.width());
 
@@ -886,7 +925,7 @@ QSizeF IconWidgetPrivate::layoutText(QTextLayout &layout, const QStyleOptionGrap
         if (action) {
             q->setToolTip(action->toolTip());
         }
-        const QString elided = elidedText(layout, option, constraints);
+        const QString elided = elidedText(layout, constraints);
         return layoutText(layout, elided, constraints.width());
     }
     q->setToolTip(QString());
@@ -923,11 +962,8 @@ QSizeF IconWidgetPrivate::layoutText(QTextLayout &layout, const QString &text, q
 // or word breaking the line if it's wider than the max width, and finally adding an
 // ellipses at the end of the last line, if there are more lines than will fit within
 // the vertical size constraints.
-QString IconWidgetPrivate::elidedText(QTextLayout &layout, const QStyleOptionGraphicsItem *option,
-                                      const QSizeF &size) const
+QString IconWidgetPrivate::elidedText(QTextLayout &layout, const QSizeF &size) const
 {
-    Q_UNUSED(option)
-
     QFontMetricsF metrics(layout.font());
     const QString text = layout.text();
     qreal maxWidth       = size.width();
@@ -998,12 +1034,12 @@ void IconWidgetPrivate::layoutTextItems(const QStyleOptionGraphicsItem *option,
     }
 
     // Lay out the label text, and adjust the max info size based on the label size
-    labelSize = layoutText(*labelLayout, option, text, maxLabelSize);
+    labelSize = layoutText(*labelLayout, text, maxLabelSize);
     maxInfoSize.rheight() -= labelSize.height();
 
     // Lay out the info text
     if (showInformation) {
-        infoSize = layoutText(*infoLayout, option, infoText, maxInfoSize);
+        infoSize = layoutText(*infoLayout, infoText, maxInfoSize);
     } else {
         infoSize = QSizeF(0, 0);
     }
@@ -1336,7 +1372,9 @@ void IconWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
         return;
     }
 
-    d->states |= IconWidgetPrivate::PressedState;
+    if (KGlobalSettings::singleClick() || (receivers(SIGNAL(clicked()))) > 0) {
+        d->states |= IconWidgetPrivate::PressedState;
+    }
     d->clickStartPos = scenePos();
 
     bool handled = false;
@@ -1415,10 +1453,14 @@ void IconWidget::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
 {
     Q_UNUSED(event)
 
+    d->states |= IconWidgetPrivate::PressedState;
+
     emit doubleClicked();
     if (!KGlobalSettings::singleClick()) {
         emit activated();
     }
+
+    update();
 }
 
 void IconWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
@@ -1431,7 +1473,6 @@ void IconWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
 
     d->oldIcon = QIcon();
     d->animateMainIcon(true, d->states|IconWidgetPrivate::HoverState);
-    update();
 
     QGraphicsWidget::hoverEnterEvent(event);
 }
@@ -1448,7 +1489,6 @@ void IconWidget::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
     d->states &= ~IconWidgetPrivate::PressedState;
 
     d->animateMainIcon(false, d->states|IconWidgetPrivate::HoverState);
-    update();
 
     QGraphicsWidget::hoverLeaveEvent(event);
 }
@@ -1459,10 +1499,8 @@ bool IconWidget::sceneEventFilter(QGraphicsItem *watched, QEvent *event)
 
     if (event->type() == QEvent::GraphicsSceneDragEnter) {
         d->animateMainIcon(true, d->states|IconWidgetPrivate::HoverState);
-        update();
     } else if (event->type() == QEvent::GraphicsSceneDragLeave) {
         d->animateMainIcon(false, d->states|IconWidgetPrivate::HoverState);
-        update();
     }
 
     return false;
@@ -1520,7 +1558,7 @@ QSizeF IconWidget::sizeFromIconSize(const qreal iconWidth) const
         return d->addMargin(QSizeF(iconWidth, iconWidth), IconWidgetPrivate::ItemMargin);
     }
 
-    QFontMetricsF fm(d->widgetFont());
+    QFontMetricsF fm(font());
     qreal width = 0;
 
     if (d->orientation == Qt::Vertical) {
@@ -1572,10 +1610,7 @@ QSizeF IconWidget::sizeFromIconSize(const qreal iconWidth) const
 
 void IconWidget::changeEvent(QEvent *event)
 {
-    if (event->type() == QEvent::FontChange) {
-        d->customFont = true;
-    }
-
+    d->changeEvent(event);
     QGraphicsWidget::changeEvent(event);
 }
 
