@@ -25,6 +25,7 @@
 #include <math.h>
 #include <string.h>
 
+#include <QApplication>
 #include <QList>
 #include <QPalette>
 #include <QtGui/QPainter>
@@ -35,8 +36,8 @@
 #include <kdebug.h>
 #include <kglobal.h>
 #include <klocale.h>
-#include <kapplication.h>
 #include <kstandarddirs.h>
+#include <kiconloader.h>
 
 #include <plasma/svg.h>
 #include <plasma/theme.h>
@@ -59,10 +60,11 @@ public:
     {
         Plasma::Theme *theme = Plasma::Theme::defaultTheme();
         backgroundColor = theme->color(Theme::BackgroundColor);
-        verticalLinesColor = theme->color(Theme::TextColor);
+        fontColor = theme->color(Theme::TextColor);
+        borderColor = fontColor;
+        verticalLinesColor = fontColor;
         verticalLinesColor.setAlphaF(0.4);
-        horizontalLinesColor = theme->color(Theme::TextColor);
-        horizontalLinesColor.setAlphaF(0.4);
+        horizontalLinesColor = verticalLinesColor;
     }
 
     int precision;
@@ -76,15 +78,6 @@ public:
     double niceVertMax;
     double niceVertRange;
 
-    bool fillPlots;
-    bool showLabels;
-    bool showTopBar;
-    bool stackPlots;
-    bool useAutoRange;
-    bool showThinFrame;
-
-    bool showVerticalLines;
-    bool verticalLinesScroll;
     uint verticalLinesOffset;
     uint verticalLinesDistance;
     QColor verticalLinesColor;
@@ -98,6 +91,7 @@ public:
     QString svgFilename;
 
     QColor fontColor;
+    QColor borderColor;
     QColor backgroundColor;
     QPixmap backgroundPixmap;
 
@@ -107,6 +101,16 @@ public:
 
     QList<PlotColor> plotColors;
     QList<QList<double> > plotData;
+
+    bool fillPlots : 1;
+    bool showLabels : 1;
+    bool showTopBar : 1;
+    bool stackPlots : 1;
+    bool useAutoRange : 1;
+    bool showThinFrame : 1;
+
+    bool showVerticalLines : 1;
+    bool verticalLinesScroll : 1;
 };
 
 SignalPlotter::SignalPlotter(QGraphicsItem *parent)
@@ -123,9 +127,6 @@ SignalPlotter::SignalPlotter(QGraphicsItem *parent)
     d->scaledBy = 1;
     d->showThinFrame = true;
 
-    // Anything smaller than this does not make sense.
-    setMinimumSize(QSizeF(16, 16));
-
     d->showVerticalLines = true;
     d->verticalLinesDistance = 30;
     d->verticalLinesScroll = true;
@@ -140,6 +141,9 @@ SignalPlotter::SignalPlotter(QGraphicsItem *parent)
     d->stackPlots = true;
     d->fillPlots = true;
 
+    // Anything smaller than this does not make sense.
+    setMinimumSize(QSizeF(KIconLoader::SizeSmall, KIconLoader::SizeSmall));
+
     setSvgBackground("widgets/plot-background");
 
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(themeChanged()));
@@ -503,10 +507,9 @@ void SignalPlotter::setSvgBackground(const QString &filename)
         d->svgFilename = filename;
     }
 
+    delete d->svgBackground;
+    d->svgBackground = 0;
     if (!d->svgFilename.isEmpty()) {
-        if (d->svgBackground) {
-            delete d->svgBackground;
-        }
         d->svgBackground = new Svg(this);
         d->svgBackground->setImagePath(d->svgFilename);
     }
@@ -536,6 +539,11 @@ void SignalPlotter::setThinFrame(bool set)
     d->backgroundPixmap = QPixmap(); // we changed a paint setting, so reset the cache
 }
 
+bool SignalPlotter::thinFrame() const
+{
+    return d->showThinFrame;
+}
+
 void SignalPlotter::setStackPlots(bool stack)
 {
     d->stackPlots = stack;
@@ -700,9 +708,9 @@ void SignalPlotter::drawBackground(QPainter *p, int w, int h)
 
 void SignalPlotter::drawThinFrame(QPainter *p, int w, int h)
 {
-    // Draw white line along the bottom and the right side of the
+    // Draw a line along the bottom and the right side of the
     // widget to create a 3D like look.
-    p->setPen(kapp->palette().color(QPalette::Light));
+    p->setPen(d->borderColor);
     p->drawLine(0, h - 1, w - 1, h - 1);
     p->drawLine(w - 1, 0, w - 1, h - 1);
 }
@@ -925,7 +933,7 @@ void SignalPlotter::drawPlots(QPainter *p, int top, int w, int h, int horizontal
                               qMax(prev_prev_datapoints[j],
                                    prev_prev_prev_datapoints[j])));
                 double current_minvalue =
-                    qMin(datapoints[j],
+                    qMin<double>(datapoints[j],
                          qMin(prev_datapoints[j],
                               qMin(prev_prev_datapoints[j],
                                    prev_prev_prev_datapoints[j])));
